-- COPYRIGHT NOTICE (NOT TO BE REMOVED):
--
-- This file, or parts of it, or modified versions of it, may not be
-- copied, reproduced or transmitted in any form, including
-- reprinting, translation, photocopying or microfilming, or by any
-- means, electronic, mechanical or otherwise, or stored in a
-- retrieval system, or used for any purpose, without the prior
-- written permission of all Owners unless it is explicitly marked as
-- having Classification `Public'.
--
-- Classification: Restricted.
--
-- Owners of this file give notice:
-- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
-- All rights, including copyrights, reserved.
--
-- This file contains or may contain restricted information and is
-- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
-- Notice(s) above do not evidence any actual or intended publication
-- of such source code. This file is additionally subject to the
-- conditions listed in the RESTRICTIONS file and is with NOWARRANTY.
--
-- END OF COPYRIGHT NOTICE
--


library IEEE;
use IEEE.std_logic_1164.all;
--USE IEEE.numeric_std.all;

--library nodepack;
package hw_node_pkg is
   type t_counter_step  is array (0 to 10) of natural range 1 to 32;
   type t_counter_width is array (0 to 10) of natural range 1 to 32; -- each number represents the bit-width of a counter
   --
   --type t_par_values is array (0 to 10) of integer; -- each number represents the default value of a parameter
   type t_par is record
      val_min  : integer;
      val_max  : integer;
      val_def  : integer;
      bitwidth : natural;
   end record;
   
   type t_par_vector is array (natural range<>) of t_par;      
        

   Function b2std(b : boolean) return std_logic;
   function int2slv(int_value : integer; size: integer) return std_logic_vector;
   function slv2int(vect : std_logic_vector; size : integer) return integer;
   --Function modulo2(a:integer; b:integer) return integer;
   Function maxf(left: integer; right: integer) return integer;
   Function minf(left: integer; right: integer) return integer;

end hw_node_pkg;

package body hw_node_pkg is

   Function b2std(b : boolean) return std_logic is
   begin
      if b then
         return '1';
      else
         return '0';
      end if;
   end b2std;

-----------------------------------------------------------------------------------

   Function int2slv(int_value : integer; size : integer) return std_logic_vector is
   variable result : std_logic_vector(size-1 downto 0);
   begin
      for i in 0 to size-1 loop
         if ((int_value/(2**i)) rem 2) = 0 then
            result(i) := '0';
         else
            result(i) := '1';
         end if;
      end loop;
      return result;
   end int2slv;

-----------------------------------------------------------------------------------

   function slv2int(vect : std_logic_vector; size : integer) return integer is

   variable result : integer range (2**size)-1 downto 0;
   begin
      result := 0;
      for i in 0 to size-1 loop

         if( vect(i) = '1' ) then
            result := result + 2**i;
         end if;

      end loop;
      return result;
   end slv2int;

----------------------------------------------------------------------------------- 

   --Function modulo2(a:integer; b:integer) return integer is
   --variable eval :std_logic_vector(7 downto 0);
   --begin
   --   eval := (int2slv(a, 8) and int2slv(1, 8)) xor int2slv(b,8);
   --   return slv2int(eval,8);
   --end modulo2;

----------------------------------------------------------------------------------- 

  Function maxf(left: integer; right:integer) return integer is
  begin
    if left > right then return left;
    else return right;
    end if;
  end maxf;

----------------------------------------------------------------------------------- 

  Function minf(left: integer; right:integer) return integer is
  begin
    if left < right then return left;
    else return right;
    end if;
  end minf;

end hw_node_pkg;
