
------------------------------------------------------------------------------
-- Filename:          fsl_const
-- Version:           1.00.a
-- Description:       Example FSL core (VHDL).
-- Date:              Mon May 24 13:16:55 2010 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------
-- Entity Section
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Entity Section
------------------------------------------------------------------------------

entity common_const_connector is
    generic (
        C_FSL_CONST     : integer := 0;     
        C_FSL_DWIDTH    : integer := 31
    );
    port 
    (
        FSL_M_CLK    : out std_logic;
        FSL_M_Wr     : out std_logic;
        FSL_M_Dout   : out std_logic_vector(C_FSL_DWIDTH downto 0);
        FSL_M_CTRL   : out std_logic;
        FSL_M_Full   : in  std_logic;
		RST          : in std_logic;
		CLK          : in std_logic
    
    );
end common_const_connector;

------------------------------------------------------------------------------
-- Architecture Section
------------------------------------------------------------------------------

architecture RTL of common_const_connector is
begin
    FSL_M_CLK       <= '0';
    FSL_M_Dout      <= STD_LOGIC_VECTOR(TO_SIGNED(C_FSL_CONST,C_FSL_DWIDTH+1));
    FSL_M_CTRL      <= '0';
    FSL_M_Wr        <= '1';
end architecture RTL;

