 -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --

library IEEE;
use IEEE.STD_LOGIC_1164.all; 
use ieee.numeric_std.all;

library compaandesign_com_common_common_1_lib;
use compaandesign_com_common_common_1_lib.hw_node_pkg.all;

entity counter is
   generic(
      C_STEP    : natural := 1;
      C_WIDTH   : natural := 10
   );
   port (
      RST       : in  std_logic;
      CLK       : in  std_logic;
      ENABLE    : in  std_logic;
      LOWER_BND : in  std_logic_vector(C_WIDTH-1 downto 0);
      UPPER_BND : in  std_logic_vector(C_WIDTH-1 downto 0);
      ITERATOR  : out std_logic_vector(C_WIDTH-1 downto 0);
      REG_CNTR  : out std_logic_vector(C_WIDTH-1 downto 0);
      DONE      : out std_logic
   );
end counter;

architecture RTL of counter is

   signal sl_counter    : unsigned(C_WIDTH-1 downto 0);
   signal sl_register   : unsigned(C_WIDTH-1 downto 0);
   signal sl_LOWER_BND  : unsigned(C_WIDTH-1 downto 0);
   signal sl_UPPER_BND  : unsigned(C_WIDTH-1 downto 0);
   signal sl_last_count : std_logic;
   signal sl_done       : std_logic;

begin 
   ITERATOR(C_WIDTH-1 downto 0) <= STD_LOGIC_VECTOR(sl_counter);
   REG_CNTR(C_WIDTH-1 downto 0) <= STD_LOGIC_VECTOR(sl_register);

   sl_LOWER_BND  <= UNSIGNED(LOWER_BND(C_WIDTH-1 downto 0));
   sl_UPPER_BND  <= UNSIGNED(UPPER_BND(C_WIDTH-1 downto 0));

   
   sl_counter    <= sl_LOWER_BND when (sl_done='1' or RST='1'    ) else (sl_register + C_STEP);
   --sl_last_count <= '1'          when (sl_counter >= sl_UPPER_BND) else '0';
   sl_last_count <= '1'          when (sl_register >= sl_UPPER_BND) else '0';
   sl_done       <= sl_last_count;

   REG_PRCS : process(CLK)
   begin
      if rising_edge(CLK) then
          if( RST='1' ) then
             sl_register <= sl_LOWER_BND;
             --sl_done     <= sl_last_count;    -- special case: (sl_LOWER_BND == sl_UPPER_BND)  
          elsif( ENABLE='1' ) then
                sl_register <= sl_counter;
                --sl_done     <= sl_last_count;
          end if;
      end if;
   end process;


--   REG_PRCS : process(CLK, RST)
--   begin
--      if( RST='1' ) then
--         sl_register <= sl_LOWER_BND;
--         sl_done     <= sl_last_count;    -- special case: (sl_LOWER_BND == sl_UPPER_BND)  
--      elsif rising_edge(CLK) then
--         if( ENABLE='1' ) then
--            sl_register <= sl_counter;
--            sl_done     <= sl_last_count;
--         end if;
--      end if;
--   end process;
   --
   DONE <= sl_done;
   

end RTL;

--==============================================================================

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library compaandesign_com_common_common_1_lib;
use compaandesign_com_common_common_1_lib.hw_node_pkg.all;


entity gen_counter is
   generic (
      WRAP         : boolean := true;
      N_CNTRS      : natural := 1;
      QUANT        : natural := 32;
      CNTR_STEP    : t_counter_step  := ( 0=> 1, 1=> 1, 2=>1, others=> 1 );
      CNTR_WIDTH   : t_counter_width := ( 0=>10, 1=>10, 2=>9, others=>10 )
   );
   port (
      RST          : in  std_logic;
      CLK          : in  std_logic;
      FIRE         : in  std_logic;
      ENABLE       : in  std_logic;
      LOWER_BND_IN : in  std_logic_vector(N_CNTRS*QUANT-1 downto 0);
      UPPER_BND_IN : in  std_logic_vector(N_CNTRS*QUANT-1 downto 0);
      ITERATORS    : out std_logic_vector(N_CNTRS*QUANT-1 downto 0);
      REG_CNTRS    : out std_logic_vector(N_CNTRS*QUANT-1 downto 0);
      DONE         : out std_logic;
      STOP         : out std_logic
   );
end gen_counter;

architecture RTL of gen_counter is

   component counter is
      generic(
         C_STEP    : natural := 10;
         C_WIDTH   : natural := 10
      );
      port (
         RST       : in  std_logic;
         CLK       : in  std_logic;
         ENABLE    : in  std_logic;
         LOWER_BND : in  std_logic_vector(C_WIDTH-1 downto 0);
         UPPER_BND : in  std_logic_vector(C_WIDTH-1 downto 0);
         ITERATOR  : out std_logic_vector(C_WIDTH-1 downto 0);
         REG_CNTR  : out std_logic_vector(C_WIDTH-1 downto 0);
         DONE      : out std_logic
      );
   end component;

   signal sl_cntr_en  : std_logic_vector(N_CNTRS   downto 0);
   signal sl_done     : std_logic_vector(N_CNTRS-1 downto 0);
   signal sl_done_all : std_logic;
   signal sl_stop       : std_logic;

begin

   GEN_LAB : for i in 0 to N_CNTRS-1 generate
      CNTR : counter
      generic map ( 
         C_STEP    => CNTR_STEP(i),
         C_WIDTH   => CNTR_WIDTH(i)
      )
      port map (
         CLK       => CLK,
         RST       => RST,
         ENABLE    => sl_cntr_en(i),
         LOWER_BND => LOWER_BND_IN(i*QUANT+CNTR_WIDTH(i)-1 downto i*QUANT),
         UPPER_BND => UPPER_BND_IN(i*QUANT+CNTR_WIDTH(i)-1 downto i*QUANT),
         ITERATOR  => ITERATORS(i*QUANT+CNTR_WIDTH(i)-1 downto i*QUANT),
         REG_CNTR  => REG_CNTRS(i*QUANT+CNTR_WIDTH(i)-1 downto i*QUANT),
         DONE      => sl_done(i)
      );
   end generate;


   DONE_PRCS: process(CLK)
   begin
     if rising_edge(CLK) then   
         if( RST = '1' ) then
            sl_stop     <= '0';
            sl_done_all <= '0';        
         else
            if (sl_cntr_en(N_CNTRS)='1' and WRAP=false) then
                sl_stop <= '1';
            end if;
            if (sl_stop='0') then
                sl_done_all <= sl_cntr_en(N_CNTRS);
            end if;      
         end if;
     end if;
   end process;


--   DONE_PRCS: process(CLK, RST)
--   begin
--     if( RST = '1' ) then
--        sl_stop     <= '0';
--        sl_done_all <= '0';        
--     elsif rising_edge(clk) then
--        if (sl_cntr_en(N_CNTRS)='1' and WRAP=false) then
--            sl_stop <= '1';
--        end if;
--        if (sl_stop='0') then
--            sl_done_all <= sl_cntr_en(N_CNTRS);
--        end if;      
--     end if;
--   end process;
   
   DONE <= sl_done_all;     -- '1' in the clock cycle when the counters roll over (from the last point in the space to the firts pont) 
   STOP <= sl_stop;         -- '1' = The couter stoped after the end of the itteration space. 
   
   sl_cntr_en(0) <=  '0' when (sl_stop='1') else
                     '1' when ((FIRE='0') or (ENABLE='1')) else '0';		   
   sl_cntr_en(N_CNTRS  downto 1) <= sl_cntr_en(N_CNTRS-1  downto 0) and sl_done(N_CNTRS-1  downto 0);         
   
end RTL;
