  -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;	

library compaandesign_com_common_common_1_lib;
use compaandesign_com_common_common_1_lib.hw_node_pkg.all;

library compaandesign_com_common_altera_1_lib;
use compaandesign_com_common_altera_1_lib.all;

entity PARAMETERS is
   generic (
      PAR_WIDTH     : natural;
      PAR_BITWIDTH  : natural;
      PAR_VECTOR    : t_par_vector;
      CNT_BITWIDTH  : natural := 10;
      N_PAR			: natural
   );
   port (
      RST        : in  std_logic;
      CLK        : in  std_logic;
      FIFO_FULL  : out std_logic;
      SOF_RD     : in std_logic;
      SOF_WR     : in std_logic;
      HALT_RD    : out std_logic;
      HALT_WR    : out std_logic;
      SYNC_NUM   : out std_logic_vector(9 downto 0);
      PARAM_DT   : in  std_logic_vector(CNT_BITWIDTH+PAR_BITWIDTH-1 downto 0);
      PARAM_LD   : in  std_logic;
      PARAMETERS_RD : out std_logic_vector(PAR_BITWIDTH-1 downto 0);
      PARAMETERS_WR : out std_logic_vector(PAR_BITWIDTH-1 downto 0)
   );
end PARAMETERS;

architecture RTL of PARAMETERS is 

component fsl_v20 is
  generic (
    C_EXT_RESET_HIGH : integer := 1;
    C_ASYNC_CLKS     : integer := 0;
    C_IMPL_STYLE     : integer := 0;
    C_USE_CONTROL    : integer := 1;
    C_FSL_DWIDTH     : integer := 32;
    C_FSL_DEPTH      : integer := 8
    );
  port (
    -- Clock and reset signals
    FSL_Clk : in  std_logic;
    SYS_Rst : in  std_logic;
    FSL_Rst : out std_logic;

    -- FSL master signals
    FSL_M_Clk     : in  std_logic;
    FSL_M_Data    : in  std_logic_vector(CNT_BITWIDTH+PAR_WIDTH-1 downto 0);
    FSL_M_Control : in  std_logic;
    FSL_M_Write   : in  std_logic;
    FSL_M_Full    : out std_logic;

    -- FSL slave signals
    FSL_S_Clk     : in  std_logic;
    FSL_S_Data    : out std_logic_vector(CNT_BITWIDTH+PAR_WIDTH-1 downto 0);
    FSL_S_Control : out std_logic;
    FSL_S_Read    : in  std_logic;
    FSL_S_Exists  : out std_logic;

    -- FIFO status signals
    FSL_Full     : out std_logic;
    FSL_Has_Data : out std_logic
    );
end component fsl_v20;

   signal sl_det_0, sl_det_1, sl_PARAM_LD : std_logic;
   
   type state_type is (s_idle, s_update_rd, s_update_wr, s_fifo_read);
   signal state : state_type;

	signal fsl_m_control : std_logic;
	signal fsl_m_clk : std_logic;
	signal fsl_write : std_logic;
	signal fsl_data_write : std_logic_vector(CNT_BITWIDTH+PAR_WIDTH-1 downto 0);
	signal fsl_m_full : std_logic;
	signal fsl_read : std_logic;
	signal fsl_data_read : std_logic_vector(CNT_BITWIDTH+PAR_WIDTH-1 downto 0);
	signal fsl_s_clk : std_logic;
	signal fsl_s_control : std_logic;
	signal fsl_s_exists : std_logic;	
	
	signal sof_cnt : natural;
	signal sof_cnt_a : natural;
	signal sof_cnt_b : natural;
	signal sof_num : std_logic_vector(CNT_BITWIDTH-1 downto 0);
	signal sof_num_a : std_logic_vector(CNT_BITWIDTH-1 downto 0);
	signal sof_num_b : std_logic_vector(CNT_BITWIDTH-1 downto 0);
    
	signal sync_in_num : std_logic_vector(CNT_BITWIDTH-1 downto 0);    
	signal sync_now : std_logic;

begin

   GenLabel1 : if N_PAR > 0 generate   
					  	 			  
      -- Rising_edge detection of PARAM_LD signal --------------------
      Edge_det_prcss : process(CLK)
      begin
         if falling_edge( CLK ) then
            sl_det_0 <= PARAM_LD;
            sl_det_1 <= sl_det_0;
         end if;
      end process;

     sl_PARAM_LD <= sl_det_0 and not(sl_det_1);

     -- Update the parameters (from the temp buffer) ----------------                                  

	 FIFO_FULL <= fsl_m_full;          
                     	                             
     FIFO_WRITE : process(CLK, RST)
     begin
		if rising_edge(CLK) then
            if( RST='1' ) then
                fsl_data_write <= (others => '0');
                fsl_write <= '0';
            else     
            	if rising_edge(CLK) then
            		fsl_data_write <= PARAM_DT;
	            if fsl_m_full = '0' and sl_PARAM_LD = '1' then						
					fsl_write <= '1';  
				else
					fsl_write <= '0';          	
   		       end if;
   		     end if;
            end if;
          end if;
     end process;    
                          
     SOF_CNT_Proc : process(CLK,RST,SOF_RD)
     begin
     	if rising_edge(CLK) then
	 	 	if( RST='1' ) then
	            sof_cnt <= 1;
	        else
	        	if(SOF_RD='1') then
		        	if sof_cnt < 100 then
		        		sof_cnt <= sof_cnt + 1;
		        	else
		        		sof_cnt <= 1;
		        	end if;        	
		        end if;
	        end if;	        
	    end if;
     end process;                     
                       
     sof_cnt_a <= sof_cnt + 1 when sof_cnt /= 100 else 1;           
                       
     sof_num <= STD_LOGIC_VECTOR(to_signed(sof_cnt,CNT_BITWIDTH));            
     sof_num_a <= STD_LOGIC_VECTOR(to_signed(sof_cnt_a,CNT_BITWIDTH));             
                          
     sync_now <= '1' when SOF_RD='1' and sync_in_num=sof_num else '0';                     
          
	sync_in_num <= fsl_data_read(CNT_BITWIDTH+PAR_BITWIDTH-1 downto PAR_BITWIDTH);          
          
	SYNC_NUM <= sof_num_a;          
                                                
      FSM : process(CLK, RST)
      begin
         if rising_edge(CLK) then
            if( RST='1' ) then
                state <= s_idle;
                parameters_rd <= STD_LOGIC_VECTOR(to_signed(403,PAR_BITWIDTH));
                parameters_wr <= STD_LOGIC_VECTOR(to_signed(403,PAR_BITWIDTH)); 
                fsl_read <= '0';
            else
                case (state) is
                    when s_idle   =>               	
                    	if (fsl_s_exists='1') then                        		                    		       		
                    		if(sync_now = '1') then
                				parameters_rd <= fsl_data_read(PAR_BITWIDTH-1 downto 0);
                				state <= s_update_wr;
                    		else
                    			state <= s_update_rd;
                    		end if; 	                    		
                     	end if; 
                    when s_update_rd =>                     	
                    	if(sync_now = '1') then
                    		parameters_rd <= fsl_data_read(PAR_BITWIDTH-1 downto 0);  
                    		if(SOF_WR = '1') then     
                    			parameters_wr <= fsl_data_read(PAR_BITWIDTH-1 downto 0);
                    			fsl_read <= '1';
                    			state <= s_fifo_read;
                    		else                    			           		
	                			state <= s_update_wr;
                    		end if;                	                    		
                    	end if;
                    when s_update_wr =>                 	
                    	if(SOF_WR = '1') then
                    		parameters_wr <= fsl_data_read(PAR_BITWIDTH-1 downto 0);
							fsl_read <= '1';
                			state <= s_fifo_read;
                    	end if;
                    when s_fifo_read =>      
                    	fsl_read <= '0';                       	
                		state <= s_idle;
                    when others => state <= s_idle;
                end case;
            end if;
         end if;
      end process;      
	
  fsl_m_control <= '0';
  fsl_m_clk <= '0';
  fsl_s_clk <= '0';
  fsl_s_control <= '0';

  fsl_cirular : fsl_v20 
   generic map (
     C_EXT_RESET_HIGH => 1,
     C_ASYNC_CLKS     =>0,
     C_USE_CONTROL    => 1,
     C_IMPL_STYLE => 0,
     C_FSL_DWIDTH => PAR_BITWIDTH + CNT_BITWIDTH,
     C_FSL_DEPTH =>  4
   )
   port map (
        -- Clock and reset signals
        FSL_Clk => CLK,
        SYS_Rst => RST,
        FSL_Rst => open,
        -- FSL master signals
        FSL_M_Clk     => fsl_m_clk,
        FSL_M_Data    => fsl_data_write,
        FSL_M_Control => fsl_m_control,
        FSL_M_Write   => fsl_write,
        FSL_M_Full    => fsl_m_full,
        -- FSL slave signals
        FSL_S_Clk     => fsl_s_clk,
        FSL_S_Data    => fsl_data_read,
        FSL_S_Control => fsl_s_control,
        FSL_S_Read    => fsl_read,
        FSL_S_Exists  => fsl_s_exists,
        -- FIFO status signals
        FSL_Full        => open,
        FSL_Has_Data    => open
   );	

   end generate; -- GenLabel1

end RTL;
