-- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library compaandesign_com_common_altera_1_lib;
use compaandesign_com_common_altera_1_lib.all;

entity read_mmux is
   generic(
      N_PORTS    : natural := 1;
      PORT_WIDTH : natural := 32;
      C_IMPL_STYLE : integer := 0;
      C_FSL_DWIDTH : natural := 32;
      C_FSL_DEPTH  : natural := 64
   );
   port(
      IN_PORTS     : in  std_logic_vector(N_PORTS*PORT_WIDTH-1 downto 0);
      EXISTS       : in  std_logic_vector(N_PORTS-1 downto 0);
      READS        : out std_logic_vector(N_PORTS-1 downto 0);
      SOFS         : in  std_logic_vector(N_PORTS-1 downto 0);

      OUT_PORT     : out std_logic_vector(PORT_WIDTH-1 downto 0);
      EXIST        : out std_logic;
      READ         : in  std_logic;
      SOF          : in  std_logic;
      CLK          : in  std_logic;

      READ_EN      : in  std_logic;
      READ_ST      : out std_logic;
      RELEASE      : in  std_logic_vector(N_PORTS-1 downto 0);
	  OBTAIN       : in  std_logic_vector(N_PORTS-1 downto 0);
      CONTROL      : in  std_logic_vector(N_PORTS-1 downto 0);
      
      RST          : in std_logic;
      OBTAIN_OOMX  : in std_logic_vector(N_PORTS-1 downto 0);
      RELEASE_OOMX : in std_logic_vector(N_PORTS-1 downto 0)
   );
end read_mmux;

architecture RTL of read_mmux is

signal fsl_m_control : std_logic;
signal fsl_m_clk : std_logic;
signal fsl_write : std_logic;
signal fsl_data_write : std_logic_vector(0 to C_FSL_DWIDTH-1);
signal fsl_m_full : std_logic;
signal fsl_read : std_logic;
signal fsl_data_read : std_logic_vector(0 to C_FSL_DWIDTH-1);
signal fsl_s_clk : std_logic;
signal fsl_s_control : std_logic;
signal fsl_s_exists : std_logic;

signal sl_read  : std_logic;
signal sl_exist : std_logic;  
 signal sl_avail : std_logic;

component fsl_v20 is
  generic (
    C_EXT_RESET_HIGH : integer := 1;
    C_ASYNC_CLKS     : integer := 0;
    C_IMPL_STYLE     : integer := 0;
    C_USE_CONTROL    : integer := 1;
    C_FSL_DWIDTH     : integer := 32;
    C_FSL_DEPTH      : integer := 16
    );
  port (
    -- Clock and reset signals
    FSL_Clk : in  std_logic;
    SYS_Rst : in  std_logic;
    FSL_Rst : out std_logic;

    -- FSL master signals
    FSL_M_Clk     : in  std_logic;
    FSL_M_Data    : in  std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_M_Control : in  std_logic;
    FSL_M_Write   : in  std_logic;
    FSL_M_Full    : out std_logic;

    -- FSL slave signals
    FSL_S_Clk     : in  std_logic;
    FSL_S_Data    : out std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_S_Control : out std_logic;
    FSL_S_Read    : in  std_logic;
    FSL_S_Exists  : out std_logic;

    -- FIFO status signals
    FSL_Full     : out std_logic;
    FSL_Has_Data : out std_logic
    );
end component fsl_v20;

   
	 
begin

	-- REUSE_READ => OBTAIN
	-- REUSE		  => RELEASE

   EXIST   <= READ_EN and sl_exist;
   sl_read <= READ_EN and READ;
   READ_ST <= (sl_read and sl_exist) or (sl_avail);
    
   DEMUX_GEN : for i in 0 to N_PORTS-1 generate
	    READS(i) <= EXISTS(i) and CONTROL(i) and sl_read and OBTAIN_OOMX(i);
		 fsl_read  <= not OBTAIN_OOMX(i) and (sl_read );
		 fsl_write <= not RELEASE_OOMX(i) and (sl_read );
   end generate;

   MUX_PRCSS :  process(CONTROL, RELEASE_OOMX, OBTAIN_OOMX, IN_PORTS, fsl_data_read)
   begin
     OUT_PORT <= (others=>'0');
     -- OUT_PORT <= IN_PORTS(PORT_WIDTH-1 downto 0);
     
     -- Extra check giving feedback that sizing FIFO is very tight or too small
     assert fsl_m_full = '0' 
     	report "Internal FIFO is FULL!!!" 
     	severity WARNING; 
     
     for i in 0 to N_PORTS-1 loop
		
			-- If control is high, we need data
         if( CONTROL(i) = '1'  ) then
			
                -- peek (take the token fron the fsl and send it to the output and back to fsl)
					 -- obtained is hoog, en release hoog
					 -- Should not happen
                if( OBTAIN_OOMX(i) = '1' and RELEASE_OOMX(i) = '1') then
							
						  OUT_PORT <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);
                    --assert false
                    --    report "Phase in which OBTAIN and RELEASE are both 1 should never be reached"
                    --    severity FAILURE;                    
								
					 sl_avail <= '0';
					 -- sl_exist <= EXISTS(i);
					 
                    --OUT_PORT <= fsl_data_read;
                    --fsl_data_write <= fsl_data_read;
                end if;
					 
                -- peek last (read from the fifo without rewriting to the fsl)
					 -- obtained is hoog, en release laag
					 -- Read from external FIFO and store locally.
                if( OBTAIN_OOMX(i) = '1' and RELEASE_OOMX(i) = '0') then
							
							-- read from outside and forward
							OUT_PORT <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);
							
							-- and put a copy in the local FIFO
                     fsl_data_write <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);
		
							sl_avail <= '0';
							-- sl_exist <= EXISTS(i);
							
							--OUT_PORT <= fsl_data_read;
                     --fsl_data_write <= fsl_data_read;
                end if;
          
                -- take first (take the first token from te input and store it to the fsl)
					 -- obtained is laag, en release hoog
					 
                if( OBTAIN_OOMX(i) = '0' and RELEASE_OOMX(i) = '1') then
					 
						  -- read from the internal FIFO, and let the FIFO run empty
                    OUT_PORT <= fsl_data_read;                    
						  sl_avail <= '1';
						  -- sl_exist <= '1' ;
						  
                end if;
					 
                --take (take from the input and wirte to the output)
					 -- obtained is laag, en release laag
                if( OBTAIN_OOMX(i) = '0'  and RELEASE_OOMX(i) = '0') then
					 
					     -- steady state...
						  -- read from internal FIFO
                    OUT_PORT <= fsl_data_read;
						  
						  -- and put the value back in the FIFO
                    fsl_data_write <= fsl_data_read;    
						  
						  sl_avail <= '1';
						  -- sl_exist <= '1';
						  
                end if;
         end if;
            
        end loop;
   end process;

    
   MUX_EXIST :  process(EXISTS, OBTAIN_OOMX, CONTROL)
   begin
   -- The default value needs to be '1'. See the node's top-level: sl_exist <= sl_exist_1 and sl_exist_0;
      --EXIST <= '0';
     -- sl_exist <= '0';
      for i in 0 to N_PORTS-1 loop
        if( CONTROL(i) = '1'  ) then
            --EXIST <= EXISTS(i);
				sl_exist <= (EXISTS(i) and OBTAIN_OOMX(i)) or (not OBTAIN_OOMX(i));
				
				-- Problem hier is dat EXISTS door FIFO wordt gestuurd. Dus in interne FIFO, moet deze hoog worden, ongeacht wat externe FIFO zegt...
			--	sl_exist <= '1' when ((EXISTS(i)='1' and (REUSE_READ(i) = '1')) or (REUSE_READ(i)='0')) else '0';
				
				
         end if;
      end loop;
   end process;
   
   
   -- Checks
   process(CONTROL)
      variable c : integer;
   begin
      c := 0;
      for i in 0 to CONTROL'Length-1 loop
         if (CONTROL(i)='1') then
            c := c + 1;
         end if;
      end loop;
      assert (c <= 1)
         report "Signal CONTROL contains more than one bit that is set to '1' ! The CONTROL signal must be one-hot encoded. "
         severity ERROR;
   end process;


  fsl_m_control <= '0';
  fsl_m_clk <= '0';
  fsl_s_clk <= '0';
  fsl_s_control <= '0';

  fsl_cirular : fsl_v20 
   generic map (
     C_EXT_RESET_HIGH => 1,
     C_ASYNC_CLKS     =>0,
     C_USE_CONTROL    => 1,
     C_IMPL_STYLE => C_IMPL_STYLE,
     C_FSL_DWIDTH => C_FSL_DWIDTH,
     C_FSL_DEPTH =>  C_FSL_DEPTH
   )
   port map (
   -- Clock and reset signals
         FSL_Clk => CLK,
         SYS_Rst => RST,
            FSL_Rst => open,
   -- FSL master signals
        FSL_M_Clk     => fsl_m_clk,
            FSL_M_Data    => fsl_data_write,
            FSL_M_Control => fsl_m_control,
            FSL_M_Write   => fsl_write,
            FSL_M_Full    => fsl_m_full,
         -- FSL slave signals
         FSL_S_Clk     => fsl_s_clk,
         FSL_S_Data    => fsl_data_read,
         FSL_S_Control => fsl_s_control,
         FSL_S_Read    => fsl_read,
         FSL_S_Exists  => fsl_s_exists,
         -- FIFO status signals
        FSL_Full        => open,
        FSL_Has_Data    => open
   );


end RTL;
