###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
import itertools

# Purpose:
# . Print the good visibilities of the intermediate 48-visibility Apertif
#   correlator.
# Usage:
# . Run and observe the output.

###############################################################################
# Our telescope/beamlet -> correlator input mapping
###############################################################################
input_list = [['T0','B0'],  # Input  0
              ['T1','B0'],  # Input  1
              ['T2','B0'],  # Input  2
              ['T0','B1'],  # Input  3
              ['T1','B1'],  # Input  4
              ['T2','B1'],  # Input  5
              ['T0','B2'],  # Input  6
              ['T1','B2'],  # Input  7
              ['T2','B2'],  # Input  8
              ['T0','B3'],  # Input  9
              ['T1','B3'],  # Input 10
              ['T2','B3'],  # Input 11
              ['T0','B4'],  # Input 12
              ['T1','B4'],  # Input 13
              ['T2','B4'],  # Input 14
              ['T0','B5'],  # Input 15
              ['T1','B5'],  # Input 16
              ['T2','B5'],  # Input 17
              ['T0','B6'],  # Input 18
              ['T1','B6'],  # Input 19
              ['T2','B6'],  # Input 20
              ['T0','B7'],  # Input 21
              ['T1','B7'],  # Input 22
              ['T2','B7']]  # Input 23

###############################################################################
# All combinations (visibilities) that will be made by the correlator
###############################################################################
visibilities = [i for i in itertools.combinations_with_replacement(input_list, 2)]

###############################################################################
# We want to know which visibilities carry a proper correlation:
# . Good: Beamlet A * BeamleT A
# .  Bad: Beamlet A * Beamlet B
###############################################################################
good_visibilities = []
for index,pair in enumerate(visibilities):
    beamlet_a = pair[0][1]
    beamlet_b = pair[1][1]
    if beamlet_a==beamlet_b:
        good_visibilities.append((index, pair))

###############################################################################
# Print (the number of) the good visibilities
###############################################################################
for index,pair in enumerate(good_visibilities):
    print pair

print 'Number of good visibilities:', len(good_visibilities)
