###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
from common import *
from mem_init_file import list_to_hex

# Purpose:
# . Generate HEX files for the 12 block generators, tagging each stream with an ID.
# Description:
# . The telescope input number and beamlet number are encoded in the real and
#   imaginary parts as follows:
#   . Stream  0: Telescope input 0, beamlet 0: complex(0,0)
#   .                              ..
#   . Stream  7: Telescope input 0, beamlet 7: complex(0,7)
#   . ..
#   . ..
#   . Stream 16: Telescope input 2, beamlet 0: complex(2,0)
#   .                              ..
#   . Stream 23: Telescope input 2, beamlet 7: complex(2,7)
# . The above 24 streams are interleaved into 12.

NOF_INPUTS = 12

NOF_TELESCOPE_INPUTS = 3
NOF_BEAMLETS_PER_INPUT = 8

COMPLEX_WIDTH = 8
NOF_WORDS_PER_BLOCK = 64

PATH = os.environ['RADIOHDL']+'/applications/apertif/designs/apertif_unb1_correlator/src/hex'
FILENAME = "ID_streams"

###############################################################################
# Create ID lists
###############################################################################

input_signals = []
for telescope_nr in range(NOF_TELESCOPE_INPUTS):
    re = telescope_nr
    for beamlet_nr in range(NOF_BEAMLETS_PER_INPUT):
        im = beamlet_nr
        input_signals.append( NOF_WORDS_PER_BLOCK*[complex(re,im)])

###############################################################################
# Convert complex to concatenated integers
###############################################################################
input_signals_concat = []
for input_signal in input_signals:
    input_signals_concat.append( concat_complex(input_signal, COMPLEX_WIDTH) )

###############################################################################
# Interleave 2 lists into one
###############################################################################
input_signals_concat_inter = []
for i in range(NOF_INPUTS):
    input_signals_concat_inter.append( interleave([input_signals_concat[2*i],input_signals_concat[2*i+1]]) )

###############################################################################
# Use this list for each block generator
###############################################################################
bg_lists = input_signals_concat_inter

###############################################################################
# Write the HEX files
###############################################################################
MEM_WIDTH  = COMPLEX_WIDTH*2
MEM_DEPTH  = 2*NOF_WORDS_PER_BLOCK # We're interleaving, hence twice the depth

for input_nr in range(NOF_INPUTS):
    list_to_hex( bg_lists[input_nr], PATH+"/"+FILENAME+'_'+str(input_nr)+".hex", MEM_WIDTH, MEM_DEPTH)

