#! /usr/bin/env python
###############################################################################
#                                                                             
# Copyright (C) 2015
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

# Purpose:
# . Read/write fields in the visibility packet headers
# Usage:
# . Read all fields:
#   . python pi_apertif_unb1_correlator_offload.py --unb 4 --fn 0
# . Read specific field:
#   . python pi_apertif_unb1_correlator_offload.py --unb 4 --fn 0 --reg id_beamlet_index
# . Write new dest MAC (a0:36:9f:48:f2:89 = 176156461036169) and IP (10.199.0.254 = 0x0ac700fe = 180814078):
#   . python pi_apertif_unb1_correlator_offload.py --unb 4 --fn 0 --reg eth_dst_mac=176156461036169,ip_dst_addr=180814078

from pi_common import *

PINAME   = 'REG_DP_OFFLOAD_TX_HDR_DAT'
PID      = 'VIS_OFFLOAD'
ADDR_W   = 6
VHDL_SRC_FILE  = '../../src/vhdl/apertif_unb1_correlator_vis_offload.vhd'
VHDL_SRC_CONST = 'c_hdr_field_arr'
REGMAP = extract_regmap_from_vhd(VHDL_SRC_FILE, VHDL_SRC_CONST)

#-------------------------------------------------------------------------------
class PiBsnMonitor(PiCommon):
    
    def __init__(self, tc, io, inst_name='', nof_inst=None, node_nrs=None, regmap=None):
        PiCommon.__init__(self, tc, io, PINAME, PID, REGMAP, ADDR_W, inst_name, nof_inst)

#-------------------------------------------------------------------------------
# On execution:
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    on_execute(class_definition=PiBsnMonitor,regmap=REGMAP)
