------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------
LIBRARY IEEE, unb1_board_lib, apertif_unb1_fn_beamformer_lib, tech_ddr_lib;
USE IEEE.STD_LOGIC_1164.ALL;                   
USE unb1_board_lib.unb1_board_pkg.ALL;  
USE tech_ddr_lib.tech_ddr_pkg.ALL;       

ENTITY apertif_unb1_fn_beamformer_base IS
  GENERIC (
    g_design_name : STRING  := "apertif_unb1_fn_beamformer_base"; 
    g_design_note : STRING  := "base revision"; 
    g_sim         : BOOLEAN := FALSE; 
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0;
    g_stamp_date  : NATURAL := 0;
    g_stamp_time  : NATURAL := 0;
    g_stamp_svn   : NATURAL := 0;
    g_use_MB_I    : NATURAL := 0 
  );
  PORT (
   -- GENERAL
    CLK                    : IN    STD_LOGIC; -- System Clock
    PPS                    : IN    STD_LOGIC; -- System Sync
    WDI                    : OUT   STD_LOGIC; -- Watchdog Clear
    INTA                   : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB                   : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION                : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID                     : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO                 : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc                : INOUT STD_LOGIC;
    sens_sd                : INOUT STD_LOGIC;

    -- 1GbE Control Interface
    ETH_clk                : IN    STD_LOGIC;
    ETH_SGIN               : IN    STD_LOGIC;
    ETH_SGOUT              : OUT   STD_LOGIC;
    
    -- Transceiver clocks
    SA_CLK                 : IN  STD_LOGIC := '0';  -- TR clock BN-BI (tr_back) / SI_FN (tr_front)
    SB_CLK                 : IN  STD_LOGIC := '0';  -- TR clock FN-BN (tr_mesh)

    -- Mesh Serial I/O
    FN_BN_0_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_0_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_1_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_1_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_2_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_2_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_3_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_3_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');

    -- Front Serial I/O
    SI_FN_0_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_0_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);

    SI_FN_0_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); -- (0 = LASI; 1=MDC; 2=MDIO)
    SI_FN_1_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_RSTN             : OUT   STD_LOGIC := '1'; -- ResetN is pulled up in the Vitesse chip, but pulled down again by external 1k resistor.
                                                    -- So we need to assign a '1' to it.
    MB_I_IN                : IN    t_tech_ddr3_phy_in_arr(g_use_MB_I-1 DOWNTO 0);     
    MB_I_IO                : INOUT t_tech_ddr3_phy_io_arr(g_use_MB_I-1 DOWNTO 0); 
    MB_I_OU                : OUT   t_tech_ddr3_phy_ou_arr(g_use_MB_I-1 DOWNTO 0) 
  );
END apertif_unb1_fn_beamformer_base;

ARCHITECTURE str OF apertif_unb1_fn_beamformer_base IS

BEGIN 

  u_revision : ENTITY apertif_unb1_fn_beamformer_lib.apertif_unb1_fn_beamformer 
  GENERIC MAP (
    g_design_name => g_design_name,
    g_design_note => g_design_note,
    g_sim         => g_sim,        
    g_sim_unb_nr  => g_sim_unb_nr, 
    g_sim_node_nr => g_sim_node_nr,
    g_stamp_date  => g_stamp_date, 
    g_stamp_time  => g_stamp_time, 
    g_stamp_svn   => g_stamp_svn,
    g_use_MB_I    => g_use_MB_I  
  )
  PORT MAP (
    CLK          => CLK,  
    PPS          => PPS,            
    WDI          => WDI,            
    INTA         => INTA,           
    INTB         => INTB,           

    VERSION      => VERSION,            
    ID           => ID,                 
    TESTIO       => TESTIO, 

    sens_sc      =>  sens_sc,           
    sens_sd      =>  sens_sd,           

    ETH_clk      => ETH_clk,            
    ETH_SGIN     => ETH_SGIN,           
    ETH_SGOUT    => ETH_SGOUT,          
    
    SA_CLK        => SA_CLK,               
    SB_CLK        => SB_CLK,               

    FN_BN_0_TX    => FN_BN_0_TX,          
    FN_BN_0_RX    => FN_BN_0_RX,          
    FN_BN_1_TX    => FN_BN_1_TX,          
    FN_BN_1_RX    => FN_BN_1_RX,          
    FN_BN_2_TX    => FN_BN_2_TX,          
    FN_BN_2_RX    => FN_BN_2_RX,          
    FN_BN_3_TX    => FN_BN_3_TX,          
    FN_BN_3_RX    => FN_BN_3_RX,          

    SI_FN_0_TX    => SI_FN_0_TX,          
    SI_FN_0_RX    => SI_FN_0_RX,          
    SI_FN_1_TX    => SI_FN_1_TX,          
    SI_FN_1_RX    => SI_FN_1_RX,          
    SI_FN_2_TX    => SI_FN_2_TX,          
    SI_FN_2_RX    => SI_FN_2_RX,          
    SI_FN_3_TX    => SI_FN_3_TX,          
    SI_FN_3_RX    => SI_FN_3_RX,          

    SI_FN_0_CNTRL => SI_FN_0_CNTRL,       
    SI_FN_1_CNTRL => SI_FN_1_CNTRL,       
    SI_FN_2_CNTRL => SI_FN_2_CNTRL,       
    SI_FN_3_CNTRL => SI_FN_3_CNTRL,       
    SI_FN_RSTN    => SI_FN_RSTN,
    
    MB_I_IN       => MB_I_IN, 
    MB_I_IO       => MB_I_IO, 
    MB_I_OU       => MB_I_OU     
  );
END str;
