#! /usr/bin/env python
###############################################################################
#
# Copyright (C) 2012
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

"""Test case for the reorder_transpose entity.

   Description:

    
   Usage:

   > python tc_reorder_transpose.py --unb 0 --fn 0 --sim 

"""

###############################################################################
# System imports
import test_case
import node_io
import unb_apertif as apr
import pi_diag_block_gen
import pi_diag_data_buffer
import pi_ss_ss_wide
import pi_bf_bf
import pi_io_ddr   
import pi_bsn_monitor
import pi_ppsh
import dsp_test

import sys, os
import subprocess  
import time
import pylab as pl
import numpy as np
import scipy as sp
import random
from tools import *
from common import *  
import mem_init_file

###############################################################################

# Create a test case object
tc = test_case.Testcase('TB - ', '')

# Constants/Generics that are shared between VHDL and Python
# Name                   Value   Default   Description
# START_VHDL_GENERICS
g_nof_signal_paths     = 64      # 64
g_nof_input_streams    = 16       # 16 
g_nof_subbands         = 24      # 24
g_nof_weights          = 256      # 256
g_nof_bf_units         =  4      # 4
g_in_dat_w             = 16      # 16
g_in_weight_w          = 16      # 16  

g_blocks_per_sync  = 800000 #32   # 781250
if tc.sim == True:                        
    g_blocks_per_sync  = 16   # 781250                                           
g_wr_chunksize     = 176 
g_rd_chunksize     = 16  
g_rd_nof_chunks    = 11  
g_rd_interval      = 16 
g_gapsize          = 0   

# END_VHDL_GENERICS

# Overwrite generics with argumented generics from autoscript or command line. 
#if tc.generics != None:  
#    g_wr_chunksize     = tc.generics['g_wr_chunksize']
#    g_rd_chunksize     = tc.generics['g_rd_chunksize']      
#    g_rd_nof_chunks    = tc.generics['g_rd_nof_chunks']    
#    g_rd_interval      = tc.generics['g_rd_interval']    
#    g_gapsize          = tc.generics['g_gapsize']      
#    g_nof_blocks       = tc.generics['g_nof_blocks']      
#    g_frame_size_in    = tc.generics['g_frame_size_in']          
#    g_frame_size_out   = tc.generics['g_frame_size_out']          


# Define settings for the block generator
c_bg_nof_streams          = g_nof_input_streams 
c_nof_sp_per_input_stream = g_nof_signal_paths / g_nof_input_streams
c_nof_subbands_per_stream = g_nof_subbands*c_nof_sp_per_input_stream 
c_bg_ram_size             = c_nof_subbands_per_stream 
c_samples_per_packet      = c_nof_sp_per_input_stream * g_nof_subbands
c_gapsize                 = g_nof_weights - c_samples_per_packet
c_mem_low_addr            = 0
c_mem_high_addr           = c_samples_per_packet-1
c_bsn_init                = 42

# Define stuff for transpose
c_nof_int_streams   = 1   
c_ena_pre_transpose = True
c_interleave        = 2
c_bf_in_dat_w       = 16  
c_tp_in_dat_w       = 8


c_write_bg_data            = False 
c_write_bg_data_to_file    = False  
c_write_weights            = False   
c_write_weights_to_file    = False 
c_write_bf_ss_wide         = False 
c_write_bf_ss_wide_to_file = False
                             
tc.append_log(3, '>>>')      
tc.append_log(1, '>>> Title : Test script for apertif_unb1_fn_beamformer_tp_bg' )
tc.append_log(3, '>>>')
tc.append_log(3, '')
tc.set_result('PASSED')

# Create access object for nodes
io = node_io.NodeIO(tc.nodeImages, tc.base_ip)

# Create block generator instance
bg = pi_diag_block_gen.PiDiagBlockGen(tc, io, g_nof_input_streams, c_bg_ram_size)

# Create instances for the beamformer units (BF)
bf=[]   
for i in range(tc.nofFnNodes):
    for j in xrange(g_nof_bf_units):
        bf.append(pi_bf_bf.PiBfBf(tc, io, g_nof_weights, g_nof_signal_paths, g_nof_input_streams, xstEnable=True, instanceNr=j, nodeNr=tc.nodeFnNrs[i]))

# BSN monitor
bsn_out = pi_bsn_monitor.PiBsnMonitor(tc, io, instanceName='OUTPUT', nofStreams=2)

# Create subandselect instance for pre-transpose.   
ss = pi_ss_ss_wide.PiSsSsWide (tc, io, g_wr_chunksize*g_rd_chunksize, c_nof_int_streams, instanceName='TRANSP') 

# Create object for DDR register map
ddr = pi_io_ddr.PiIoDdr(tc, io, nof_inst = 1)

# Create dsp_test instance for helpful methods
dsp_test_bg = dsp_test.DspTest(inDatW=c_bf_in_dat_w)

pps = pi_ppsh.PiPpsh(tc, io, nodeNr=tc.nodeFnNrs)


if __name__ == "__main__":  

    # Wait until the DDR3 model is initialized.                                              
    if tc.sim == True:                                                                   
        do_until_eq(ddr.read_init_done, ms_retry=1000, val=1, s_timeout=13600) 

    ddr.read_usedw_wr_fifo()
    ddr.read_usedw_rd_fifo()
    ddr.read_wait_request_n()
    ddr.read_cal_success()
    ddr.read_cal_fail()
    
    bsn_out.read_bsn_monitor(0)
    bsn_out.read_bsn_monitor(1)

    ################################################################################
    ##
    ## Initialize the blockgenerators
    ##
    ################################################################################
    # - Write settings to the block generator 
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Write settings to the block generator')
    tc.append_log(3, '>>>')

    if tc.sim == False:                                                                   
        bg.write_block_gen_settings(256, g_blocks_per_sync, 0, 0, 127, c_bsn_init)
    else:
        bg.write_block_gen_settings(c_samples_per_packet, g_blocks_per_sync, c_gapsize, c_mem_low_addr, c_mem_high_addr, c_bsn_init)
    
    # - Create a list with the input data and write it to the RAMs of the block generator
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Write data to the waveform RAM of all channels')
    tc.append_log(3, '>>>')
    inputData = []
    for i in xrange(g_nof_input_streams):
        dataList = bg.generate_data_list(c_nof_sp_per_input_stream, g_nof_subbands, 2048*i*4, i, c_bf_in_dat_w)
        if c_write_bg_data == True:                                                                   
            bg.write_waveform_ram(dataList, i)
        if c_write_bg_data_to_file == True:                                                                   
            filename = "../../src/hex/bg_in_data_" + str(i) + ".hex"
            mem_init_file.list_to_hex(list_in=dataList, filename=filename, mem_width=c_nof_complex*c_bf_in_dat_w, mem_depth=2**(ceil_log2(c_bg_ram_size)))
        dataListComplex = bg.convert_concatenated_to_complex(dataList, c_bf_in_dat_w)
        inputData.append(dataListComplex)
    
    ################################################################################
    ##
    ## Create and Write the weight factors 
    ##
    ################################################################################
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Create and write weightfactors for all signal paths on all bf_units ')
    tc.append_log(3, '>>>')
   
    weightsNodes = []
    for k in xrange(tc.nofFnNodes):                    
        weightsBf = []
        for i in range(g_nof_bf_units):
            weightsBfUnit=[]
            for j in range(g_nof_signal_paths):
                weightsSignalPath = bf[k*g_nof_bf_units+i].generate_weights(g_nof_weights, i+j, i, g_in_weight_w)
                if c_write_weights_to_file == True:                                                                   
                    filename = "../../src/hex/bf_weights_" + str(i) + "_" + str(j) + ".hex"
                    mem_init_file.list_to_hex(list_in=weightsSignalPath, filename=filename, mem_width=c_nof_complex*g_in_weight_w, mem_depth=g_nof_weights)
                if c_write_weights == True:                                                                   
                    bf[k*g_nof_bf_units+i].write_weights(weightsSignalPath, j) 
                weightsSignalPathComplex = bg.convert_concatenated_to_complex(weightsSignalPath, g_in_weight_w)
                weightsBfUnit.append(weightsSignalPathComplex)
            weightsBf.append(weightsBfUnit)
        weightsNodes.append(weightsBf)
    
    ################################################################################
    ##
    ## Create and Write the selection buffers
    ##
    ################################################################################
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Create and write selection settings for ss_wides in all bf_units ')
    tc.append_log(3, '>>>')

    select_buf = []
    for b in xrange(g_nof_bf_units):
        for i in range(c_nof_sp_per_input_stream): 
            select_buf_line = []
            for j in range(4):
                for k in range(g_nof_weights/4): 
                    select_buf_line.append(i*g_nof_subbands + j)  
            select_buf.append(select_buf_line)  
            if c_write_bf_ss_wide_to_file == True:                                                                   
                filename = "../../src/hex/bf_ss_wide_" + str(b) + "_" + str(i) + ".hex"
                mem_init_file.list_to_hex(list_in=flatten(select_buf_line), filename=filename, mem_width=ceil_log2(c_nof_subbands_per_stream), mem_depth=g_nof_weights)

    if c_write_bf_ss_wide == True:                                                                   
        for i in range(tc.nofFnNodes):
            for j in xrange(g_nof_bf_units):
                for k in range(g_nof_input_streams):
                    bf[i*g_nof_bf_units + j].ss_wide[k].write_selects(flatten(select_buf)); 

 
      
    ###############################################################################
    #
    # Create setting for the pre-transpose (subbandselect)
    #
    ###############################################################################
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Create settings for pre-transpose')
    tc.append_log(3, '>>>')
    tc.append_log(3, '')

    ss_list = []
    for h in range(g_wr_chunksize/c_interleave):
        for i in range(g_rd_chunksize):
            for j in range(c_interleave):
                ss_list.append(h*c_interleave + i*g_nof_weights + j)
            
    if c_ena_pre_transpose:
        ss.write_selects(ss_list) 

    # - Enable the block generator
    tc.append_log(3, '>>>')
    tc.append_log(3, '>>> Arm the block generator')
    tc.append_log(3, '>>>')
    tc.append_log(3, '')
    
    toggle = pps.read_ppsh_toggle()
    do_until_ne(pps.read_ppsh_toggle, ms_retry=1000, val=toggle, s_timeout=13600)     
    
    bg.write_enable_pps()

    bsn_out.read_bsn_monitor(0)
    bsn_out.read_bsn_monitor(1)
      
    
    ###############################################################################
    # End
    tc.set_section_id('')
    tc.append_log(3, '')
    tc.append_log(3, '>>>')
    tc.append_log(0, '>>> Test bench result: %s' % tc.get_result())
    tc.append_log(3, '>>>')
    
    sys.exit(tc.get_result())



