-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose:
-- . Simulate a minimal Apertif BF->XC subsystem
-- Description:
-- . FIXME:
--   . 1) First use 1 fn_beamformer and copy its 10G output 3 times and feed that to the XC.
--   . 2) Instantiate 3x fn_beamformer
-- Usage:

LIBRARY IEEE, common_lib, unb1_board_lib, apertif_unb1_fn_bf_emu_lib, apertif_unb1_correlator_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE common_lib.tb_common_pkg.ALL; 

ENTITY tb_apertif_bg_xc IS
END tb_apertif_bg_xc;

ARCHITECTURE tb OF tb_apertif_bg_xc IS

  CONSTANT c_sim             : BOOLEAN := TRUE;

  CONSTANT c_unb_nr          : NATURAL := 0;
  CONSTANT c_node_nr         : NATURAL := 0;
  CONSTANT c_id              : STD_LOGIC_VECTOR(7 DOWNTO 0) := TO_UVEC(c_unb_nr, 5) & TO_UVEC(c_node_nr, 3);

  CONSTANT c_eth_clk_period  : TIME := 40 ns;  -- 25 MHz XO on UniBoard
  CONSTANT c_tr_clk_period   : TIME := 6.4 ns;  
  CONSTANT c_clk_period      : TIME := 5 ns; 
  CONSTANT c_pps_period      : NATURAL := 1000; 
  
  SIGNAL clk                 : STD_LOGIC := '0';
  SIGNAL pps                 : STD_LOGIC := '0';
  SIGNAL pps_rst             : STD_LOGIC := '0';
  SIGNAL tr_clk              : STD_LOGIC := '1';
  SIGNAL eth_clk             : STD_LOGIC := '0';
  
  SIGNAL ID                  : STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0) := c_id;

  TYPE t_emu_si_fn_0_tr_arr IS ARRAY (2 DOWNTO 0) OF STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);  
  SIGNAL emu_si_fn_0_tx_arr  : t_emu_si_fn_0_tr_arr;
  SIGNAL emu_si_fn_0_rx_arr  : t_emu_si_fn_0_tr_arr;

BEGIN

  ----------------------------------------------------------------------------
  -- Clocks and PPS
  ----------------------------------------------------------------------------
  clk     <= NOT clk AFTER c_clk_period/2; 
  eth_clk <= NOT eth_clk AFTER c_eth_clk_period/2;
  tr_clk  <= NOT tr_clk  AFTER c_tr_clk_period/2;
  proc_common_gen_pulse(1, c_pps_period, '1', pps_rst, clk, pps);
  
  ------------------------------------------------------------------------------
  -- 3x apertif_unb1_fn_bf_emu
  ------------------------------------------------------------------------------
  gen_apertif_unb1_fn_bf_emu : FOR i IN 0 TO 2 GENERATE
    u_apertif_unb1_fn_bf_emu : ENTITY apertif_unb1_fn_bf_emu_lib.apertif_unb1_fn_bf_emu
      GENERIC MAP (
        g_sim         => c_sim,
        g_sim_unb_nr  => c_unb_nr,
        g_sim_node_nr => i
      )
      PORT MAP (
        CLK         => clk,
        PPS         => pps,
        VERSION     => "00",
        ID          => ID,
  
        ETH_clk     => eth_clk,
        ETH_SGIN    => '0',
        
        SA_CLK      => tr_clk,
    
        SI_FN_0_TX  => emu_si_fn_0_tx_arr(i),
        SI_FN_0_RX  => emu_si_fn_0_rx_arr(i), -- Only connected for OK link status check in emu 10GbE TX

        SI_FN_1_RX  => (OTHERS=>'0'),
        SI_FN_2_RX  => (OTHERS=>'0'),
        SI_FN_3_RX  => (OTHERS=>'0')  
      );  
  END GENERATE;

  ------------------------------------------------------------------------------
  -- apertif_unb1_correlator
  ------------------------------------------------------------------------------
  u_apertif_unb1_correlator : ENTITY apertif_unb1_correlator_lib.apertif_unb1_correlator
    GENERIC MAP (
      g_sim         => c_sim,
      g_sim_unb_nr  => c_unb_nr,
      g_sim_node_nr => c_node_nr,
      g_use_bg      => FALSE
    )
    PORT MAP (
      CLK         => clk,
      PPS         => pps,
      VERSION     => "00",
      ID          => ID,

      ETH_clk     => eth_clk,
      ETH_SGIN    => '0',
      
      SA_CLK      => tr_clk,
      SB_CLK      => tr_clk,

      SI_FN_0_RX  => emu_si_fn_0_tx_arr(0),
      SI_FN_0_TX  => emu_si_fn_0_rx_arr(0),
      SI_FN_1_RX  => emu_si_fn_0_tx_arr(1),
      SI_FN_1_TX  => emu_si_fn_0_rx_arr(1),
      SI_FN_2_RX  => emu_si_fn_0_tx_arr(2),
      SI_FN_2_TX  => emu_si_fn_0_rx_arr(2),

      SI_FN_3_RX  => (OTHERS=>'0'),

      FN_BN_0_RX  => (OTHERS=>'0'),
      FN_BN_1_RX  => (OTHERS=>'0'),
      FN_BN_2_RX  => (OTHERS=>'0'),
      FN_BN_3_RX  => (OTHERS=>'0')
    );

END tb;
