-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose:
-- . Creates image where the output of the dp_offload_rx is connected to the input of dp_offload_tx 
--   Loopback. 
-- Description:

LIBRARY IEEE, unb1_board_lib;
use IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;

ENTITY compaan_unb1_dp_offload_lb IS
  GENERIC (
    g_design_name  : STRING  := "compaan_unb1_dp_offload_lb"; 
    g_design_note  : STRING  := "loopback"; 
    g_sim          : BOOLEAN := FALSE;                 -- set by     ModelSim
    g_sim_unb_nr   : NATURAL := 0;                     -- set by     ModelSim
    g_sim_node_nr  : NATURAL := 0;                     -- set by     ModelSim
    g_stamp_date   : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time   : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn    : NATURAL := 0   -- SVN revision    -- set by QSF
  );
  PORT (
    -- GENERAL
    CLK           : IN    STD_LOGIC; -- dp_clk is generated by SOPC altpll
    PPS           : IN    STD_LOGIC; 
    WDI           : OUT   STD_LOGIC; 
    INTA          : INOUT STD_LOGIC; 
    INTB          : INOUT STD_LOGIC; 
                                                                                 
    -- Others
    VERSION       : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID            : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO        : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc       : INOUT STD_LOGIC;
    sens_sd       : INOUT STD_LOGIC;
  
    -- 1GbE Control Interface
    ETH_clk       : IN    STD_LOGIC;
    ETH_SGIN      : IN    STD_LOGIC;
    ETH_SGOUT     : OUT   STD_LOGIC
  );
END compaan_unb1_dp_offload_lb;


ARCHITECTURE str OF compaan_unb1_dp_offload_lb IS
                                        
BEGIN

  u_revision : ENTITY work.compaan_unb1_dp_offload 
    GENERIC MAP (
      g_design_name  => g_design_name, 
      g_design_note  => g_design_note, 
      g_sim          => g_sim,          
      g_sim_unb_nr   => g_sim_unb_nr,   
      g_sim_node_nr  => g_sim_node_nr,  
      g_stamp_date   => g_stamp_date,  
      g_stamp_time   => g_stamp_time,  
      g_stamp_svn    => g_stamp_svn   
    )
    PORT MAP(
      -- GENERAL
      CLK            => CLK,   
      PPS            => PPS,   
      WDI            => WDI,   
      INTA           => INTA,  
      INTB           => INTB,  
                                                                                   
      -- Others
      VERSION        => VERSION,   
      ID             => ID,        
      TESTIO         => TESTIO,    
  
      -- I2C Interface to Sensors
      sens_sc        => sens_sc,    
      sens_sd        => sens_sd,    
    
      -- 1GbE Control Interface
      ETH_clk        => ETH_clk,     
      ETH_SGIN       => ETH_SGIN,    
      ETH_SGOUT      => ETH_SGOUT   
    );

END str;
