------------------------------------------------------------------
-- TOP LEVEL                                                      
------------------------------------------------------------------
library IEEE, dp_lib, compaan_lib;                                                     
use IEEE.STD_LOGIC_1164.ALL;          
USE dp_lib.dp_stream_pkg.ALL;                           
                                                                  
entity compaan_design is                                                  
  generic (                                                       
    	BLOCKS_PER_SYNC : natural := 10                             
  );                                                              
  port (                                                                                                                             
    
    -- ST sink
    snk_out         : OUT t_dp_siso;
    snk_in          : IN  t_dp_sosi;
    
    -- ST source
    src_in          : IN  t_dp_siso;
    src_out         : OUT t_dp_sosi;                                 
                                                                  
    TEST_STOP       : out std_logic_vector(2 downto 0 );                
    TEST_ERROR      : out std_logic_vector(2 downto 0 );               
    TEST_FIFO_FULL  : out std_logic_vector(1 downto 0 );           
    TEST_BLOCK_RD   : out std_logic_vector(2 downto 0 );           
    address         : in std_logic_vector(18 downto 0 );                  
    read_data       : out std_logic_vector(31 downto 0 );               
    read_en         : in std_logic;                                       
    write_en        : in std_logic;                                      
    write_data      : in std_logic_vector(31 downto 0 );               
                                                                  
    MM_CLK          : in std_logic;                               
    MM_RST          : in std_logic;                                       
    KPN_CLK         : in std_logic;                                       
    KPN_RST         : in std_logic                                        
  );                                                              
end compaan_design;                                                       
                                                                  
architecture STRUCTURE of compaan_design is


	signal data_in_Data : std_logic_vector(31 downto 0);
	signal data_in_control : std_logic;
	signal data_in_Read : std_logic;
	signal data_in_Exists : std_logic; 	
	signal data_in_SOP : std_logic;
	signal data_in_EOP : std_logic;
	
  signal data_out_Data_c : std_logic_vector(31 downto 0);
  signal data_out_Control_c : std_logic;
	signal data_out_Write_c : std_logic;	
	
  signal data_out_Data : std_logic_vector(31 downto 0);
  signal data_out_Control : std_logic;
	signal data_out_Write : std_logic;
	signal data_out_Full : std_logic;
	signal data_out_SOP : std_logic;
	signal data_out_EOP : std_logic;
	
	signal snk_in_nop : t_dp_sosi;
	                                                                                                                                                                                                                                              
begin                                 

  -- wrapper -> ipcore     		
	data_in_Data <= snk_in.data(31 downto 0);
	snk_out.ready <= data_in_Read;    
	data_in_Exists <= snk_in.valid;    
	data_in_SOP <= snk_in.sop;
	data_in_EOP <= snk_in.eop;
		
	-- ipcore --> wrapper	
	--data_out_Full <= not src_in.ready;
	--src_out.valid <= data_out_Write;
	--src_out.data <= RESIZE_DP_DATA(data_out_Data);
	--src_out.sop <= data_out_SOP;
	--src_out.eop <= data_out_EOP;   
	
	snk_in_nop.valid <= data_out_Write_c;
	snk_in_nop.data <= RESIZE_DP_DATA(data_out_Data_c);   
	data_out_Full <= not src_in.ready;                      		 
                        
  -- Compaan ipcore                                         
  u_compaan_design : ENTITY compaan_lib.ipcore                                                             
  PORT MAP (                                                
                                                            
    data_in_Data        => data_in_Data,                    
    data_in_Control     => data_in_Control,                 
    data_in_Read        => data_in_Read,                    
    data_in_Exists      => data_in_Exists,	                                   
                                                            
    data_out_Data       => data_out_Data_c,                   
    data_out_Control    => data_out_Control_c,                
    data_out_Write      => data_out_Write_c,                  
    data_out_Full       => data_out_Full,	                                     
                                                            
    TEST_STOP           => open,                       
    TEST_ERROR          => open,                      
    TEST_FIFO_FULL      => open,                  
    TEST_BLOCK_RD       => open,                   
    address             => address,                         
    read_data           => read_data,                       
    read_en             => read_en,                         
    write_en            => write_en,                        
    write_data          => write_data,                      
    
    MM_CLK              => MM_CLK,  
    MM_RST              => MM_RST,                                                        
    KPN_CLK             => KPN_CLK,                          
    KPN_RST             => KPN_RST                           
  );                                                        
                                                          
	u_pkg_signals_gen : ENTITY dp_lib.dp_block_gen
	GENERIC MAP (                     
    g_use_src_in        => FALSE,  -- when true use src_in.ready else use snk_in.valid for flow control
    g_nof_data          => BLOCKS_PER_SYNC    -- nof data per block                 
	)    
	port map (
    rst        					=> KPN_RST,
    clk         				=> KPN_CLK,
    -- Streaming sink
    snk_in     => snk_in_nop,
    -- Streaming source
    src_in     => src_in,
    src_out    => src_out,
    -- MM control
    en         => '1'
	);
                                                                  
end architecture STRUCTURE;                                       
