-------------------------------------------------------------------------------
-- ipcore2RTL_ed_1_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity pkg_signals is    
  generic (
    BLOCKS_PER_SYNC	: natural := 10
  );
	port (
		write_in		: in std_logic;
		data_in 	  : in std_logic_vector(31 downto 0);
		control_in  : in std_logic;
		write_out 	: out std_logic;
		data_out 	  : out std_logic_vector(31 downto 0);
		control_out : out std_logic;
		eop_out  	  : out std_logic;
		sop_out		  : out std_logic;
		RST 				: in std_logic;
		CLK 				: in std_logic
	);
end pkg_signals;

architecture BEHAVIOURAL of pkg_signals is

  signal sync_count : natural;
  signal sl_eop : std_logic;
  signal sl_sop : std_logic;  
  
	type state_type is (s_idle, s_count);
	signal state       : state_type;
  
--  TYPE reg_type IS RECORD
--   write_in		: std_logic;
--		data_in 	  : std_logic_vector(31 downto 0);
--		control_in  : std_logic;    
--		sop         : std_logic;
--		eop         : std_logic;
--		sync_count  : natural;
--    state       : state_type;  -- The state machine. 
--  END RECORD;

--SIGNAL r, rin      : reg_type;


begin
 
  sop_out <= sl_sop;
  eop_out <= sl_eop;

  pkg_signals_gen : PROCESS(CLK, RST, write_in)
  begin
    if rising_edge(CLK) then
      if RST = '1' then
        sl_sop <= '0';
        sl_eop <= '0';
        state <= s_idle;
        sync_count <= 0;
      else
        write_out <= write_in;
        data_out <= data_in;
        control_out <= control_in;
        sync: case state is
          when s_idle => 
            sl_sop <= write_in;
            sl_eop <= '0';
            if write_in = '1' then
              state <= s_count; 
              sync_count <= 0;
            end if;
          when s_count =>    
            sl_sop <= '0';
            if write_in = '1' then
							if sync_count < BLOCKS_PER_SYNC - 2 then
								sync_count <= sync_count + 1;
							else
		            sl_eop <= '1';
		            state <= s_idle;
		          end if;
	          end if;
	        when others =>
	          state <= s_idle;
        end case;
      end if;
    end if;
  end process;

-- there is an issue when using the two processes below

--  p_comb : PROCESS(r, RST, write_in)
--    VARIABLE v : reg_type;
--  BEGIN
--   
--    v           := r;
--    
--    v.sop        := '0';  
--    v.eop        := '0';
--    v.write_in	 := write_in;		   
--    v.data_in 	 := data_in; 	  
--    v.control_in := control_in;  
--    
--    CASE v.state IS
--      WHEN s_idle => 
--        if write_in = '1' then    
--          v.sop   := '1';
--          v.state := s_count;  
--        END IF;
--
--      WHEN s_count =>  
--        if write_in = '1' then
--				  if r.sync_count < BLOCKS_PER_SYNC - 2 then
--					  v.sync_count := r.sync_count + 1;
--				  else
--		         v.eop := '1';
--		         v.state := s_idle;
--		         v.sync_count := 0;
--		       end if;
--        end if;
--      WHEN OTHERS =>
--        v.state := s_idle;
--
--    END CASE;
--    
--    IF(RST = '1') THEN 
--      v.sop        := '0';
--      v.eop        := '0';
--      v.state      := s_idle;
--      v.sync_count := 0;
--      v.data_in    := (others => '0');
--      v.write_in   := '0';
--      v.control_in   := '0';      
--    END IF;
--
--    rin <= v;  
--         
--  END PROCESS;
--
--  p_regs : PROCESS(CLK)
--  BEGIN 
--    IF RISING_EDGE(CLK) THEN 
--      r <= rin; 
--    END IF; 
--  END PROCESS;
--
--  sop_out <= r.sop;
--  eop_out <= r.eop;
--
--  data_out    <= r.data_in;
--  write_out 	<= r.write_in;
--	control_out <= r.control_in;

end architecture;                                                           