
-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: 
-- . Test bench compaan_unb1_dp_offload.
-- Description:
-- . The block generator in the design is enabled by default.
-- . Design 'compaan_unb1_dp_offload' requires the 1GbE demux to be set up so
--   received streams are forwarded based on the received destination UDP port.
--   This is done by /tb/python/tc_unb1_board1_dp_offload.py.
-- Usage (manual mode, run compaan_unb1_dp_offload.py for auto mode):
-- . Start ModelSim
-- . lp compaan_unb1_dp_offload
-- . mk compile
-- . double click simulation configuration
-- . as 8
-- . run 2us (wait until MM master did the initial ETH settings before Python ETH access)
-- . in separate console: python tc_unb1_board1_dp_offload.py --unb 0 --fn 0 -r 0:2 --sim
-- . run -a

LIBRARY IEEE, common_lib, unb1_board_lib, i2c_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;

ENTITY tb_compaan_unb1_dp_offload_bg_lb IS
END tb_compaan_unb1_dp_offload_bg_lb;

ARCHITECTURE tb OF tb_compaan_unb1_dp_offload_bg_lb IS

  CONSTANT c_sim             : BOOLEAN := TRUE;

  CONSTANT c_unb1_board_nr   : NATURAL := 0; -- UniBoard 0
  CONSTANT c_node_nr_lcu     : NATURAL := 1; -- FN1
  CONSTANT c_node_nr_dut     : NATURAL := 0; -- FN0
  CONSTANT c_id_lcu          : STD_LOGIC_VECTOR(7 DOWNTO 0) := TO_UVEC(c_unb1_board_nr, c_unb1_board_nof_uniboard_w ) & TO_UVEC(c_node_nr_lcu, c_unb1_board_nof_chip_w);
  CONSTANT c_id_dut          : STD_LOGIC_VECTOR(7 DOWNTO 0) := TO_UVEC(c_unb1_board_nr, c_unb1_board_nof_uniboard_w ) & TO_UVEC(c_node_nr_dut, c_unb1_board_nof_chip_w);
  CONSTANT c_version         : STD_LOGIC_VECTOR(1 DOWNTO 0) := "00";

  CONSTANT c_eth_clk_period  : TIME := 40 ns;
  CONSTANT c_sa_clk_period   : TIME := 6.4 ns;  

  SIGNAL sa_clk              : STD_LOGIC := '1';
  SIGNAL eth_clk             : STD_LOGIC := '0';

  SIGNAL eth_lcu_in      		 : STD_LOGIC;
  SIGNAL eth_lcu_out         : STD_LOGIC;
 
  SIGNAL si_fn_lpbk_0        : STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
  SIGNAL si_fn_lpbk_1        : STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
  SIGNAL si_fn_lpbk_2        : STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
  SIGNAL si_fn_lpbk_3        : STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
 
  SIGNAL VERSION             : STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0) := c_version; 
  SIGNAL ID_lcu          	   : STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0) := c_id_lcu;
  SIGNAL ID_dut          	   : STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0) := c_id_dut;
BEGIN

  ----------------------------------------------------------------------------
  -- Externally generated clocks 
  ----------------------------------------------------------------------------
  eth_clk <= NOT eth_clk AFTER c_eth_clk_period/2;
  sa_clk  <= NOT sa_clk  AFTER c_sa_clk_period/2;
  
  ------------------------------------------------------------------------------
  -- u_lcu
  ------------------------------------------------------------------------------
  u_lcu : ENTITY work.compaan_unb1_dp_offload
    GENERIC MAP (
      g_design_name  => "compaan_unb1_dp_offload_bg",
      g_sim          => c_sim,
      g_sim_unb_nr   => c_unb1_board_nr,
      g_sim_node_nr  => c_node_nr_lcu
    )
    PORT MAP (
      CLK         => '0',
      PPS         => '0',
      VERSION     => VERSION,
      ID          => ID_lcu,

      ETH_clk     => eth_clk,
      ETH_SGIN    => eth_lcu_in,
      ETH_SGOUT   => eth_lcu_out
      );

  ------------------------------------------------------------------------------
  -- u_dut
  ------------------------------------------------------------------------------
  u_dut : ENTITY work.compaan_unb1_dp_offload
    GENERIC MAP ( 
      g_design_name  => "compaan_unb1_dp_offload_lb",    
      g_sim          => c_sim,
      g_sim_unb_nr   => c_unb1_board_nr,
      g_sim_node_nr  => c_node_nr_dut
    )
    PORT MAP (
      CLK         => '0',    
      PPS         => '0',
      VERSION     => VERSION,
      ID          => ID_dut,

      ETH_clk     => eth_clk,
      ETH_SGIN    => eth_lcu_out,
      ETH_SGOUT   => eth_lcu_in
      );

END tb;
