 -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --
 
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CONTROLLER is 
  generic(
    N_STAGES  : natural := 1; -- number of pipeline stages or delay
    BLOCKING  : natural := 0  -- '1'-block the pipeline if there is no input data
  );
  port (
    READ      : out std_logic;
    EXIST     : in  std_logic;
    WRITE     : out std_logic;
    FULL      : in  std_logic;

    ENABLE_EX    : out std_logic_vector(N_STAGES-1 downto 0);
    STALL_FRONT  : in  std_logic_vector(N_STAGES-1 downto 0);
    STALL_BACK   : in  std_logic_vector(N_STAGES-1 downto 0);
	 
	 EXIST_WR     : out std_logic;

    CLK       : in  std_logic;
    RST       : in  std_logic
  );
end CONTROLLER;

architecture RTL of CONTROLLER is
  
  signal p_en          : std_logic_vector(N_STAGES downto 0);  -- Enable signals to the execution pipline stages
  signal blocked       : std_logic;                            -- pipeline is blocked on Write
  signal pipe          : std_logic_vector(N_STAGES downto 0);  -- delay pipeline
  signal execute_pipe  : std_logic_vector(N_STAGES downto 0);  -- reverse signal of pipe
  
  function reverse_any_vector (a: in std_logic_vector)
  return std_logic_vector is variable result: std_logic_vector(a'RANGE);
  alias aa: std_logic_vector(a'REVERSE_RANGE) is a;
  begin
     for i in aa'RANGE loop
         result(i) := aa(i);
     end loop;
     return result;
  end; -- function reverse_any_vector


 begin
  --
  Pipe_Fill: process( CLK, RST )
  begin
     if ( RST = '1' ) then
       pipe <= (others => '0');
     elsif ( rising_edge(CLK) ) then
        --
        BUBBLE_COMPRESS:
        for i in N_STAGES downto 1 loop
            if ( p_en(i)='1' ) then
                pipe(i) <= pipe(i-1) and not STALL_FRONT(i-1);
            end if;
        end loop;
        --
        if ( p_en(0)='1' ) then
            pipe(0) <= EXIST;
        end if;

     end if;
  end process Pipe_Fill;
  
  Pipe_Reverse: process(pipe, RST)
  begin
    if (RST = '1') then
        execute_pipe <= (others => '0');
    else
        execute_pipe <= reverse_any_vector(pipe);
    end if;
  end process Pipe_Reverse;

  --p_en(N_STAGES-1 downto 0)      <= not pipe(N_STAGES-1 downto 0) or  p_en(N_STAGES downto 1);
  p_en(N_STAGES-1 downto 0)      <= p_en(N_STAGES downto 1) and not STALL_BACK(N_STAGES-1 downto 0);
  p_en(N_STAGES)                 <= not pipe(N_STAGES)            or  not FULL;
  --
  WRITE                          <=     pipe(N_STAGES)            and not FULL;
  READ                           <=     EXIST                     and p_en(0);
  ENABLE_EX(N_STAGES-1 downto 0) <=     pipe(N_STAGES-1 downto 0) and p_en(N_STAGES downto 1);

	EXIST_WR <= pipe(N_STAGES);

end RTL;

