-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity register_rf is
 generic (
	C_split_x_rf_address   : std_logic_vector(18 downto 0) := B"0000000000000000000"; -- 0
	C_commit_rf_address   : std_logic_vector(18 downto 0) := B"0000000000000000100" -- 4
 );

port (
	rst  : in std_logic;
	clk  : in std_logic;
	pci_clk  : in std_logic;
	--
	
	-- Interface to PCIe 
	address         : in std_logic_vector(18 downto 0);
	read_data       : out std_logic_vector(31 downto 0);
	read_en         : in std_logic;
	write_en        : in std_logic;
	write_data      : in std_logic_vector(31 downto 0);
	--
	-- Interface to reg split_x
	split_x_rf_read_data       : in std_logic_vector(32-1 downto 0);
	split_x_rf_read_en         : out std_logic;
	split_x_rf_write_en        : out std_logic;
	split_x_rf_write_data      : out std_logic_vector(32-1 downto 0);
	--
 
	-- Interface to reg commit
	commit_rf_read_data       : in std_logic_vector(32-1 downto 0);
	commit_rf_read_en         : out std_logic;
	commit_rf_write_en        : out std_logic;
	commit_rf_write_data      : out std_logic_vector(32-1 downto 0)
 
);
end register_rf;

architecture RTL of register_rf is
 
	signal sl_read_data : std_logic_vector(32-1 downto 0)   := (others=>'0');
 
begin
process (pci_clk,rst)
	begin
		if (rising_edge(pci_clk)) then
			if (rst = '1') then
				split_x_rf_write_en <= '0';
				split_x_rf_read_en  <= '0';
				split_x_rf_write_data  <= (others => '0');
			else

				if ( (address(18 downto 2) = C_split_x_rf_address(18 downto 2)) and write_en = '1') then
					if ( (write_data(32-1 downto 0) >= STD_LOGIC_VECTOR(to_signed(2,32))) and (write_data(32-1 downto 0) <= STD_LOGIC_VECTOR(to_signed(805,32))) ) then
						split_x_rf_write_data <= write_data(32-1 downto 0);
						split_x_rf_write_en <= '1';
					end if;
				else
					split_x_rf_write_en <= '0';
				end if;

				if( (address(18 downto 2) = C_split_x_rf_address(18 downto 2)) and read_en= '1') then
					split_x_rf_read_en <= '1';
				else
					split_x_rf_read_en <= '0';
				end if;
		end if;
	end if;
end process;

process (pci_clk,rst)
	begin
		if (rising_edge(pci_clk)) then
			if (rst = '1') then
				commit_rf_write_en <= '0';
				commit_rf_read_en  <= '0';
				commit_rf_write_data  <= (others => '0');
			else

				if ( (address(18 downto 2) = C_commit_rf_address(18 downto 2)) and write_en = '1') then
						commit_rf_write_data <= write_data(32-1 downto 0);
						commit_rf_write_en <= '1';
				else
					commit_rf_write_en <= '0';
				end if;

				if( (address(18 downto 2) = C_commit_rf_address(18 downto 2)) and read_en= '1') then
					commit_rf_read_en <= '1';
				else
					commit_rf_read_en <= '0';
				end if;
		end if;
	end if;
end process;

process (pci_clk,rst)
begin
  if (rising_edge(pci_clk)) then
     if (rst = '1') then
	  else
	    case  address(18 downto 2)  is
			when (C_split_x_rf_address(18 downto 2) ) =>
			  sl_read_data(32-1 downto 0) <=   split_x_rf_read_data;
			when (C_commit_rf_address(18 downto 2) ) =>
			  sl_read_data(32-1 downto 0) <=   commit_rf_read_data;
			when others => 
				sl_read_data <=   (others => '0');
		end case;
	 end if;
	end if;
end process;

read_data <=sl_read_data;

end RTL;
