-------------------------------------------------------------------------------
-- system.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity invert2rtl_invert is
  port (

    data_in_Data    : in  std_logic_vector(31 downto 0 );
    data_in_Control : in  std_logic;
    data_in_Read    : out std_logic;
    data_in_Exists  : in  std_logic;

    data_out_Data    : out std_logic_vector(31 downto 0 );
    data_out_Control : out std_logic;
    data_out_Write   : out std_logic;
    data_out_Full    : in  std_logic;

    TEST_STOP : out std_logic_vector(2 downto 0 );
    TEST_ERROR : out std_logic_vector(2 downto 0 );
    TEST_FIFO_FULL : out std_logic_vector(1 downto 0 );
    TEST_BLOCK_RD : out std_logic_vector(2 downto 0 );

    KPN_CLK : in std_logic;
    KPN_RST : in std_logic

  );
end invert2rtl_invert;

architecture STRUCTURE of invert2rtl_invert is

  component invert2rtl_hwn_nd_1_ip_wrapper is
    port (
		data_in_Rd : out std_logic;
		data_in_Din : in std_logic_vector(31 downto 0);
		data_in_Exist : in std_logic;
		data_in_CLK : out std_logic;
		data_in_CTRL : in std_logic;

		ND_1OP_1_Wr : out std_logic;
		ND_1OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_1OP_1_Full : in std_logic;
		ND_1OP_1_CLK : out std_logic;
		ND_1OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(10 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component invert2rtl_hwn_nd_2_ip_wrapper is
    port (
		ND_2IP_1_Rd : out std_logic;
		ND_2IP_1_Din : in std_logic_vector(31 downto 0);
		ND_2IP_1_Exist : in std_logic;
		ND_2IP_1_CLK : out std_logic;
		ND_2IP_1_CTRL : in std_logic;

		ND_2OP_1_Wr : out std_logic;
		ND_2OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_2OP_1_Full : in std_logic;
		ND_2OP_1_CLK : out std_logic;
		ND_2OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(10 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component invert2rtl_hwn_nd_3_ip_wrapper is
    port (
		ND_3IP_2_Rd : out std_logic;
		ND_3IP_2_Din : in std_logic_vector(31 downto 0);
		ND_3IP_2_Exist : in std_logic;
		ND_3IP_2_CLK : out std_logic;
		ND_3IP_2_CTRL : in std_logic;

		data_out_Wr : out std_logic;
		data_out_Dout : out std_logic_vector(31 downto 0);
		data_out_Full : in std_logic;
		data_out_CLK : out std_logic;
		data_out_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(10 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component invert2rtl_ed_1_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component invert2rtl_ed_2_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  -- Internal signals

	signal signal_ed_1_out_FSL_M_Control : std_logic;
	signal signal_ed_1_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_1_out_FSL_M_Full : std_logic;
	signal signal_ed_1_out_FSL_M_Write : std_logic;
	signal signal_ed_1_in_FSL_S_Control : std_logic;
	signal signal_ed_1_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_1_in_FSL_S_Exists : std_logic;
	signal signal_ed_1_in_FSL_S_Read : std_logic;
	signal signal_ed_2_out_FSL_M_Control : std_logic;
	signal signal_ed_2_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_2_out_FSL_M_Full : std_logic;
	signal signal_ed_2_out_FSL_M_Write : std_logic;
	signal signal_ed_2_in_FSL_S_Control : std_logic;
	signal signal_ed_2_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_2_in_FSL_S_Exists : std_logic;
	signal signal_ed_2_in_FSL_S_Read : std_logic;

  -- AD HOC Internal signals

	signal signal_hwn_nd_1_STOP : std_logic;
	signal signal_hwn_nd_1_ERROR : std_logic;
	signal signal_hwn_nd_1_BLOCK_RD : std_logic;
	signal signal_hwn_nd_2_STOP : std_logic;
	signal signal_hwn_nd_2_ERROR : std_logic;
	signal signal_hwn_nd_2_BLOCK_RD : std_logic;
	signal signal_hwn_nd_3_STOP : std_logic;
	signal signal_hwn_nd_3_ERROR : std_logic;
	signal signal_hwn_nd_3_BLOCK_RD : std_logic;
	signal signal_ed_1_FIFO_FULL : std_logic;
	signal signal_ed_2_FIFO_FULL : std_logic;
	signal signal_PARAM_DT : std_logic_vector(10 downto 0);
	signal signal_PARAM_LD : std_logic;
	signal signal_PARAMETERS : std_logic_vector(0 downto 0);
	signal signal_PARAMETERS_LD : std_logic;
	signal signal_PARAM_FIFO_FULL : std_logic;
	signal signal_SYNC_NUM : std_logic_vector(9 downto 0);

  -- Hierarchical signals 

	signal I_data_in_Control : std_logic;
	signal I_data_in_Data : std_logic_vector(31 downto 0);
	signal I_data_in_Exists : std_logic;
	signal I_data_in_Read : std_logic;
	signal I_data_out_Control : std_logic;
	signal I_data_out_Data : std_logic_vector(31 downto 0);
	signal I_data_out_Full : std_logic;
	signal I_data_out_Write : std_logic;

	-- Default signals

	signal net_gnd0 : std_logic;
	signal net_gnd16 : std_logic_vector(15 downto 0);

	signal sys_clk_s : std_logic;
	signal sys_rst_s : std_logic;


	-- START the actual definition of a Design

begin

	-- Connect Clock

	sys_clk_s <= KPN_CLK;
	sys_rst_s <= KPN_RST;


	-- Connect Hiercical Interconnections

  -- Hierarchical signals -3- compaandesign.com:invert2rtl:design_invert:1:data_in
	I_data_in_Data <= data_in_Data;
	I_data_in_Control <= data_in_Control;
	data_in_Read <= I_data_in_Read;
	I_data_in_Exists <= data_in_Exists;
  -- Hierarchical signals -3- compaandesign.com:invert2rtl:design_invert:1:data_out
	data_out_Data <= I_data_out_Data;
	data_out_Control <= I_data_out_Control;
	data_out_Write <= I_data_out_Write;
	I_data_out_Full <= data_out_Full;

	-- AD HOC EXTERNAL CONNECTIONS 
 -- DESIGN ID : compaandesign.com:invert2rtl:design_invert:1
	TEST_STOP(0) <= signal_hwn_nd_1_STOP;
	TEST_ERROR(0) <= signal_hwn_nd_1_ERROR;
	TEST_BLOCK_RD(0) <= signal_hwn_nd_1_BLOCK_RD;
	TEST_STOP(1) <= signal_hwn_nd_2_STOP;
	TEST_ERROR(1) <= signal_hwn_nd_2_ERROR;
	TEST_BLOCK_RD(1) <= signal_hwn_nd_2_BLOCK_RD;
	TEST_STOP(2) <= signal_hwn_nd_3_STOP;
	TEST_ERROR(2) <= signal_hwn_nd_3_ERROR;
	TEST_BLOCK_RD(2) <= signal_hwn_nd_3_BLOCK_RD;
	TEST_FIFO_FULL(0) <= signal_ed_1_FIFO_FULL;
	TEST_FIFO_FULL(1) <= signal_ed_2_FIFO_FULL;

-- Give default signals, default values

	net_gnd0 <= '0';
	net_gnd16(15 downto 0) <= B"0000000000000000";

-- Instanciate the wrappers (HWN and Edges)


  invert2rtl_hwn_nd_1_ip : invert2rtl_hwn_nd_1_ip_wrapper
	port map (
		data_in_Rd => I_data_in_Read,
		data_in_Din => I_data_in_Data(31 downto 0),
		data_in_Exist => I_data_in_Exists,
		data_in_CLK => open,
		data_in_CTRL => I_data_in_Control,
		ND_1OP_1_Wr => signal_ed_1_out_FSL_M_Write,
		ND_1OP_1_Dout(31 downto 0) => signal_ed_1_out_FSL_M_Data(0 to 31),
		ND_1OP_1_Full => signal_ed_1_out_FSL_M_Full,
		ND_1OP_1_CLK => open,
		ND_1OP_1_CTRL => signal_ed_1_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_1_STOP,
		ERROR => signal_hwn_nd_1_ERROR,
		BLOCK_RD => signal_hwn_nd_1_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  invert2rtl_hwn_nd_2_ip : invert2rtl_hwn_nd_2_ip_wrapper
	port map (
		ND_2IP_1_Rd => signal_ed_1_in_FSL_S_Read,
		ND_2IP_1_Din(31 downto 0) => signal_ed_1_in_FSL_S_Data(0 to 31),
		ND_2IP_1_Exist => signal_ed_1_in_FSL_S_Exists,
		ND_2IP_1_CLK => open,
		ND_2IP_1_CTRL => signal_ed_1_in_FSL_S_Control,
		ND_2OP_1_Wr => signal_ed_2_out_FSL_M_Write,
		ND_2OP_1_Dout(31 downto 0) => signal_ed_2_out_FSL_M_Data(0 to 31),
		ND_2OP_1_Full => signal_ed_2_out_FSL_M_Full,
		ND_2OP_1_CLK => open,
		ND_2OP_1_CTRL => signal_ed_2_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_2_STOP,
		ERROR => signal_hwn_nd_2_ERROR,
		BLOCK_RD => signal_hwn_nd_2_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  invert2rtl_hwn_nd_3_ip : invert2rtl_hwn_nd_3_ip_wrapper
	port map (
		ND_3IP_2_Rd => signal_ed_2_in_FSL_S_Read,
		ND_3IP_2_Din(31 downto 0) => signal_ed_2_in_FSL_S_Data(0 to 31),
		ND_3IP_2_Exist => signal_ed_2_in_FSL_S_Exists,
		ND_3IP_2_CLK => open,
		ND_3IP_2_CTRL => signal_ed_2_in_FSL_S_Control,
		data_out_Wr => I_data_out_Write,
		data_out_Dout => I_data_out_Data(31 downto 0),
		data_out_Full => I_data_out_Full,
		data_out_CLK => open,
		data_out_CTRL => I_data_out_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_3_STOP,
		ERROR => signal_hwn_nd_3_ERROR,
		BLOCK_RD => signal_hwn_nd_3_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  invert2rtl_ed_1 : invert2rtl_ed_1_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_1_out_FSL_M_Data,
		FSL_M_Control => signal_ed_1_out_FSL_M_Control,
		FSL_M_Write => signal_ed_1_out_FSL_M_Write,
		FSL_M_Full => signal_ed_1_out_FSL_M_Full,
		FSL_S_Data => signal_ed_1_in_FSL_S_Data,
		FSL_S_Control => signal_ed_1_in_FSL_S_Control,
		FSL_S_Read => signal_ed_1_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_1_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_1_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  invert2rtl_ed_2 : invert2rtl_ed_2_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_2_out_FSL_M_Data,
		FSL_M_Control => signal_ed_2_out_FSL_M_Control,
		FSL_M_Write => signal_ed_2_out_FSL_M_Write,
		FSL_M_Full => signal_ed_2_out_FSL_M_Full,
		FSL_S_Data => signal_ed_2_in_FSL_S_Data,
		FSL_S_Control => signal_ed_2_in_FSL_S_Control,
		FSL_S_Read => signal_ed_2_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_2_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_2_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

end architecture STRUCTURE;
