  -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;	

use work.hw_node_pkg.all;

entity PARAMETERS is
   generic (
      PAR_WIDTH     : natural;
      PAR_BITWIDTH  : natural;
      PAR_VECTOR    : t_par_vector;
      N_PAR					: natural
   );
   port (
      RST        : in  std_logic;
      CLK        : in  std_logic;
      HALT       : out std_logic;
      HALTED     : in  std_logic; 
      PARAM_DT   : in  std_logic_vector(PAR_WIDTH-1 downto 0);
      PARAM_LD   : in  std_logic;
      PARAMETERS : out std_logic_vector(PAR_BITWIDTH-1 downto 0)
   );
end PARAMETERS;

architecture RTL of PARAMETERS is 

   --constant N_PAR : natural := PAR_VECTOR'Length-2; -- The last two elements in PAR_VECTOR are always a dummy component
	 -- we will always read all the paramerets in parallel
	 --constant N_PAR : natural := 1;

   signal sl_tmp_parameters : std_logic_vector(N_PAR*PAR_WIDTH-1 downto 0);
   signal sl_update : std_logic;
   signal sl_halt   : std_logic;
   signal sl_halted : std_logic;
   signal sl_det_0, sl_det_1, sl_PARAM_LD : std_logic;
   
   type state_type is (s_idle, s_load, s_halt, s_update);
   signal state : state_type;

begin

   HALT      <= sl_halt; 
   sl_halted <= sl_halt and HALTED;
   
   GenLabel1 : if N_PAR > 0 generate
      -- Rising_edge detection of PARAM_LD signal --------------------
      Edge_det_prcss : process(CLK)
      begin
         if falling_edge( CLK ) then
            sl_det_0 <= PARAM_LD;
            sl_det_1 <= sl_det_0;
         end if;
      end process;

      sl_PARAM_LD <= sl_det_0 and not(sl_det_1);

      -- Load the parameters into a temp buffer ----------------------
      GenLabel2 : if( N_PAR > 1) generate
         shift_reg: process (CLK, RST)
         begin
            if rising_edge(CLK) then  -- shift LEFT register
               if( sl_PARAM_LD='1' ) then

                  sl_tmp_parameters(PAR_WIDTH-1 downto 0) <= PARAM_DT;
                  sl_tmp_parameters(N_PAR*PAR_WIDTH-1 downto PAR_WIDTH) <= sl_tmp_parameters((N_PAR-1)*PAR_WIDTH-1 downto 0);

               end if;
            end if;
         end process;
      end generate;

      GenLable3 : if( N_PAR = 1) generate
         shift_reg: process (CLK, RST)
         begin
            if rising_edge(CLK) then  
               if( sl_PARAM_LD='1' ) then
                  sl_tmp_parameters(PAR_WIDTH-1 downto 0) <= PARAM_DT;
               end if;
            end if;
         end process;
      end generate;

      -- Update the parameters (from the temp buffer) ----------------
      Ld_ctrl_prcss : process(CLK, RST)
         variable b : integer := 0;
      begin
         if rising_edge(CLK) then
            b := 0;
            for i in 0 to N_PAR-1 loop
                if( RST='1' ) then
                      --PARAMETERS(i*PAR_WIDTH-1 downto (i-1)*PAR_WIDTH) <= CONV_STD_LOGIC_VECTOR(PAR_VALUES(i-1),PAR_WIDTH);       
                      --PARAMETERS(PAR_VECTOR(i).bitwidth+b-1 downto b) <= CONV_STD_LOGIC_VECTOR(PAR_VECTOR(i).val_def,PAR_VECTOR(i).bitwidth);
                      PARAMETERS(PAR_VECTOR(i).bitwidth+b-1 downto b) <= STD_LOGIC_VECTOR(to_signed(PAR_VECTOR(i).val_def,PAR_VECTOR(i).bitwidth));
                else
                   if( sl_update='1' ) then
                      PARAMETERS <= sl_tmp_parameters;
                      --PARAMETERS(PAR_VECTOR(i).bitwidth+b-1 downto b) <= sl_tmp_parameters(i*PAR_WIDTH+PAR_VECTOR(i).bitwidth-1 downto i*PAR_WIDTH);
                   end if;
                end if;
                b := b + PAR_VECTOR(i).bitwidth;
            end loop;
         end if;
      end process;
      
      
      
      
      FSM: process(CLK, RST)
         variable cntr : natural;
      begin
         if rising_edge(CLK) then
            if( RST='1' ) then
                state <= s_idle;
                cntr  := 0;
            else
                case (state) is
                    when s_idle   => if (sl_PARAM_LD='1') then 
                                        if (cntr>=N_PAR-1)  then 
                                            cntr  := 0;
                                            state <= s_load;   
                                        else
                                            cntr  := cntr + 1;
                                        end if;                         
                                     end if;
                    when s_load   => if (sl_PARAM_LD='0') then state <= s_halt;   end if;
                    when s_halt   => if (sl_halted='1'  ) then state <= s_update; end if;
                    when s_update =>                           state <= s_idle;
                    when others   =>                           state <= s_idle;
                end case;
            end if;
         end if;
      end process;

      sl_halt   <= '1' when (state = s_halt  ) else '0';
      sl_update <= '1' when (state = s_update) else '0';


   end generate; -- GenLabel1

end RTL;
