 -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity read_mux is
   generic(
      N_PORTS    : natural := 1;
      PORT_WIDTH : natural := 32
   );
   port(
      IN_PORTS   : in  std_logic_vector(N_PORTS*PORT_WIDTH-1 downto 0);
      EXISTS     : in  std_logic_vector(N_PORTS-1 downto 0);
      READS      : out std_logic_vector(N_PORTS-1 downto 0);
      SOFS       : in  std_logic_vector(N_PORTS-1 downto 0);

      OUT_PORT   : out std_logic_vector(PORT_WIDTH-1 downto 0);
      EXIST      : out std_logic;
      READ       : in  std_logic;
      SOF        : in  std_logic;
      
      READ_EN    : in  std_logic;
      READ_ST    : out std_logic;
      --CLK        : in  std_logic;

      RELEASE    : in  std_logic_vector(N_PORTS-1 downto 0);
      OBTAIN     : in  std_logic_vector(N_PORTS-1 downto 0);
      CONTROL    : in  std_logic_vector(N_PORTS-1 downto 0)
   );
end read_mux;

architecture RTL of read_mux is
   signal sl_read  : std_logic;
   signal sl_exist : std_logic;
   signal TMP      : std_logic_vector(PORT_WIDTH-1 downto 0);
begin
   EXIST   <= READ_EN and sl_exist;
   sl_read <= READ_EN and READ;
   READ_ST <= sl_read and sl_exist;

   DEMUX_GEN : for i in 0 to N_PORTS-1 generate
       -- 
       -- READS(i) <= EXISTS(i) and CONTROL(i) and RELEASE(i) and sl_read and (not SOFS(i) or SOF);
       -- The Reorder work showed problems as a result of SOFS signal. For now switched off until
       -- its use becomes clear again. BK 31mar14
       --
      READS(i) <= EXISTS(i) and CONTROL(i) and RELEASE(i) and sl_read;
   end generate;

   MUX_DATA :  process(CONTROL, IN_PORTS, OBTAIN, RELEASE, TMP)
   begin
     OUT_PORT <= IN_PORTS(PORT_WIDTH-1 downto 0);
     for i in 0 to N_PORTS-1 loop
        if( CONTROL(i) = '1') then
        
	        if( OBTAIN(i) = '1' and RELEASE(i) = '0') then
					OUT_PORT <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);     
					TMP <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);     -- keep a copy of the value
	        end if ;
	        
			  if( OBTAIN(i) = '0' and RELEASE(i) = '0') then
					OUT_PORT <= TMP;        
	        end if ;	

			  if( OBTAIN(i) = '0' and RELEASE(i) = '1') then
					OUT_PORT <= TMP;        
	        end if ;	
	
	        if( OBTAIN(i) = '1' and RELEASE(i) = '1') then
	        		--assert false
               --     report "Phase in which OBTAIN and RELEASE are both 1 should never be reached"
               --     severity WARNING;    
					OUT_PORT <= IN_PORTS((i+1)*PORT_WIDTH-1 downto (i)*PORT_WIDTH);   
	        end if ;
		end if;        
     end loop;
   end process;

   MUX_EXIST :  process(EXISTS, READ, CONTROL)
   begin
      sl_exist <= '0';
      for i in 0 to N_PORTS-1 loop
         if( CONTROL(i) = '1'  ) then
            sl_exist <= EXISTS(i);
         end if;
      end loop;
   end process;
   
   
   -- Checks. For simulation only
   process(CONTROL)
      variable c : integer;
   begin
      c := 0;
      for i in 0 to CONTROL'Length-1 loop
         if (CONTROL(i)='1') then
            c := c + 1;
         end if;
      end loop;
      assert (c <= 1)
         report "Signal CONTROL contains more than one bit that is set to '1' ! The CONTROL signal must be one-hot encoded. "
         severity ERROR;
   end process;

end RTL;