------------------------------------------------------------------------------
--
-- Copyright (C) 2011
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb1_board_lib, dp_lib, eth_lib, tech_tse_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;

ENTITY unb1_fn_terminal_db IS
  GENERIC (
    -- General
    g_rev_multi_unb : BOOLEAN := FALSE; -- Set to TRUE by Quartus revision fn_terminal_db_rev_multi.
    g_design_name   : STRING  := "unb1_fn_terminal_db";  -- Set to "fn_terminal_db_rev_multi_unb" by revision fn_terminal_db_rev_multi
    g_design_note   : STRING  := "UNUSED";
    g_sim           : BOOLEAN := FALSE; --Overridden by TB
    g_sim_unb_nr    : NATURAL := 0;
    g_sim_node_nr   : NATURAL := 0;
    g_stamp_date    : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time    : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn     : NATURAL := 0   -- SVN revision    -- set by QSF
  );
  PORT (
   -- GENERAL
    CLK                    : IN    STD_LOGIC; -- System Clock
    PPS                    : IN    STD_LOGIC; -- System Sync
    WDI                    : OUT   STD_LOGIC; -- Watchdog Clear
    INTA                   : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB                   : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION                : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID                     : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO                 : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc                : INOUT STD_LOGIC;
    sens_sd                : INOUT STD_LOGIC;

    -- 1GbE Control Interface
    ETH_clk                : IN    STD_LOGIC;
    ETH_SGIN               : IN    STD_LOGIC;
    ETH_SGOUT              : OUT   STD_LOGIC;
    
    -- Transceiver clocks
    --SA_CLK                 : IN  STD_LOGIC := '0';  -- TR clock    BN-BI (backplane)
    SB_CLK                 : IN  STD_LOGIC := '0';  -- TR clock FN-BN    (mesh)

    -- Serial I/O
    FN_BN_0_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_0_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_1_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_1_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_2_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_2_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_3_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_3_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0')
  );
END unb1_fn_terminal_db;


ARCHITECTURE str OF unb1_fn_terminal_db IS
  
  CONSTANT c_use_phy                : t_c_unb1_board_use_phy := (1, 0, 1, 0, 0, 0, 0, 1);         
  CONSTANT c_fw_version             : t_unb1_board_fw_version := (1, 0);  -- firmware version x.y 
   
  CONSTANT c_use_mesh               : BOOLEAN := c_use_phy.tr_mesh=1; 
  CONSTANT c_mesh_mon_select        : NATURAL := 1;     -- > 0 = enable SOSI data buffers monitor via MM
  CONSTANT c_mesh_mon_nof_words     : NATURAL := c_unb1_board_peripherals_mm_reg_default.ram_diag_db_buf_size;  -- = 1024
  CONSTANT c_mesh_mon_use_sync      : BOOLEAN := TRUE;  -- when TRUE use dp_pps to trigger the data buffer capture, else new data capture after read access of last data word

  CONSTANT c_reg_diag_db_adr_w      : NATURAL := 5;

  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  SIGNAL xo_clk                     : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_locked                  : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  SIGNAL cal_clk                    : STD_LOGIC;

  SIGNAL dp_rst                     : STD_LOGIC;
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_pps                     : STD_LOGIC;

  SIGNAL this_chip_id               : STD_LOGIC_VECTOR(c_unb1_board_nof_chip_w-1 DOWNTO 0);  -- [2:0], so range 0-3 for FN and range 4-7 BN

  -- PIOs
  SIGNAL pout_debug_wave            : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL pout_wdi                   : STD_LOGIC;
  SIGNAL pin_pps                    : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL pin_intab                  : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL pout_intab                 : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);

  -- WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;

  -- UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;  
  
  -- UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;  -- mms_unb_sens registers
  SIGNAL reg_unb_sens_miso          : t_mem_miso;

  -- eth1g
  SIGNAL eth1g_tse_clk              : STD_LOGIC;
  SIGNAL eth1g_mm_rst               : STD_LOGIC;
  SIGNAL eth1g_tse_mosi             : t_mem_mosi := c_mem_mosi_rst;  -- ETH TSE MAC registers
  SIGNAL eth1g_tse_miso             : t_mem_miso;
  SIGNAL eth1g_reg_mosi             : t_mem_mosi := c_mem_mosi_rst;  -- ETH control and status registers
  SIGNAL eth1g_reg_miso             : t_mem_miso;
  SIGNAL eth1g_reg_interrupt        : STD_LOGIC;   -- Interrupt
  SIGNAL eth1g_ram_mosi             : t_mem_mosi := c_mem_mosi_rst;  -- ETH rx frame and tx frame memory
  SIGNAL eth1g_ram_miso             : t_mem_miso;

  -- tr_mesh
  SIGNAL tx_serial_2arr             : t_unb1_board_mesh_sl_2arr;    -- Tx
  SIGNAL rx_serial_2arr             : t_unb1_board_mesh_sl_2arr;    -- Rx support for diagnostics
    
  -- MM tr_nonbonded with diagnostics
  SIGNAL reg_tr_nonbonded_mosi      : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_tr_nonbonded_miso      : t_mem_miso;

  SIGNAL reg_diagnostics_mosi       : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_diagnostics_miso       : t_mem_miso;

  -- MM diag_data_buffer_mesh
  SIGNAL ram_mesh_diag_data_buf_mosi : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_mesh_diag_data_buf_miso : t_mem_miso;

  -- MM diag_data_buffer (main)
  SIGNAL ram_diag_data_buf_mosi     : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_diag_data_buf_miso     : t_mem_miso;
  SIGNAL reg_diag_data_buf_mosi     : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_diag_data_buf_miso     : t_mem_miso;  
  
  -- MM bsn_monitor
  SIGNAL reg_bsn_monitor_mosi       : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_bsn_monitor_miso       : t_mem_miso;

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb1_board_lib.ctrl_unb1_board
  GENERIC MAP (
    g_sim           => g_sim,
    g_design_name   => g_design_name,
    g_design_note   => g_design_note,
    g_stamp_date    => g_stamp_date,
    g_stamp_time    => g_stamp_time, 
    g_stamp_svn     => g_stamp_svn, 
    g_fw_version    => c_fw_version,
    g_mm_clk_freq   => c_unb1_board_mm_clk_freq_50M,
    g_use_phy       => c_use_phy,
    g_aux           => c_unb1_board_aux
  )
  PORT MAP (
    -- Clock an reset signals
    cs_sim                   => cs_sim,
    xo_clk                   => xo_clk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_locked                => mm_locked,
    mm_rst                   => mm_rst,

    dp_rst                   => dp_rst,
    dp_clk                   => dp_clk,
    dp_pps                   => OPEN,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,
    
    -- Toggle WDI
    pout_wdi                 => pout_wdi,    
    pin_pps                  => pin_pps,

    -- MM buses
    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- eth1g
    eth1g_tse_clk            => eth1g_tse_clk,   -- 125 MHz from xo_clk PLL in SOPC system
    eth1g_mm_rst             => eth1g_mm_rst,
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,

    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    sens_sc                  => sens_sc,
    sens_sd                  => sens_sd,        
    -- . 1GbE Control Interface
    ETH_clk                  => ETH_clk,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- MM master
  -----------------------------------------------------------------------------
  u_mmm : ENTITY work.mmm_unb1_fn_terminal_db
  GENERIC MAP (
    g_sim         => g_sim,
    g_sim_unb_nr  => g_sim_unb_nr,
    g_sim_node_nr => g_sim_node_nr

   )
  PORT MAP(  
    xo_clk                      => xo_clk,       
    xo_rst_n                    => xo_rst_n,     
    xo_rst                      => xo_rst,       
                                
    mm_rst                      => mm_rst,
    mm_clk                      => mm_clk,       
    mm_locked                   => mm_locked,  
    cal_clk                     => cal_clk,  
                                
    -- PIOs                     
    pout_wdi                    => pout_wdi,   
    pin_pps                     => pin_pps,
                                
    -- Manual WDI override      
    reg_wdi_mosi                => reg_wdi_mosi,
    reg_wdi_miso                => reg_wdi_miso,
                                
    -- system_info              
    reg_unb_system_info_mosi    => reg_unb_system_info_mosi,
    reg_unb_system_info_miso    => reg_unb_system_info_miso,
    rom_unb_system_info_mosi    => rom_unb_system_info_mosi,
    rom_unb_system_info_miso    => rom_unb_system_info_miso, 
                                
    -- UniBoard I2C sensors     
    reg_unb_sens_mosi           => reg_unb_sens_mosi,
    reg_unb_sens_miso           => reg_unb_sens_miso, 
 
    -- eth1g
    eth1g_tse_clk               => eth1g_tse_clk,
    eth1g_mm_rst                => eth1g_mm_rst,
    eth1g_tse_mosi              => eth1g_tse_mosi,
    eth1g_tse_miso              => eth1g_tse_miso,
    eth1g_reg_mosi              => eth1g_reg_mosi,
    eth1g_reg_miso              => eth1g_reg_miso,
    eth1g_reg_interrupt         => eth1g_reg_interrupt,
    eth1g_ram_mosi              => eth1g_ram_mosi,
    eth1g_ram_miso              => eth1g_ram_miso,
                                
    -- . tr_nonbonded           
    reg_tr_nonbonded_mosi       => reg_tr_nonbonded_mosi,  
    reg_tr_nonbonded_miso       => reg_tr_nonbonded_miso,  
    reg_diagnostics_mosi        => reg_diagnostics_mosi,   
    reg_diagnostics_miso        => reg_diagnostics_miso,   
                                   
    -- . diag_data_buffer          
    ram_diag_data_buf_mosi      => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso      => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi      => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso      => reg_diag_data_buf_miso,
    
    -- . diag_data_buffer_mesh
    ram_mesh_diag_data_buf_mosi => ram_mesh_diag_data_buf_mosi,
    ram_mesh_diag_data_buf_miso => ram_mesh_diag_data_buf_miso,
    
    -- . bsn_monitor
    reg_bsn_monitor_mosi        => reg_bsn_monitor_mosi,
    reg_bsn_monitor_miso        => reg_bsn_monitor_miso   

  );

  -----------------------------------------------------------------------------
  -- Node functioon: Terminals and data buffer
  -----------------------------------------------------------------------------  
  u_node_unb1_fn_terminal_db : ENTITY unb1_board_lib.node_unb1_fn_terminal_db
  GENERIC MAP(
    g_multi_unb                 => g_rev_multi_unb,
    -- Terminals interface
    g_use_mesh                  => c_use_mesh,
    g_mesh_mon_select           => c_mesh_mon_select,
    g_mesh_mon_nof_words        => c_mesh_mon_nof_words,
    g_mesh_mon_use_sync         => c_mesh_mon_use_sync,
    -- Auxiliary Interface
    g_aux                       => c_unb1_board_aux
  )
  PORT MAP(
    -- System
    mm_rst                      => mm_rst,
    mm_clk                      => mm_clk,
    dp_rst                      => dp_rst,
    dp_clk                      => dp_clk,
    dp_pps                      => dp_pps,
    tr_mesh_clk                 => SB_CLK,
    cal_clk                     => cal_clk,

    chip_id                     => this_chip_id,

    -- MM interface
    -- . tr_nonbonded
    reg_tr_nonbonded_mosi       => reg_tr_nonbonded_mosi,
    reg_tr_nonbonded_miso       => reg_tr_nonbonded_miso,
    reg_diagnostics_mosi        => reg_diagnostics_mosi,
    reg_diagnostics_miso        => reg_diagnostics_miso,
    -- . diag_data_buffer
    ram_diag_data_buf_mosi      => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso      => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi      => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso      => reg_diag_data_buf_miso,
    -- . diag_data_buffer_mesh
    ram_mesh_diag_data_buf_mosi => ram_mesh_diag_data_buf_mosi,
    ram_mesh_diag_data_buf_miso => ram_mesh_diag_data_buf_miso,
    -- . bsn_monitor
    reg_bsn_monitor_mosi        => reg_bsn_monitor_mosi,
    reg_bsn_monitor_miso        => reg_bsn_monitor_miso,
    
    -- Mesh interface
    tx_serial_2arr              => tx_serial_2arr,
    rx_serial_2arr              => rx_serial_2arr
  );

  -----------------------------------------------------------------------------
  -- Mesh I/O
  -----------------------------------------------------------------------------  
  no_tr_mesh : IF c_use_phy.tr_mesh=0 GENERATE
    rx_serial_2arr <= (OTHERS=>(OTHERS=>'0'));
  END GENERATE;
  
  gen_tr_mesh : IF c_use_phy.tr_mesh/=0 GENERATE
    u_mesh_io : ENTITY unb1_board_lib.unb1_board_mesh_io
    GENERIC MAP (
      g_bus_w => c_unb1_board_tr_mesh.bus_w
    )
    PORT MAP (
      tx_serial_2arr => tx_serial_2arr,
      rx_serial_2arr => rx_serial_2arr,
      
      -- Serial I/O
      FN_BN_0_TX     => FN_BN_0_TX,
      FN_BN_0_RX     => FN_BN_0_RX,
      FN_BN_1_TX     => FN_BN_1_TX,
      FN_BN_1_RX     => FN_BN_1_RX,
      FN_BN_2_TX     => FN_BN_2_TX,
      FN_BN_2_RX     => FN_BN_2_RX,
      FN_BN_3_TX     => FN_BN_3_TX,
      FN_BN_3_RX     => FN_BN_3_RX
    );
  END GENERATE;
  
END;












