-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: 
--
-- Description:
--
-- Remarks:
--
--   
-- Some more remarks:

LIBRARY IEEE, common_lib, dp_lib, unb1_board_lib, diag_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;

ENTITY node_unb1_terminal_bg_mesh_db IS
  GENERIC(
    g_sim                     : BOOLEAN := FALSE;
    g_sim_level               : NATURAL := 0; 
    
    -- BLOCK GENERATOR
    g_use_bg                  : BOOLEAN := TRUE; 
    
    -- MESH TERMINAL
    -- System
    g_node_type               : t_e_unb1_board_node := e_any;   -- or e_fn, or e_bn
    g_nof_bus                 : NATURAL := 4;      -- one bus to each of the 4 nodes on the other side of the mesh
    -- User
    g_usr_use_complex         : BOOLEAN := FALSE;  -- when TRUE transport sosi im & re fields via DP data, else transport sosi data via DP data
    g_usr_data_w              : NATURAL := 16;     -- <= 32, to avoid need for DP packet data packing and to fit on the tr_nonbonded PHY data width of 32 bit
    g_usr_frame_len           : NATURAL := 20;
    g_usr_nof_streams         : NATURAL := 3;      -- number of user streams per bus
    -- Phy
    g_phy_nof_serial          : NATURAL := 3;      -- up to 4 serial lanes per bus
    g_phy_gx_mbps             : NATURAL := 5000;
    g_phy_rx_fifo_size        : NATURAL := c_bram_m9k_fifo_depth;   -- g_fifos=TRUE in mms_tr_nonbonded, choose to use full BRAM size = 256 for FIFO depth at output from PHY
    g_phy_ena_reorder         : BOOLEAN := TRUE;
    -- Tx
    g_use_tx                  : BOOLEAN := TRUE;
    g_tx_input_use_fifo       : BOOLEAN := TRUE;   -- Tx input uses FIFO to create slack for inserting DP Packet and Uthernet frame headers
    g_tx_input_fifo_size      : NATURAL := c_bram_m9k_fifo_depth;   -- g_tx_input_use_fifo=TRUE, choose to use full BRAM size = 256 for FIFO depth at input to uth_terminal_tx
    g_tx_input_fifo_fill      : NATURAL := 0;
    -- Rx
    g_use_rx                  : BOOLEAN := TRUE;
    g_rx_output_use_fifo      : BOOLEAN := TRUE;   -- Rx output provides FIFOs to ensure that dp_distribute does not get blocked due to substantial backpressure on another output
    g_rx_output_fifo_size     : NATURAL := c_bram_m9k_fifo_depth;   -- g_rx_output_use_fifo, choose to use full BRAM size = 256 for FIFO depth at output of uth_terminal_rx
    g_rx_output_fifo_fill     : NATURAL := 0;
    g_rx_timeout_w            : NATURAL := 0;      -- when 0 then no timeout else when > 0 then flush pending rx payload after 2**g_timeout_w clk cylces of inactive uth_rx snk_in.valid
    -- Monitoring
    g_mon_select              : NATURAL := 0;      -- 0 = no SOSI data buffers monitor via MM
                                                   -- 1 = enable monitor the Rx UTH packets per serial lane after the tr_nonbonded
                                                   -- 2 = enable monitor the Rx UTH packets per serial lane after the mesh reorder
                                                   -- 3 = enable monitor the Rx DP  packets per serial lane after the uth_rx
                                                   -- 4 = enable monitor the Rx DP  packets per user stream after the dp_distribute
                                                   -- 5 = enable monitor the Tx UTH packets per serial lane to the tr_nonbonded
                                                   -- 6 = enable monitor the Tx UTH packets per serial lane to the mesh reorder
    g_mon_nof_words           : NATURAL := 1024;
    g_mon_use_sync            : BOOLEAN := TRUE;
    -- UTH
    g_uth_len_max             : NATURAL := 255;    -- uth_rx g_uth_len_max < uth_tx g_uth_typ_ofs
    g_uth_typ_ofs             : NATURAL := 256;    -- uth_rx g_uth_len_max < uth_tx g_uth_typ_ofs

    -- Auxiliary Interface
    g_aux                     : t_c_unb1_board_aux := c_unb1_board_aux;
    -- BSN ALIGNER 
    g_use_bsn_align           : BOOLEAN := TRUE;  -- default TRUE, support FALSE for packet flow debugging purposes (faster synthesis)
    -- DATA BUFFER
    g_use_data_buf            : BOOLEAN := TRUE 
  );
  PORT(
    -- System
    chip_id                     : IN  STD_LOGIC_VECTOR(g_aux.chip_id_w-1 DOWNTO 0);  -- [2:0]
    
    mm_rst                      : IN  STD_LOGIC;
    mm_clk                      : IN  STD_LOGIC;   -- 50 MHz from xo_clk PLL in SOPC system
    dp_rst                      : IN  STD_LOGIC;
    dp_clk                      : IN  STD_LOGIC;   -- 200 MHz from CLK system clock
    dp_pps                      : IN  STD_LOGIC := '0';
    tr_mesh_clk                 : IN  STD_LOGIC;   -- 156.25 MHz from SB_CLK transceiver clock
    cal_clk                     : IN  STD_LOGIC;   -- 40 MHz from xo_clk PLL in SOPC system

    -- MM interface
    -- . block generator
    reg_diag_bg_mosi            : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_diag_bg_miso            : OUT t_mem_miso; 
    ram_diag_bg_mosi            : IN  t_mem_mosi := c_mem_mosi_rst; 
    ram_diag_bg_miso            : OUT t_mem_miso; 
    -- . diag_data_buffer
    ram_diag_data_buf_mosi      : IN  t_mem_mosi := c_mem_mosi_rst;
    ram_diag_data_buf_miso      : OUT t_mem_miso;
    reg_diag_data_buf_mosi      : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_diag_data_buf_miso      : OUT t_mem_miso;
    -- . tr_nonbonded
    reg_tr_nonbonded_mosi       : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_tr_nonbonded_miso       : OUT t_mem_miso;
    reg_diagnostics_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_diagnostics_miso        : OUT t_mem_miso;
    -- . diag_data_buffer_mesh
    ram_mesh_diag_data_buf_mosi : IN  t_mem_mosi := c_mem_mosi_rst;
    ram_mesh_diag_data_buf_miso : OUT t_mem_miso;
    -- . bsn_monitor
    reg_bsn_monitor_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_bsn_monitor_miso        : OUT t_mem_miso;
   
    -- Datapath User interface (4 nodes)(4 input streams)
    tx_usr_siso_2arr            : OUT t_unb1_board_mesh_siso_2arr;
    tx_usr_sosi_2arr            : IN  t_unb1_board_mesh_sosi_2arr := (OTHERS=>(OTHERS=>c_dp_sosi_rst));  -- Tx
    rx_usr_siso_2arr            : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rdy));
    rx_usr_sosi_2arr            : OUT t_unb1_board_mesh_sosi_2arr;                                       -- Rx

    -- Mesh serial interface (tr_nonbonded)
    tx_serial_2arr              : OUT t_unb1_board_mesh_sl_2arr;                            -- Tx
    rx_serial_2arr              : IN  t_unb1_board_mesh_sl_2arr:= (OTHERS=>(OTHERS=>'0'))   -- Rx support for diagnostics
  );
END node_unb1_terminal_bg_mesh_db;

ARCHITECTURE str OF node_unb1_terminal_bg_mesh_db IS
  
  -----------------------------------------------------------------------------
  -- Block Generator
  -----------------------------------------------------------------------------
  CONSTANT c_use_usr_input           : BOOLEAN := NOT(g_use_bg);
  CONSTANT c_bg_nof_streams          : NATURAL := g_nof_bus * g_usr_nof_streams; -- 4 x 3 = 12
  CONSTANT c_in_dat_w                : NATURAL := g_usr_data_w;
  CONSTANT c_bg_addr_w               : NATURAL := 7;
  CONSTANT c_file_name_prefix        : STRING  := "hex/bg_in_data";
  
  -----------------------------------------------------------------------------
  -- Terminals
  -----------------------------------------------------------------------------
  CONSTANT c_usr_nof_streams_per_bus : NATURAL := g_usr_nof_streams/c_unb1_board_nof_bn;  -- 16 / 4 = 4

  -----------------------------------------------------------------------------
  -- BSN Aligner 
  -----------------------------------------------------------------------------
  CONSTANT c_bsn_align_nof_streams   : NATURAL := c_bg_nof_streams;
  CONSTANT c_block_size              : NATURAL := 128; --FIXME: current fn_beamformer output block size.   
  CONSTANT c_block_period            : NATURAL := 192;
  CONSTANT c_bsn_align_latency       : NATURAL := 3;
  CONSTANT c_bsn_align_xoff_timeout  : NATURAL :=  c_bsn_align_latency * 2  * c_block_period;  -- flush factor 2 longer than needed
  CONSTANT c_bsn_align_sop_timeout   : NATURAL := (c_bsn_align_latency + 1) * c_block_period;  -- wait somewhat more than c_bsn_align_latency periods

  -----------------------------------------------------------------------------
  -- BSN Monitor 
  -----------------------------------------------------------------------------
  CONSTANT c_bsn_mon_nof_streams     : NATURAL := 1;
  CONSTANT c_bsn_sync_time_out       : NATURAL := (800000*256*10)/8;

  -----------------------------------------------------------------------------
  -- Data Buffer
  -----------------------------------------------------------------------------
  CONSTANT c_db_nof_streams          : NATURAL := c_bg_nof_streams; 
  CONSTANT c_db_nof_data             : NATURAL := 128;

  --. Block generator
  SIGNAL bg_snk_out_arr           : t_dp_siso_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bg_snk_in_arr            : t_dp_sosi_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bg_src_in_arr            : t_dp_siso_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bg_src_out_arr           : t_dp_sosi_arr(c_bg_nof_streams-1 DOWNTO 0);

  SIGNAL bg_out_siso_2arr         : t_unb1_board_mesh_siso_2arr;
  SIGNAL bg_out_sosi_2arr         : t_unb1_board_mesh_sosi_2arr;

  --. Mesh terminal
  SIGNAL rx_usr_i_siso_2arr       : t_unb1_board_mesh_siso_2arr;
  SIGNAL rx_usr_i_sosi_2arr       : t_unb1_board_mesh_sosi_2arr;
  
  --. BSN Aligner 
  SIGNAL bsn_align_snk_in_arr     : t_dp_sosi_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bsn_align_snk_out_arr    : t_dp_siso_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bsn_align_src_in_arr     : t_dp_siso_arr(c_bg_nof_streams-1 DOWNTO 0);
  SIGNAL bsn_align_src_out_arr    : t_dp_sosi_arr(c_bg_nof_streams-1 DOWNTO 0);    
  
  --. BSN Monitor
  SIGNAL bsn_mon_snk_out_arr      : t_dp_siso_arr(c_bsn_mon_nof_streams-1 DOWNTO 0);
  SIGNAL bsn_mon_snk_in_arr       : t_dp_sosi_arr(c_bsn_mon_nof_streams-1 DOWNTO 0);
   
BEGIN

  ---------------------------------------------------------------------------------------
  -- From 2d to 1d array. Input port to input BG.
  ---------------------------------------------------------------------------------------
  gen_i_a : FOR I IN 0 TO g_nof_bus-1 GENERATE
    gen_j_a : FOR J IN 0 TO g_usr_nof_streams-1 GENERATE
      bg_snk_in_arr(I*g_usr_nof_streams + J) <= tx_usr_sosi_2arr(I)(J);
      tx_usr_siso_2arr(I)(J)                 <= bg_snk_out_arr(I*g_usr_nof_streams + J);
    END GENERATE;
  END GENERATE;

  ---------------------------------------------------------------------------------------
  -- Block Generator
  ---------------------------------------------------------------------------------------
  u_bg : ENTITY diag_lib.mms_diag_block_gen 
  GENERIC MAP(   
    -- Generate configurations
    g_use_usr_input      => c_use_usr_input,
    g_use_bg             => g_use_bg,
    g_use_tx_seq         => FALSE,
    -- General
    g_nof_streams        => c_bg_nof_streams,
    -- BG settings
    g_use_bg_buffer_ram  => TRUE,
    g_buf_dat_w          => c_nof_complex*c_in_dat_w, 
    g_buf_addr_w         => c_bg_addr_w,        
    g_file_name_prefix   => c_file_name_prefix, 
    -- User input multiplexer option
    g_usr_bypass_xonoff  => FALSE
  )
  PORT MAP(
    -- System
    mm_rst           => mm_rst, 
    mm_clk           => mm_clk, 
    dp_rst           => dp_rst, 
    dp_clk           => dp_clk,
    en_sync          => dp_pps,
    -- MM interface
    reg_bg_ctrl_mosi => reg_diag_bg_mosi, 
    reg_bg_ctrl_miso => reg_diag_bg_miso, 
    ram_bg_data_mosi => ram_diag_bg_mosi, 
    ram_bg_data_miso => ram_diag_bg_miso, 
    
    -- ST interface
    usr_siso_arr     => bg_snk_out_arr,   
    usr_sosi_arr     => bg_snk_in_arr,    
    out_siso_arr     => bg_src_in_arr, 
    out_sosi_arr     => bg_src_out_arr
  );

  ---------------------------------------------------------------------------------------
  -- From 1d to 2d array. Output BG to input Mesh
  ---------------------------------------------------------------------------------------
  gen_i_b : FOR I IN 0 TO g_nof_bus-1 GENERATE
    gen_j_b : FOR J IN 0 TO g_usr_nof_streams-1 GENERATE
      bg_src_in_arr(I*g_usr_nof_streams + J) <= bg_out_siso_2arr(I)(J);
      bg_out_sosi_2arr(I)(J)                 <= bg_src_out_arr(I*g_usr_nof_streams + J);
    END GENERATE;
  END GENERATE;

  u_mesh_terminal : ENTITY unb1_board_lib.unb1_board_terminals_mesh
  GENERIC MAP(
    g_sim                 => g_sim,                
    g_sim_level           => g_sim_level,          
    -- System                                      
    g_node_type           => g_node_type,          
    g_nof_bus             => g_nof_bus,            
    -- User                                        
    g_usr_use_complex     => g_usr_use_complex,    
    g_usr_data_w          => g_usr_data_w,         
    g_usr_frame_len       => g_usr_frame_len,      
    g_usr_nof_streams     => g_usr_nof_streams,    
    -- Phy                                         
    g_phy_nof_serial      => g_phy_nof_serial,     
    g_phy_gx_mbps         => g_phy_gx_mbps,        
    g_phy_rx_fifo_size    => g_phy_rx_fifo_size,   
    g_phy_ena_reorder     => g_phy_ena_reorder,    
    -- Tx
    g_use_tx              => g_use_tx,             
    g_tx_input_use_fifo   => g_tx_input_use_fifo,  
    g_tx_input_fifo_size  => g_tx_input_fifo_size, 
    g_tx_input_fifo_fill  => g_tx_input_fifo_fill, 
    -- Rx
    g_use_rx              => g_use_rx,             
    g_rx_output_use_fifo  => g_rx_output_use_fifo, 
    g_rx_output_fifo_size => g_rx_output_fifo_size,
    g_rx_output_fifo_fill => g_rx_output_fifo_fill,
    g_rx_timeout_w        => g_rx_timeout_w,       
    
    -- Monitoring
    g_mon_select          => g_mon_select,         
    g_mon_nof_words       => g_mon_nof_words,      
    g_mon_use_sync        => g_mon_use_sync,       
    
    -- UTH
    g_uth_len_max         => g_uth_len_max,        
    g_uth_typ_ofs         => g_uth_typ_ofs         
  )
  PORT MAP (
    chip_id                => chip_id, 
    
    mm_rst                 => mm_rst,    
    mm_clk                 => mm_clk,    
    dp_rst                 => dp_rst,    
    dp_clk                 => dp_clk,    
    dp_sync                => dp_pps,   
    tr_clk                 => tr_mesh_clk,    
    cal_clk                => cal_clk,   
    
    -- User interface (4 nodes)(4 input streams)
    tx_usr_siso_2arr       => bg_out_siso_2arr,
    tx_usr_sosi_2arr       => bg_out_sosi_2arr, -- <== Data to the Mesh       
    rx_usr_siso_2arr       => rx_usr_i_siso_2arr,
    rx_usr_sosi_2arr       => rx_usr_i_sosi_2arr, -- ==> Data from the Mesh
    
    -- Serial (tr_nonbonded)
    tx_serial_2arr         => tx_serial_2arr,
    rx_serial_2arr         => rx_serial_2arr,
    
    -- MM Control
    -- . tr_nonbonded
    reg_tr_nonbonded_mosi  => reg_tr_nonbonded_mosi, 
    reg_tr_nonbonded_miso  => reg_tr_nonbonded_miso, 
    reg_diagnostics_mosi   => reg_diagnostics_mosi,  
    reg_diagnostics_miso   => reg_diagnostics_miso,  
    
    -- . monitor data buffer
    ram_diag_data_buf_mosi => ram_mesh_diag_data_buf_mosi,  
    ram_diag_data_buf_miso => ram_mesh_diag_data_buf_miso 
  );

  ---------------------------------------------------------------------------------------
  -- From 2d to 1d array. Input port to input BG.
  ---------------------------------------------------------------------------------------
  gen_i_c : FOR I IN 0 TO g_nof_bus-1 GENERATE
    gen_j_c : FOR J IN 0 TO g_usr_nof_streams-1 GENERATE
      bsn_align_snk_in_arr(I*g_usr_nof_streams + J) <= rx_usr_i_sosi_2arr(I)(J);
      rx_usr_i_siso_2arr(I)(J)                      <= bsn_align_snk_out_arr(I*g_usr_nof_streams + J);
    END GENERATE;
  END GENERATE;

  -----------------------------------------------------------------------------
  -- BSN ALIGNER
  -----------------------------------------------------------------------------       
  gen_bsn_align : IF g_use_bsn_align GENERATE
    u_dp_bsn_align : ENTITY dp_lib.dp_bsn_align
    GENERIC MAP (
      g_block_size           => c_block_size,
      g_block_period         => c_block_period,
      g_nof_input            => c_bsn_align_nof_streams,
      g_xoff_timeout         => c_bsn_align_xoff_timeout,
      g_sop_timeout          => c_bsn_align_sop_timeout,
      g_bsn_latency          => c_bsn_align_latency,
      g_bsn_request_pipeline => 2
    )
    PORT MAP (
      rst         => dp_rst,
      clk         => dp_clk,
    
      snk_out_arr => bsn_align_snk_out_arr,
      snk_in_arr  => bsn_align_snk_in_arr,
    
      src_in_arr  => bsn_align_src_in_arr,
      src_out_arr => bsn_align_src_out_arr
    );
  END GENERATE;   
  
  gen_no_bsn_align : IF NOT(g_use_bsn_align) GENERATE
    bsn_align_src_out_arr <= bsn_align_snk_in_arr;
    bsn_align_snk_out_arr <= bsn_align_src_in_arr;
  END GENERATE;   
  
  -----------------------------------------------------------------------------
  -- BSN monitors at the output of the BSN aligner
  -----------------------------------------------------------------------------
  u_dp_bsn_monitor : ENTITY dp_lib.mms_dp_bsn_monitor
  GENERIC MAP (
    g_nof_streams        => c_bsn_mon_nof_streams,
    g_sync_timeout       => c_bsn_sync_time_out,
    g_log_first_bsn      => TRUE
  )
  PORT MAP (
    mm_rst      => mm_rst,
    mm_clk      => mm_clk,
    reg_mosi    => reg_bsn_monitor_mosi,
    reg_miso    => reg_bsn_monitor_miso,
    
    dp_rst      => dp_rst,
    dp_clk      => dp_clk,
    in_siso_arr => bsn_mon_snk_out_arr,
    in_sosi_arr => bsn_mon_snk_in_arr
  );   
  
  bsn_mon_snk_in_arr  <= bsn_align_src_out_arr(c_bsn_mon_nof_streams -1 DOWNTO 0);
  bsn_mon_snk_out_arr <= bsn_align_src_in_arr(c_bsn_mon_nof_streams -1 DOWNTO 0);

  gen_data_buf : IF g_use_data_buf GENERATE
    u_data_buf : ENTITY diag_lib.mms_diag_data_buffer
    GENERIC MAP (    
      g_nof_streams  => c_db_nof_streams,
      g_data_w       => g_usr_data_w, 
      g_buf_nof_data => c_db_nof_data,
      g_buf_use_sync => TRUE
    )
    PORT MAP (
      -- System
      mm_rst            => mm_rst,
      mm_clk            => mm_clk,
      dp_rst            => dp_rst,
      dp_clk            => dp_clk,
      -- MM interface
      ram_data_buf_mosi => ram_diag_data_buf_mosi,
      ram_data_buf_miso => ram_diag_data_buf_miso,
      reg_data_buf_mosi => reg_diag_data_buf_mosi,
      reg_data_buf_miso => reg_diag_data_buf_miso,
      -- ST interface
      in_sync           => bsn_align_src_out_arr(0).sync,
      in_sosi_arr       => bsn_align_src_out_arr
    );
  END GENERATE;
  
  ---------------------------------------------------------------------------------------
  -- From 1d to 2d array. Output BSN Aligner to port output
  ---------------------------------------------------------------------------------------
  gen_i_d : FOR I IN 0 TO g_nof_bus-1 GENERATE
    gen_j_d : FOR J IN 0 TO g_usr_nof_streams-1 GENERATE
      bsn_align_src_in_arr(I*g_usr_nof_streams + J) <= rx_usr_siso_2arr(I)(J);
      rx_usr_sosi_2arr(I)(J)                        <= bsn_align_src_out_arr(I*g_usr_nof_streams + J);
    END GENERATE;
  END GENERATE;

END str;
