-------------------------------------------------------------------------------
--
-- Copyright (C) 2014-2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb1_board_lib, dp_lib, eth_lib, tr_10GbE_lib, diag_lib, io_ddr_lib, technology_lib, tech_ddr_lib, reorder_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE diag_lib.diag_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tech_ddr_lib.tech_ddr_pkg.ALL;
USE reorder_lib.reorder_pkg.ALL;
USE work.unb1_test_pkg.ALL;

ENTITY unb1_test IS
  GENERIC (
    g_sim         : BOOLEAN := FALSE;                 -- Overridden by TB
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0;
    g_design_name : STRING  := "unb1_test";           -- set by QSF
    g_design_note : STRING  := "Test Design";
    g_technology  : NATURAL := c_tech_stratixiv;
    g_stamp_date  : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time  : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn   : NATURAL := 0   -- SVN revision    -- set by QSF
  );
  PORT (
    -- GENERAL
    CLK          : IN    STD_LOGIC; -- System Clock
    PPS          : IN    STD_LOGIC; -- System Sync
    WDI          : OUT   STD_LOGIC; -- Watchdog Clear
    INTA         : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB         : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION      : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID           : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO       : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);
    
    -- I2C Interface to Sensors
    SENS_SC      : INOUT STD_LOGIC;
    SENS_SD      : INOUT STD_LOGIC;
  
    -- 1GbE Control Interface
    ETH_CLK      : IN    STD_LOGIC;
    ETH_SGIN     : IN    STD_LOGIC;
    ETH_SGOUT    : OUT   STD_LOGIC;

    -- Transceiver clocks
    SA_CLK       : IN  STD_LOGIC := '0'; -- SerDes Clock BN-BI / SI_FN

    -- Serial I/O
    SI_FN_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_0_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    SI_FN_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    SI_FN_2_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    SI_FN_3_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');

    SI_FN_0_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); -- (0 = LASI; 1=MDC; 2=MDIO)
    SI_FN_1_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_RSTN    : OUT   STD_LOGIC := '1'; -- ResetN is pulled up in the Vitesse chip, but pulled down again by external 1k resistor.

    BN_BI_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_0_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    BN_BI_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_1_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    BN_BI_2_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_2_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    BN_BI_3_TX    : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    BN_BI_3_RX    : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0) := (OTHERS=>'0');

    -- SO-DIMM Memory Bank I
    MB_I_IN       : IN    t_tech_ddr3_phy_in := c_tech_ddr3_phy_in_x;
    MB_I_IO       : INOUT t_tech_ddr3_phy_io;
    MB_I_OU       : OUT   t_tech_ddr3_phy_ou

    -- SO-DIMM Memory Bank II                                                                    
    -- MB_II_IN      : IN    t_tech_ddr3_phy_in := c_tech_ddr3_phy_in_x;
    -- MB_II_IO      : INOUT t_tech_ddr3_phy_io;
    -- MB_II_OU      : OUT   t_tech_ddr3_phy_ou
  );
END unb1_test;



ARCHITECTURE str OF unb1_test IS

  -- Firmware version x.y
  CONSTANT c_fw_version             : t_unb1_board_fw_version := (1, 2);

  CONSTANT c_use_front              : BOOLEAN := g_design_name="unb1_test_10GbE" OR g_design_name="unb1_test_all";
  CONSTANT c_use_back               : BOOLEAN := FALSE; -- however SI_FN_[0..2] do connect to copper connectors of single board unb

  -- Revision controlled constants
  CONSTANT c_use_1GbE               : BOOLEAN := g_design_name="unb1_test_1GbE"  OR g_design_name="unb1_test_10GbE" OR g_design_name="unb1_test_all";
  CONSTANT c_use_10GbE              : BOOLEAN := g_design_name="unb1_test_10GbE" OR g_design_name="unb1_test_all";
  CONSTANT c_use_ddr                : BOOLEAN := g_design_name="unb1_test_ddr"   OR g_design_name="unb1_test_all";


  -- max useful peripherals (FIXME read these constants from board lib)
 -- CONSTANT c_max_nof_


  -- ddr
  CONSTANT c_nof_MB                 : NATURAL := c_unb1_board_nof_ddr3;  -- Fixed control infrastructure for 2 modules per FPGA
  CONSTANT c_use_MB_I               : NATURAL := sel_a_b(c_use_ddr,1,0); -- 1: use MB_I  0: do not use
  CONSTANT c_use_MB_II              : NATURAL := 0;

  CONSTANT c_use_phy                : t_c_unb1_board_use_phy  := (sel_a_b(c_use_1GbE, 1, 0), 
                                                                  sel_a_b(c_use_front,1, 0), 
                                                                  0, 
                                                                  sel_a_b(c_use_back, 1, 0), 
                                                                  sel_a_b(c_use_MB_I, 1, 0), 
                                                                  sel_a_b(c_use_MB_II,1, 0), 
                                                                  0, 1);

  CONSTANT c_nof_streams_10GbE          : NATURAL := sel_a_b(c_use_10GbE,3,0);
  CONSTANT c_nof_streams_1GbE           : NATURAL := sel_a_b(c_use_1GbE,1,0);
  CONSTANT c_nof_streams_ddr            : NATURAL := sel_a_b(c_use_MB_I,sel_a_b(c_use_MB_II,2,1),0);
  CONSTANT c_nof_streams                : NATURAL := c_nof_streams_1GbE + c_nof_streams_10GbE + c_nof_streams_ddr;
  CONSTANT c_nof_streams_eth            : NATURAL := c_nof_streams_1GbE + c_nof_streams_10GbE;
  CONSTANT c_data_w_32                  : NATURAL := c_eth_data_w;   --  1GbE
  CONSTANT c_data_w_64                  : NATURAL := c_xgmii_data_w; -- 10GbE

  -- ddr
  CONSTANT c_ddr_master                 : t_c_tech_ddr := c_tech_ddr3_4g_800m_master;
  CONSTANT c_ddr_slave                  : t_c_tech_ddr := c_tech_ddr3_4g_800m_slave;

  -- Block generator constants
  CONSTANT c_bg_block_size              : NATURAL := 900;
  CONSTANT c_bg_gapsize                 : NATURAL := 100;
  CONSTANT c_bg_blocks_per_sync         : NATURAL := sel_a_b(g_sim, 10, 200000); -- 200000*(900+100) = 200000000 cycles = 1 second

  CONSTANT c_use_jumbo_frames           : BOOLEAN := FALSE;
  CONSTANT c_def_1GbE_block_size        : NATURAL := 0;   -- 0 first so we have time to set RX demux reg in dest. node
  CONSTANT c_def_10GbE_block_size       : NATURAL := 700; -- (700/1000) * 200MHz * 64b = 8.96Gbps user rate (excl. header overhead (16 words/packet) )

  CONSTANT c_max_frame_len              : NATURAL := sel_a_b(c_use_jumbo_frames, 9018, 1518);
  CONSTANT c_nof_header_bytes           : NATURAL := field_slv_len(c_hdr_field_arr) / c_byte_w;
  CONSTANT c_max_udp_payload_len        : NATURAL := c_max_frame_len-c_nof_header_bytes-c_network_eth_crc_len;

  CONSTANT c_max_udp_payload_nof_words_1GbE  : NATURAL := (c_max_udp_payload_len * c_byte_w) / c_data_w_32;
  CONSTANT c_max_udp_payload_nof_words_10GbE : NATURAL := (c_max_udp_payload_len * c_byte_w) / c_data_w_64;
  CONSTANT c_min_nof_words_per_block         : NATURAL := 1;
  CONSTANT c_max_nof_blocks_per_packet_1GbE  : NATURAL := c_max_udp_payload_nof_words_1GbE/c_min_nof_words_per_block;
  CONSTANT c_max_nof_blocks_per_packet_10GbE : NATURAL := c_max_udp_payload_nof_words_10GbE/c_min_nof_words_per_block;


  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  SIGNAL xo_clk                     : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_locked                  : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  
  SIGNAL cal_rec_clk                : STD_LOGIC;
  SIGNAL epcs_clk                   : STD_LOGIC;
  SIGNAL sa_rst                     : STD_LOGIC;
  
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_rst                     : STD_LOGIC;
  
  -- PIOs
  SIGNAL pout_wdi                   : STD_LOGIC;

  -- WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;

  -- PPSH
  SIGNAL reg_ppsh_mosi              : t_mem_mosi;
  SIGNAL reg_ppsh_miso              : t_mem_miso;
  
  -- UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;

  -- UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;
  SIGNAL reg_unb_sens_miso          : t_mem_miso;

  -- eth1g
  SIGNAL eth1g_tse_clk              : STD_LOGIC;
  SIGNAL eth1g_mm_rst               : STD_LOGIC;
  SIGNAL eth1g_tse_mosi             : t_mem_mosi;  -- ETH TSE MAC registers
  SIGNAL eth1g_tse_miso             : t_mem_miso;
  SIGNAL eth1g_reg_mosi             : t_mem_mosi;  -- ETH control and status registers
  SIGNAL eth1g_reg_miso             : t_mem_miso;
  SIGNAL eth1g_reg_interrupt        : STD_LOGIC;
  SIGNAL eth1g_ram_mosi             : t_mem_mosi;  -- ETH rx frame and tx frame memory
  SIGNAL eth1g_ram_miso             : t_mem_miso;

  -- EPCS read
  SIGNAL reg_dpmm_data_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_data_miso         : t_mem_miso;
  SIGNAL reg_dpmm_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_ctrl_miso         : t_mem_miso;

  -- EPCS write
  SIGNAL reg_mmdp_data_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_data_miso         : t_mem_miso;
  SIGNAL reg_mmdp_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_ctrl_miso         : t_mem_miso;

  -- EPCS status/control
  SIGNAL reg_epcs_mosi              : t_mem_mosi;
  SIGNAL reg_epcs_miso              : t_mem_miso;

  -- Remote Update
  SIGNAL reg_remu_mosi              : t_mem_mosi;
  SIGNAL reg_remu_miso              : t_mem_miso;

  -- 10GbE
  SIGNAL xaui_tx_arr                : t_unb1_board_xaui_sl_2arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL xaui_rx_arr                : t_unb1_board_xaui_sl_2arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL i_xaui_tx_arr              : t_xaui_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL i_xaui_rx_arr              : t_xaui_arr(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL mdio_mdc_arr               : STD_LOGIC_VECTOR(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL mdio_mdat_in_arr           : STD_LOGIC_VECTOR(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL mdio_mdat_oen_arr          : STD_LOGIC_VECTOR(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL reg_tr_10GbE_mosi          : t_mem_mosi;
  SIGNAL reg_tr_10GbE_miso          : t_mem_miso;

  SIGNAL reg_tr_xaui_mosi           : t_mem_mosi;
  SIGNAL reg_tr_xaui_miso           : t_mem_miso;

  SIGNAL reg_diag_bg_1GbE_mosi      : t_mem_mosi;
  SIGNAL reg_diag_bg_1GbE_miso      : t_mem_miso;
  SIGNAL ram_diag_bg_1GbE_mosi      : t_mem_mosi;
  SIGNAL ram_diag_bg_1GbE_miso      : t_mem_miso;
  SIGNAL reg_diag_tx_seq_1GbE_mosi  : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_1GbE_miso  : t_mem_miso;

  SIGNAL reg_diag_bg_10GbE_mosi     : t_mem_mosi;
  SIGNAL reg_diag_bg_10GbE_miso     : t_mem_miso;
  SIGNAL ram_diag_bg_10GbE_mosi     : t_mem_mosi;
  SIGNAL ram_diag_bg_10GbE_miso     : t_mem_miso;
  SIGNAL reg_diag_tx_seq_10GbE_mosi : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_10GbE_miso : t_mem_miso;

  SIGNAL reg_diag_bg_ddr_mosi       : t_mem_mosi;
  SIGNAL reg_diag_bg_ddr_miso       : t_mem_miso;
  SIGNAL ram_diag_bg_ddr_mosi       : t_mem_mosi;
  SIGNAL ram_diag_bg_ddr_miso       : t_mem_miso;
  SIGNAL reg_diag_tx_seq_ddr_mosi   : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_ddr_miso   : t_mem_miso;

  SIGNAL reg_dp_offload_tx_1GbE_mosi          : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_1GbE_miso          : t_mem_miso;
  SIGNAL reg_dp_offload_tx_1GbE_hdr_dat_mosi  : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_1GbE_hdr_dat_miso  : t_mem_miso;

  SIGNAL reg_dp_offload_tx_10GbE_mosi         : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_10GbE_miso         : t_mem_miso;
  SIGNAL reg_dp_offload_tx_10GbE_hdr_dat_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_10GbE_hdr_dat_miso : t_mem_miso;

  SIGNAL reg_dp_offload_rx_1GbE_hdr_dat_mosi  : t_mem_mosi;
  SIGNAL reg_dp_offload_rx_1GbE_hdr_dat_miso  : t_mem_miso;
  SIGNAL reg_dp_offload_rx_10GbE_hdr_dat_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_rx_10GbE_hdr_dat_miso : t_mem_miso;

  SIGNAL reg_bsn_monitor_1GbE_mosi       : t_mem_mosi;
  SIGNAL reg_bsn_monitor_1GbE_miso       : t_mem_miso;
  SIGNAL reg_bsn_monitor_10GbE_mosi      : t_mem_mosi;
  SIGNAL reg_bsn_monitor_10GbE_miso      : t_mem_miso;
  SIGNAL reg_bsn_monitor_ddr_mosi        : t_mem_mosi;
  SIGNAL reg_bsn_monitor_ddr_miso        : t_mem_miso;

  SIGNAL ram_diag_data_buf_1GbE_mosi     : t_mem_mosi;
  SIGNAL ram_diag_data_buf_1GbE_miso     : t_mem_miso;
  SIGNAL reg_diag_data_buf_1GbE_mosi     : t_mem_mosi;
  SIGNAL reg_diag_data_buf_1GbE_miso     : t_mem_miso;
  SIGNAL reg_diag_rx_seq_1GbE_mosi       : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_1GbE_miso       : t_mem_miso;

  SIGNAL ram_diag_data_buf_10GbE_mosi    : t_mem_mosi;
  SIGNAL ram_diag_data_buf_10GbE_miso    : t_mem_miso;
  SIGNAL reg_diag_data_buf_10GbE_mosi    : t_mem_mosi;
  SIGNAL reg_diag_data_buf_10GbE_miso    : t_mem_miso;
  SIGNAL reg_diag_rx_seq_10GbE_mosi      : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_10GbE_miso      : t_mem_miso;

  SIGNAL ram_diag_data_buf_ddr_mosi      : t_mem_mosi;
  SIGNAL ram_diag_data_buf_ddr_miso      : t_mem_miso;
  SIGNAL reg_diag_data_buf_ddr_mosi      : t_mem_mosi;
  SIGNAL reg_diag_data_buf_ddr_miso      : t_mem_miso;
  SIGNAL reg_diag_rx_seq_ddr_mosi        : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_ddr_miso        : t_mem_miso;

  SIGNAL block_gen_1GbE_src_out_arr      : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL block_gen_10GbE_src_out_arr     : t_dp_sosi_arr(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL dp_offload_tx_1GbE_src_out_arr  : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_tx_1GbE_src_in_arr   : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_tx_10GbE_src_out_arr : t_dp_sosi_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL dp_offload_tx_10GbE_src_in_arr  : t_dp_siso_arr(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL dp_offload_rx_1GbE_snk_in_arr   : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_1GbE_snk_out_arr  : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_10GbE_snk_in_arr  : t_dp_sosi_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_10GbE_snk_out_arr : t_dp_siso_arr(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL ram_ss_ss_transp_mosi           : t_mem_mosi;
  SIGNAL ram_ss_ss_transp_miso           : t_mem_miso;

  SIGNAL reg_io_ddr_mosi                 : t_mem_mosi;
  SIGNAL reg_io_ddr_miso                 : t_mem_miso;

   -- Interface: 1GbE UDP streaming ports
  SIGNAL eth1g_udp_tx_sosi_arr           : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_tx_siso_arr           : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_sosi_arr           : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_siso_arr           : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb1_board_lib.ctrl_unb1_board
  GENERIC MAP (
    g_sim                     => g_sim,
    g_design_name             => g_design_name,
    g_design_note             => g_design_note,
    g_stamp_date              => g_stamp_date,
    g_stamp_time              => g_stamp_time, 
    g_stamp_svn               => g_stamp_svn, 
    g_fw_version              => c_fw_version,
    g_mm_clk_freq             => c_unb1_board_mm_clk_freq_125M,
    g_use_phy                 => c_use_phy,
    g_aux                     => c_unb1_board_aux,
    g_udp_offload             => c_use_1GbE,
    g_udp_offload_nof_streams => c_nof_streams_1GbE,
    g_dp_clk_use_pll          => TRUE,
    g_xo_clk_use_pll          => TRUE
  )
  PORT MAP (
    -- Clock and reset signals
    cs_sim                   => cs_sim,
    xo_clk                   => xo_clk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk_out               => mm_clk,
    mm_clk                   => mm_clk,
    mm_rst                   => mm_rst,

    mm_locked                => mm_locked,
    mm_locked_out            => mm_locked,

    epcs_clk                 => epcs_clk,
    epcs_clk_out             => epcs_clk,

    dp_rst                   => dp_rst,
    dp_clk                   => dp_clk,
    dp_pps                   => OPEN,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,

    cal_rec_clk              => cal_rec_clk,
    
    -- Toggle WDI
    pout_wdi                 => pout_wdi,

    -- MM buses
    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,

    -- REMU
    reg_remu_mosi            => reg_remu_mosi,
    reg_remu_miso            => reg_remu_miso,

    -- EPCS read
    reg_dpmm_data_mosi       => reg_dpmm_data_mosi,
    reg_dpmm_data_miso       => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi       => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso       => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi       => reg_mmdp_data_mosi,
    reg_mmdp_data_miso       => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi       => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso       => reg_mmdp_ctrl_miso,

    -- EPCS status/control
    reg_epcs_mosi            => reg_epcs_mosi,
    reg_epcs_miso            => reg_epcs_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- . PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso,
    
    -- eth1g
    eth1g_tse_clk_out        => eth1g_tse_clk,
    eth1g_tse_clk            => eth1g_tse_clk,
    eth1g_mm_rst             => eth1g_mm_rst,
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,
        
    -- eth1g UDP streaming ports
    udp_tx_sosi_arr          =>  eth1g_udp_tx_sosi_arr,
    udp_tx_siso_arr          =>  eth1g_udp_tx_siso_arr,
    udp_rx_sosi_arr          =>  eth1g_udp_rx_sosi_arr,
    udp_rx_siso_arr          =>  eth1g_udp_rx_siso_arr,

    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    SENS_SC                  => SENS_SC,
    SENS_SD                  => SENS_SD,        
    -- . 1GbE Control Interface
    ETH_CLK                  => ETH_CLK,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  --dp_clk <= CLK;
  --dp_rst <= mm_rst;

  -----------------------------------------------------------------------------
  -- MM master
  -----------------------------------------------------------------------------
  u_mmm : ENTITY work.mmm_unb1_test
  GENERIC MAP (
    g_sim               => g_sim,
    g_sim_unb_nr        => g_sim_unb_nr,
    g_sim_node_nr       => g_sim_node_nr,
    g_nof_streams_1GbE  => 1,--c_nof_streams_1GbE,
    g_nof_streams_10GbE => 3,--c_nof_streams_10GbE,
    g_nof_streams_ddr   => 1,--c_nof_streams_ddr,
    g_bg_block_size     => c_bg_block_size
   )
  PORT MAP(  
    mm_rst                         => mm_rst,
    mm_clk                         => mm_clk,       

    -- PIOs
    pout_wdi                       => pout_wdi,

    -- Manual WDI override
    reg_wdi_mosi                   => reg_wdi_mosi,
    reg_wdi_miso                   => reg_wdi_miso,

    -- system_info
    reg_unb_system_info_mosi       => reg_unb_system_info_mosi,
    reg_unb_system_info_miso       => reg_unb_system_info_miso,
    rom_unb_system_info_mosi       => rom_unb_system_info_mosi,
    rom_unb_system_info_miso       => rom_unb_system_info_miso, 

    -- UniBoard I2C sensors
    reg_unb_sens_mosi              => reg_unb_sens_mosi,
    reg_unb_sens_miso              => reg_unb_sens_miso, 
 
    -- PPSH
    reg_ppsh_mosi                  => reg_ppsh_mosi,
    reg_ppsh_miso                  => reg_ppsh_miso, 
  
    -- eth1g
    eth1g_mm_rst                   => eth1g_mm_rst,
    eth1g_tse_mosi                 => eth1g_tse_mosi,
    eth1g_tse_miso                 => eth1g_tse_miso,
    eth1g_reg_mosi                 => eth1g_reg_mosi,
    eth1g_reg_miso                 => eth1g_reg_miso,
    eth1g_reg_interrupt            => eth1g_reg_interrupt,
    eth1g_ram_mosi                 => eth1g_ram_mosi,
    eth1g_ram_miso                 => eth1g_ram_miso,

    -- EPCS read
    reg_dpmm_data_mosi             => reg_dpmm_data_mosi,
    reg_dpmm_data_miso             => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi             => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso             => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi             => reg_mmdp_data_mosi,
    reg_mmdp_data_miso             => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi             => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso             => reg_mmdp_ctrl_miso,

    reg_epcs_mosi                  => reg_epcs_mosi,
    reg_epcs_miso                  => reg_epcs_miso,

    reg_remu_mosi                  => reg_remu_mosi,
    reg_remu_miso                  => reg_remu_miso,

    -- block gen
    ram_diag_bg_1GbE_mosi          => ram_diag_bg_1GbE_mosi,
    ram_diag_bg_1GbE_miso          => ram_diag_bg_1GbE_miso,
    reg_diag_bg_1GbE_mosi          => reg_diag_bg_1GbE_mosi,
    reg_diag_bg_1GbE_miso          => reg_diag_bg_1GbE_miso,
    reg_diag_tx_seq_1GbE_mosi      => reg_diag_tx_seq_1GbE_mosi,
    reg_diag_tx_seq_1GbE_miso      => reg_diag_tx_seq_1GbE_miso,

    ram_diag_bg_10GbE_mosi         => ram_diag_bg_10GbE_mosi,
    ram_diag_bg_10GbE_miso         => ram_diag_bg_10GbE_miso,
    reg_diag_bg_10GbE_mosi         => reg_diag_bg_10GbE_mosi,
    reg_diag_bg_10GbE_miso         => reg_diag_bg_10GbE_miso,
    reg_diag_tx_seq_10GbE_mosi     => reg_diag_tx_seq_10GbE_mosi,
    reg_diag_tx_seq_10GbE_miso     => reg_diag_tx_seq_10GbE_miso,

    ram_diag_bg_ddr_mosi           => ram_diag_bg_ddr_mosi,
    ram_diag_bg_ddr_miso           => ram_diag_bg_ddr_miso,
    reg_diag_bg_ddr_mosi           => reg_diag_bg_ddr_mosi,
    reg_diag_bg_ddr_miso           => reg_diag_bg_ddr_miso,
    reg_diag_tx_seq_ddr_mosi       => reg_diag_tx_seq_ddr_mosi,
    reg_diag_tx_seq_ddr_miso       => reg_diag_tx_seq_ddr_miso,

    -- dp_offload_tx
    reg_dp_offload_tx_1GbE_mosi          => reg_dp_offload_tx_1GbE_mosi,
    reg_dp_offload_tx_1GbE_miso          => reg_dp_offload_tx_1GbE_miso,
    reg_dp_offload_tx_1GbE_hdr_dat_mosi  => reg_dp_offload_tx_1GbE_hdr_dat_mosi,
    reg_dp_offload_tx_1GbE_hdr_dat_miso  => reg_dp_offload_tx_1GbE_hdr_dat_miso,

    reg_dp_offload_tx_10GbE_mosi         => reg_dp_offload_tx_10GbE_mosi,
    reg_dp_offload_tx_10GbE_miso         => reg_dp_offload_tx_10GbE_miso,
    reg_dp_offload_tx_10GbE_hdr_dat_mosi => reg_dp_offload_tx_10GbE_hdr_dat_mosi,
    reg_dp_offload_tx_10GbE_hdr_dat_miso => reg_dp_offload_tx_10GbE_hdr_dat_miso,

    -- dp_offload_rx
    reg_dp_offload_rx_1GbE_hdr_dat_mosi  => reg_dp_offload_rx_1GbE_hdr_dat_mosi,
    reg_dp_offload_rx_1GbE_hdr_dat_miso  => reg_dp_offload_rx_1GbE_hdr_dat_miso,

    reg_dp_offload_rx_10GbE_hdr_dat_mosi => reg_dp_offload_rx_10GbE_hdr_dat_mosi,
    reg_dp_offload_rx_10GbE_hdr_dat_miso => reg_dp_offload_rx_10GbE_hdr_dat_miso,

    -- bsn
    reg_bsn_monitor_1GbE_mosi            => reg_bsn_monitor_1GbE_mosi,
    reg_bsn_monitor_1GbE_miso            => reg_bsn_monitor_1GbE_miso,
    reg_bsn_monitor_10GbE_mosi           => reg_bsn_monitor_10GbE_mosi,
    reg_bsn_monitor_10GbE_miso           => reg_bsn_monitor_10GbE_miso,
    reg_bsn_monitor_ddr_mosi             => reg_bsn_monitor_ddr_mosi,
    reg_bsn_monitor_ddr_miso             => reg_bsn_monitor_ddr_miso,

    -- databuffer
    ram_diag_data_buf_1GbE_mosi          => ram_diag_data_buf_1GbE_mosi,
    ram_diag_data_buf_1GbE_miso          => ram_diag_data_buf_1GbE_miso,
    reg_diag_data_buf_1GbE_mosi          => reg_diag_data_buf_1GbE_mosi,
    reg_diag_data_buf_1GbE_miso          => reg_diag_data_buf_1GbE_miso,
    reg_diag_rx_seq_1GbE_mosi            => reg_diag_rx_seq_1GbE_mosi,
    reg_diag_rx_seq_1GbE_miso            => reg_diag_rx_seq_1GbE_miso,

    ram_diag_data_buf_10GbE_mosi         => ram_diag_data_buf_10GbE_mosi,
    ram_diag_data_buf_10GbE_miso         => ram_diag_data_buf_10GbE_miso,
    reg_diag_data_buf_10GbE_mosi         => reg_diag_data_buf_10GbE_mosi,
    reg_diag_data_buf_10GbE_miso         => reg_diag_data_buf_10GbE_miso,
    reg_diag_rx_seq_10GbE_mosi           => reg_diag_rx_seq_10GbE_mosi,
    reg_diag_rx_seq_10GbE_miso           => reg_diag_rx_seq_10GbE_miso,

    ram_diag_data_buf_ddr_mosi           => ram_diag_data_buf_ddr_mosi,
    ram_diag_data_buf_ddr_miso           => ram_diag_data_buf_ddr_miso,
    reg_diag_data_buf_ddr_mosi           => reg_diag_data_buf_ddr_mosi,
    reg_diag_data_buf_ddr_miso           => reg_diag_data_buf_ddr_miso,
    reg_diag_rx_seq_ddr_mosi             => reg_diag_rx_seq_ddr_mosi,
    reg_diag_rx_seq_ddr_miso             => reg_diag_rx_seq_ddr_miso,

    -- tr_10GbE
    reg_tr_10GbE_mosi              => reg_tr_10GbE_mosi,
    reg_tr_10GbE_miso              => reg_tr_10GbE_miso,
    reg_tr_xaui_mosi               => reg_tr_xaui_mosi,
    reg_tr_xaui_miso               => reg_tr_xaui_miso,

    ram_ss_ss_transp_mosi          => ram_ss_ss_transp_mosi,
    ram_ss_ss_transp_miso          => ram_ss_ss_transp_miso,

    reg_io_ddr_mosi                => reg_io_ddr_mosi,
    reg_io_ddr_miso                => reg_io_ddr_miso
  );


  gen_udp_stream_1GbE : IF c_use_1GbE = TRUE GENERATE
    u_udp_stream_1GbE : ENTITY work.udp_stream
    GENERIC MAP (
      g_sim                       => g_sim,
      g_nof_streams               => c_nof_streams_1GbE,
      g_data_w                    => c_data_w_32,
      g_bg_block_size             => c_bg_block_size,
      g_bg_gapsize                => c_bg_gapsize,
      g_bg_blocks_per_sync        => c_bg_blocks_per_sync,
      g_def_block_size            => c_def_1GbE_block_size,
      g_max_nof_blocks_per_packet => c_max_nof_blocks_per_packet_1GbE,
      g_remove_crc                => TRUE
    )
    PORT MAP (
      mm_rst                         => mm_rst,
      mm_clk                         => mm_clk,
  
      dp_rst                         => dp_rst,
      dp_clk                         => dp_clk,
  
      ID                             => ID,
  
      -- blockgen MM
      reg_diag_bg_mosi               => reg_diag_bg_1GbE_mosi,
      reg_diag_bg_miso               => reg_diag_bg_1GbE_miso,
      ram_diag_bg_mosi               => ram_diag_bg_1GbE_mosi,
      ram_diag_bg_miso               => ram_diag_bg_1GbE_miso,
      reg_diag_tx_seq_mosi           => reg_diag_tx_seq_1GbE_mosi,
      reg_diag_tx_seq_miso           => reg_diag_tx_seq_1GbE_miso,
  
      -- dp_offload_tx
      reg_dp_offload_tx_mosi         => reg_dp_offload_tx_1GbE_mosi,
      reg_dp_offload_tx_miso         => reg_dp_offload_tx_1GbE_miso,
      reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_1GbE_hdr_dat_mosi,
      reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_1GbE_hdr_dat_miso,
      dp_offload_tx_src_out_arr      => dp_offload_tx_1GbE_src_out_arr,
      dp_offload_tx_src_in_arr       => dp_offload_tx_1GbE_src_in_arr,
  
      -- dp_offload_rx
      reg_dp_offload_rx_hdr_dat_mosi => reg_dp_offload_rx_1GbE_hdr_dat_mosi,
      reg_dp_offload_rx_hdr_dat_miso => reg_dp_offload_rx_1GbE_hdr_dat_miso,
      dp_offload_rx_snk_in_arr       => dp_offload_rx_1GbE_snk_in_arr,
      dp_offload_rx_snk_out_arr      => dp_offload_rx_1GbE_snk_out_arr,
  
      -- bsn
      reg_bsn_monitor_mosi           => reg_bsn_monitor_1GbE_mosi,
      reg_bsn_monitor_miso           => reg_bsn_monitor_1GbE_miso,
  
      -- databuffer
      reg_diag_data_buf_mosi         => reg_diag_data_buf_1GbE_mosi,
      reg_diag_data_buf_miso         => reg_diag_data_buf_1GbE_miso,
      ram_diag_data_buf_mosi         => ram_diag_data_buf_1GbE_mosi,
      ram_diag_data_buf_miso         => ram_diag_data_buf_1GbE_miso,
      reg_diag_rx_seq_mosi           => reg_diag_rx_seq_1GbE_mosi,
      reg_diag_rx_seq_miso           => reg_diag_rx_seq_1GbE_miso
    );
  END GENERATE;


  gen_udp_stream_10GbE : IF c_use_10GbE = TRUE GENERATE
    u_udp_stream_10GbE : ENTITY work.udp_stream
    GENERIC MAP (
      g_sim                       => g_sim,
      g_nof_streams               => c_nof_streams_10GbE,
      g_data_w                    => c_data_w_64,
      g_bg_block_size             => c_bg_block_size,
      g_bg_gapsize                => c_bg_gapsize,
      g_bg_blocks_per_sync        => c_bg_blocks_per_sync,
      g_def_block_size            => c_def_10GbE_block_size,
      g_max_nof_blocks_per_packet => c_max_nof_blocks_per_packet_1GbE,
      g_remove_crc                => FALSE
    )
    PORT MAP (
      mm_rst                         => mm_rst,
      mm_clk                         => mm_clk,
  
      dp_rst                         => dp_rst,
      dp_clk                         => dp_clk,
  
      ID                             => ID,
  
      -- blockgen mm
      reg_diag_bg_mosi               => reg_diag_bg_10GbE_mosi,
      reg_diag_bg_miso               => reg_diag_bg_10GbE_miso,
      ram_diag_bg_mosi               => ram_diag_bg_10GbE_mosi,
      ram_diag_bg_miso               => ram_diag_bg_10GbE_miso,
      reg_diag_tx_seq_mosi           => reg_diag_tx_seq_10GbE_mosi,
      reg_diag_tx_seq_miso           => reg_diag_tx_seq_10GbE_miso,
  
      -- dp_offload_tx
      reg_dp_offload_tx_mosi         => reg_dp_offload_tx_10GbE_mosi,
      reg_dp_offload_tx_miso         => reg_dp_offload_tx_10GbE_miso,
      reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_10GbE_hdr_dat_mosi,
      reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_10GbE_hdr_dat_miso,
      dp_offload_tx_src_out_arr      => dp_offload_tx_10GbE_src_out_arr,
      dp_offload_tx_src_in_arr       => dp_offload_tx_10GbE_src_in_arr,
  
      -- dp_offload_rx
      reg_dp_offload_rx_hdr_dat_mosi => reg_dp_offload_rx_10GbE_hdr_dat_mosi,
      reg_dp_offload_rx_hdr_dat_miso => reg_dp_offload_rx_10GbE_hdr_dat_miso,
      dp_offload_rx_snk_in_arr       => dp_offload_rx_10GbE_snk_in_arr,
      dp_offload_rx_snk_out_arr      => dp_offload_rx_10GbE_snk_out_arr,
  
      -- bsn
      reg_bsn_monitor_mosi           => reg_bsn_monitor_10GbE_mosi,
      reg_bsn_monitor_miso           => reg_bsn_monitor_10GbE_miso,
  
      -- databuffer
      reg_diag_data_buf_mosi         => reg_diag_data_buf_10GbE_mosi,
      reg_diag_data_buf_miso         => reg_diag_data_buf_10GbE_miso,
      ram_diag_data_buf_mosi         => ram_diag_data_buf_10GbE_mosi,
      ram_diag_data_buf_miso         => ram_diag_data_buf_10GbE_miso,
      reg_diag_rx_seq_mosi           => reg_diag_rx_seq_10GbE_mosi,
      reg_diag_rx_seq_miso           => reg_diag_rx_seq_10GbE_miso
    );
  END GENERATE;



  -----------------------------------------------------------------------------
  -- Interface : 1GbE
  -----------------------------------------------------------------------------
  gen_wires_1GbE : IF c_use_1GbE=TRUE GENERATE
    eth1g_udp_tx_sosi_arr(0)         <= dp_offload_tx_1GbE_src_out_arr(0);
    dp_offload_tx_1GbE_src_in_arr(0) <= eth1g_udp_tx_siso_arr(0);

    dp_offload_rx_1GbE_snk_in_arr(0) <= eth1g_udp_rx_sosi_arr(0);
    eth1g_udp_rx_siso_arr(0)         <= dp_offload_rx_1GbE_snk_out_arr(0);
  END GENERATE;

  -----------------------------------------------------------------------------
  -- tr_10GbE
  -----------------------------------------------------------------------------

  u_areset_sa_rst : ENTITY common_lib.common_areset
  GENERIC MAP(
    g_rst_level => '1',
    g_delay_len => 4
  )
  PORT MAP(
    clk     => SA_CLK,
    in_rst  => '0',
    out_rst => sa_rst
  );    

  gen_tr_10GbE : IF c_use_10GbE=TRUE GENERATE
    u_tr_10GbE: ENTITY tr_10GbE_lib.tr_10GbE
    GENERIC MAP (
      g_sim           => g_sim,
      g_sim_level     => 1,
      g_nof_macs      => c_nof_streams_10GbE,
      g_use_mdio      => TRUE,
      g_tx_fifo_fill  => c_def_10GbE_block_size,
      g_tx_fifo_size  => c_def_10GbE_block_size*2
    )
    PORT MAP (
      tr_ref_clk_156      => SA_CLK,
      tr_ref_rst_156      => sa_rst,

      cal_rec_clk         => cal_rec_clk, -- mm_clk, --cal_clk, mm_clk required by XAUI phy

      -- MM interface
      mm_rst              => mm_rst,
      mm_clk              => mm_clk,
      
      reg_mac_mosi        => reg_tr_10GbE_mosi,
      reg_mac_miso        => reg_tr_10GbE_miso,

      xaui_mosi           => reg_tr_xaui_mosi,
      xaui_miso           => reg_tr_xaui_miso,

      -- DP interface
      dp_rst              => dp_rst,
      dp_clk              => dp_clk,

      src_out_arr         => dp_offload_rx_10GbE_snk_in_arr,
      src_in_arr          => dp_offload_rx_10GbE_snk_out_arr,
  
      snk_out_arr         => dp_offload_tx_10GbE_src_in_arr,
      snk_in_arr          => dp_offload_tx_10GbE_src_out_arr,

      -- Serial XAUI IO
      xaui_tx_arr         => i_xaui_tx_arr,
      xaui_rx_arr         => i_xaui_rx_arr,

      -- MDIO External clock and serial data.
      mdio_rst            => SI_FN_RSTN,
      mdio_mdc_arr        => mdio_mdc_arr,
      mdio_mdat_in_arr    => mdio_mdat_in_arr,
      mdio_mdat_oen_arr   => mdio_mdat_oen_arr
    );

    -- Wire together different types
    gen_wires: FOR i IN 0 TO c_nof_streams_10GbE-1 GENERATE
      xaui_tx_arr(i) <= i_xaui_tx_arr(i);
      i_xaui_rx_arr(i) <= xaui_rx_arr(i);
    END GENERATE;

    gen_tr_front : IF c_use_front=TRUE GENERATE
    u_front_io : ENTITY unb1_board_lib.unb1_board_front_io
    GENERIC MAP (
      g_nof_xaui => c_nof_streams_10GbE
    )
    PORT MAP (
      xaui_tx_arr       => xaui_tx_arr,
      xaui_rx_arr       => xaui_rx_arr,

      mdio_mdc_arr      => mdio_mdc_arr,
      mdio_mdat_in_arr  => mdio_mdat_in_arr,
      mdio_mdat_oen_arr => mdio_mdat_oen_arr,

      -- Serial I/O
      SI_FN_0_TX        => SI_FN_0_TX,
      SI_FN_0_RX        => SI_FN_0_RX,
      SI_FN_1_TX        => SI_FN_1_TX,
      SI_FN_1_RX        => SI_FN_1_RX,
      SI_FN_2_TX        => SI_FN_2_TX,
      SI_FN_2_RX        => SI_FN_2_RX,

      SI_FN_0_CNTRL     => SI_FN_0_CNTRL,
      SI_FN_1_CNTRL     => SI_FN_1_CNTRL,
      SI_FN_2_CNTRL     => SI_FN_2_CNTRL,
      SI_FN_3_CNTRL     => SI_FN_3_CNTRL
    );
    END GENERATE;
  END GENERATE;


  gen_ddr_stream : IF c_use_ddr = TRUE GENERATE
    u_ddr_stream : ENTITY work.ddr_stream
    GENERIC MAP (
      g_sim                       => g_sim,
      g_technology                => g_technology,
      g_nof_streams               => c_nof_streams_ddr,
      g_data_w                    => c_data_w_32,
      --g_bg_block_size             => c_bg_block_size,
      --g_bg_gapsize                => c_bg_gapsize,
      g_bg_blocks_per_sync        => c_bg_blocks_per_sync,
      g_tech_ddr                  => c_ddr_master,
      g_reorder_seq               => c_reorder_seq_same,
      g_ena_pre_transp            => FALSE
    )
    PORT MAP (
      mm_rst                      => mm_rst,
      mm_clk                      => mm_clk,
  
      dp_rst                      => dp_rst,
      dp_clk                      => dp_clk,
  
      -- blockgen mm
      reg_diag_bg_mosi            => reg_diag_bg_ddr_mosi,
      reg_diag_bg_miso            => reg_diag_bg_ddr_miso,
      ram_diag_bg_mosi            => ram_diag_bg_ddr_mosi,
      ram_diag_bg_miso            => ram_diag_bg_ddr_miso,
      reg_diag_tx_seq_mosi        => reg_diag_tx_seq_ddr_mosi,
      reg_diag_tx_seq_miso        => reg_diag_tx_seq_ddr_miso,
  
      -- bsn
      reg_bsn_monitor_mosi        => reg_bsn_monitor_ddr_mosi,
      reg_bsn_monitor_miso        => reg_bsn_monitor_ddr_miso,
  
      -- databuffer
      reg_diag_data_buf_mosi      => reg_diag_data_buf_ddr_mosi,
      reg_diag_data_buf_miso      => reg_diag_data_buf_ddr_miso,
      ram_diag_data_buf_mosi      => ram_diag_data_buf_ddr_mosi,
      ram_diag_data_buf_miso      => ram_diag_data_buf_ddr_miso,
      reg_diag_rx_seq_mosi        => reg_diag_rx_seq_ddr_mosi,
      reg_diag_rx_seq_miso        => reg_diag_rx_seq_ddr_miso,
  
      -- IO DDR register map      
      reg_io_ddr_mosi             => reg_io_ddr_mosi,
      reg_io_ddr_miso             => reg_io_ddr_miso,
  
      -- Reorder transpose        
      ram_ss_ss_transp_mosi       => ram_ss_ss_transp_mosi,
      ram_ss_ss_transp_miso       => ram_ss_ss_transp_miso,
  
      -- SO-DIMM Memory Bank I
      MB_I_IN                     => MB_I_IN,
      MB_I_IO                     => MB_I_IO,
      MB_I_OU                     => MB_I_OU
    );
  END GENERATE;
END str;

