-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Model the links between the FN and BN in the UniBoard TR mesh
-- Description: See unb1_board_mesh_reorder_bidir.vhd

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE work.unb1_board_pkg.ALL;


ENTITY unb1_board_mesh_model_siso IS
  GENERIC (
    g_reorder   : BOOLEAN := TRUE
  );
  PORT (
    -- FN to BN
    fn0_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));  -- _2arr = (node id 3,2,1,0)(tr lane 3,2,1,0)
    fn1_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    fn2_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    fn3_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    
    bn0_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    bn1_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    bn2_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    bn3_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    
    -- BN to FN
    bn0_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    bn1_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    bn2_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    bn3_rx_siso_2arr  : IN  t_unb1_board_mesh_siso_2arr := (OTHERS=>(OTHERS=>c_dp_siso_rst));
    
    fn0_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    fn1_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    fn2_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr;
    fn3_tx_siso_2arr  : OUT t_unb1_board_mesh_siso_2arr
  );
END unb1_board_mesh_model_siso;


ARCHITECTURE beh OF unb1_board_mesh_model_siso IS
BEGIN

  -- Functional mesh connect for transpose
  no_reorder : IF g_reorder=FALSE GENERATE
    -- BN(i)(j) <= FN(j)(i)
    bn0_tx_siso_2arr(0) <= fn0_rx_siso_2arr(0);
    bn0_tx_siso_2arr(1) <= fn1_rx_siso_2arr(0);
    bn0_tx_siso_2arr(2) <= fn2_rx_siso_2arr(0);
    bn0_tx_siso_2arr(3) <= fn3_rx_siso_2arr(0);
    
    bn1_tx_siso_2arr(0) <= fn0_rx_siso_2arr(1);
    bn1_tx_siso_2arr(1) <= fn1_rx_siso_2arr(1);
    bn1_tx_siso_2arr(2) <= fn2_rx_siso_2arr(1);
    bn1_tx_siso_2arr(3) <= fn3_rx_siso_2arr(1);
    
    bn2_tx_siso_2arr(0) <= fn0_rx_siso_2arr(2);
    bn2_tx_siso_2arr(1) <= fn1_rx_siso_2arr(2);
    bn2_tx_siso_2arr(2) <= fn2_rx_siso_2arr(2);
    bn2_tx_siso_2arr(3) <= fn3_rx_siso_2arr(2);
    
    bn3_tx_siso_2arr(0) <= fn0_rx_siso_2arr(3);
    bn3_tx_siso_2arr(1) <= fn1_rx_siso_2arr(3);
    bn3_tx_siso_2arr(2) <= fn2_rx_siso_2arr(3);
    bn3_tx_siso_2arr(3) <= fn3_rx_siso_2arr(3);
    
    -- FN(i)(j) <= BN(j)(i)
    fn0_tx_siso_2arr(0) <= bn0_rx_siso_2arr(0);
    fn0_tx_siso_2arr(1) <= bn1_rx_siso_2arr(0);
    fn0_tx_siso_2arr(2) <= bn2_rx_siso_2arr(0);
    fn0_tx_siso_2arr(3) <= bn3_rx_siso_2arr(0);
    
    fn1_tx_siso_2arr(0) <= bn0_rx_siso_2arr(1);
    fn1_tx_siso_2arr(1) <= bn1_rx_siso_2arr(1);
    fn1_tx_siso_2arr(2) <= bn2_rx_siso_2arr(1);
    fn1_tx_siso_2arr(3) <= bn3_rx_siso_2arr(1);
    
    fn2_tx_siso_2arr(0) <= bn0_rx_siso_2arr(2);
    fn2_tx_siso_2arr(1) <= bn1_rx_siso_2arr(2);
    fn2_tx_siso_2arr(2) <= bn2_rx_siso_2arr(2);
    fn2_tx_siso_2arr(3) <= bn3_rx_siso_2arr(2);
    
    fn3_tx_siso_2arr(0) <= bn0_rx_siso_2arr(3);
    fn3_tx_siso_2arr(1) <= bn1_rx_siso_2arr(3);
    fn3_tx_siso_2arr(2) <= bn2_rx_siso_2arr(3);
    fn3_tx_siso_2arr(3) <= bn3_rx_siso_2arr(3);
  END GENERATE;
  
  -- Actual UniBoard PCB mesh connect for transpose
  gen_reorder : IF g_reorder=TRUE GENERATE
                                            -- BN, phy  <= FN, phy
    bn0_tx_siso_2arr(0) <= fn3_rx_siso_2arr(1);  -- 0,0 <= 3,1
    bn0_tx_siso_2arr(1) <= fn2_rx_siso_2arr(0);  -- 0,1 <= 2,0
    bn0_tx_siso_2arr(2) <= fn1_rx_siso_2arr(0);  -- 0,2 <= 1,0
    bn0_tx_siso_2arr(3) <= fn0_rx_siso_2arr(0);  -- 0,3 <= 0,0
    
    bn1_tx_siso_2arr(0) <= fn3_rx_siso_2arr(0);  -- 1,0 <= 3,0
    bn1_tx_siso_2arr(1) <= fn2_rx_siso_2arr(1);  -- 1,1 <= 2,1
    bn1_tx_siso_2arr(2) <= fn1_rx_siso_2arr(1);  -- 1,2 <= 1,1
    bn1_tx_siso_2arr(3) <= fn0_rx_siso_2arr(3);  -- 1,3 <= 0,3
    
    bn2_tx_siso_2arr(0) <= fn3_rx_siso_2arr(2);  -- 2,0 <= 3,2
    bn2_tx_siso_2arr(1) <= fn2_rx_siso_2arr(2);  -- 2,1 <= 2,2
    bn2_tx_siso_2arr(2) <= fn0_rx_siso_2arr(2);  -- 2,2 <= 0,2
    bn2_tx_siso_2arr(3) <= fn1_rx_siso_2arr(3);  -- 2,3 <= 1,3
    
    bn3_tx_siso_2arr(0) <= fn3_rx_siso_2arr(3);  -- 3,0 <= 3,3
    bn3_tx_siso_2arr(1) <= fn0_rx_siso_2arr(1);  -- 3,1 <= 0,1
    bn3_tx_siso_2arr(2) <= fn2_rx_siso_2arr(3);  -- 3,2 <= 2,3
    bn3_tx_siso_2arr(3) <= fn1_rx_siso_2arr(2);  -- 3,3 <= 1,2
    
                                             -- FN, phy <= BN, phy
    fn0_tx_siso_2arr(0) <= bn0_rx_siso_2arr(3);  -- 0,0 <= 0,3
    fn0_tx_siso_2arr(1) <= bn3_rx_siso_2arr(1);  -- 0,1 <= 3,1
    fn0_tx_siso_2arr(2) <= bn2_rx_siso_2arr(2);  -- 0,2 <= 2,2
    fn0_tx_siso_2arr(3) <= bn1_rx_siso_2arr(3);  -- 0,3 <= 1,3
    
    fn1_tx_siso_2arr(0) <= bn0_rx_siso_2arr(2);  -- 1,0 <= 0,2
    fn1_tx_siso_2arr(1) <= bn1_rx_siso_2arr(2);  -- 1,1 <= 1,2
    fn1_tx_siso_2arr(2) <= bn3_rx_siso_2arr(3);  -- 1,2 <= 3,3
    fn1_tx_siso_2arr(3) <= bn2_rx_siso_2arr(3);  -- 1,3 <= 2,3
    
    fn2_tx_siso_2arr(0) <= bn0_rx_siso_2arr(1);  -- 2,0 <= 0,1
    fn2_tx_siso_2arr(1) <= bn1_rx_siso_2arr(1);  -- 2,1 <= 1,1
    fn2_tx_siso_2arr(2) <= bn2_rx_siso_2arr(1);  -- 2,2 <= 2,1
    fn2_tx_siso_2arr(3) <= bn3_rx_siso_2arr(2);  -- 2,3 <= 3,2
    
    fn3_tx_siso_2arr(0) <= bn1_rx_siso_2arr(0);  -- 3,0 <= 1,0
    fn3_tx_siso_2arr(1) <= bn0_rx_siso_2arr(0);  -- 3,1 <= 0,0
    fn3_tx_siso_2arr(2) <= bn2_rx_siso_2arr(0);  -- 3,2 <= 2,0
    fn3_tx_siso_2arr(3) <= bn3_rx_siso_2arr(0);  -- 3,3 <= 3,0
  END GENERATE;
  
END beh;
