-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb2_board_lib, dp_lib, eth_lib, tr_10GbE_lib, diag_lib, technology_lib, tech_ddr_lib, io_ddr_lib, tech_clkbuf_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE unb2_board_lib.unb2_board_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE diag_lib.diag_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tech_ddr_lib.tech_ddr_pkg.ALL;
USE work.unb2_test_pkg.ALL;

ENTITY unb2_test IS
  GENERIC (
    g_design_name      : STRING  := "unb2_test";
    g_design_note      : STRING  := "UNUSED";
    g_technology       : NATURAL := c_tech_arria10;
    g_ddr_MB_I         : t_c_tech_ddr := c_tech_ddr4_4g_1600m;  -- DDR4 has no master or slave, so no need to check number of MB
    g_ddr_MB_II        : t_c_tech_ddr := c_tech_ddr4_4g_1600m;  -- DDR4 has no master or slave, so no need to check number of MB
    g_sim              : BOOLEAN := FALSE; --Overridden by TB
    g_sim_unb_nr       : NATURAL := 0;
    g_sim_node_nr      : NATURAL := 0;
    g_sim_model_ddr    : BOOLEAN := FALSE;
    g_stamp_date       : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time       : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn        : NATURAL := 0;  -- SVN revision    -- set by QSF
    g_factory_image    : BOOLEAN := FALSE
  );
  PORT (
    -- GENERAL
    CLK          : IN    STD_LOGIC; -- System Clock
    PPS          : IN    STD_LOGIC; -- System Sync
    WDI          : OUT   STD_LOGIC; -- Watchdog Clear
    INTA         : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB         : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION      : IN    STD_LOGIC_VECTOR(c_unb2_board_aux.version_w-1 DOWNTO 0);
    ID           : IN    STD_LOGIC_VECTOR(c_unb2_board_aux.id_w-1 DOWNTO 0);
    TESTIO       : INOUT STD_LOGIC_VECTOR(c_unb2_board_aux.testio_w-1 DOWNTO 0);
    
    -- I2C Interface to Sensors
    SENS_SC      : INOUT STD_LOGIC;
    SENS_SD      : INOUT STD_LOGIC;
  
    -- 1GbE Control Interface
    ETH_CLK      : IN    STD_LOGIC;
    ETH_SGIN     : IN    STD_LOGIC_VECTOR(c_unb2_board_nof_eth-1 DOWNTO 0);
    ETH_SGOUT    : OUT   STD_LOGIC_VECTOR(c_unb2_board_nof_eth-1 DOWNTO 0);

    -- Transceiver clocks
    SA_CLK       : IN    STD_LOGIC := '0'; -- Clock 10GbE front (qsfp) and ring lines
    SB_CLK       : IN    STD_LOGIC := '0'; -- Clock 10GbE back upper 24 lines
    BCK_REF_CLK  : IN    STD_LOGIC := '0'; -- Clock 10GbE back lower 24 lines

    -- DDR reference clocks
    MB_I_REF_CLK  : IN   STD_LOGIC := '0';  -- Reference clock for MB_I
    MB_II_REF_CLK : IN   STD_LOGIC := '0';  -- Reference clock for MB_II

    -- back transceivers
    --BCK_RX       : IN    STD_LOGIC_VECTOR((c_unb2_board_tr_back.bus_w * c_unb2_board_tr_back.nof_bus)-1 downto 0) := (OTHERS=>'0');
    --BCK_TX       : OUT   STD_LOGIC_VECTOR((c_unb2_board_tr_back.bus_w * c_unb2_board_tr_back.nof_bus)-1 downto 0);
    BCK_SDA      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);
    BCK_SCL      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);
    BCK_ERR      : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_back.i2c_w-1 downto 0);

    -- ring transceivers
    RING_0_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0) := (OTHERS=>'0');
    RING_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
    RING_1_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0) := (OTHERS=>'0');
    RING_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_ring.bus_w-1 downto 0);
    -- pmbus
    PMBUS_SC     : INOUT STD_LOGIC;
    PMBUS_SD     : INOUT STD_LOGIC;
    PMBUS_ALERT  : IN    STD_LOGIC := '0';
    -- front transceivers
    QSFP_0_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_0_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_1_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_1_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_2_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_2_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_3_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_3_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_4_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_4_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);
    QSFP_5_RX    : IN    STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0) := (OTHERS=>'0');
    QSFP_5_TX    : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.bus_w-1 downto 0);

    QSFP_SDA     : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_SCL     : INOUT STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.i2c_w-1 downto 0);
    QSFP_RST     : INOUT STD_LOGIC;

    -- SO-DIMM Memory Bank I
    MB_I_IN      : IN    t_tech_ddr4_phy_in := c_tech_ddr4_phy_in_x;
    MB_I_IO      : INOUT t_tech_ddr4_phy_io;
    MB_I_OU      : OUT   t_tech_ddr4_phy_ou;

    -- SO-DIMM Memory Bank II
    MB_II_IN     : IN    t_tech_ddr4_phy_in := c_tech_ddr4_phy_in_x;
    MB_II_IO     : INOUT t_tech_ddr4_phy_io;
    MB_II_OU     : OUT   t_tech_ddr4_phy_ou;

    -- Leds
    QSFP_LED     : OUT   STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp_nof_leds-1 DOWNTO 0)
  );
END unb2_test;


ARCHITECTURE str OF unb2_test IS

  -- Firmware version x.y
  CONSTANT c_fw_version                 : t_unb2_board_fw_version := (1, 1);
  CONSTANT c_mm_clk_freq                : NATURAL := c_unb2_board_mm_clk_freq_125M;

  -- Revision controlled constants
  CONSTANT c_use_1GbE                   : BOOLEAN := g_design_name="unb2_test_1GbE"  OR g_design_name="unb2_test_10GbE" OR g_design_name="unb2_test_all";
  CONSTANT c_use_10GbE                  : BOOLEAN := g_design_name="unb2_test_10GbE" OR g_design_name="unb2_test_all";
  CONSTANT c_use_10GbE_qsfp             : BOOLEAN := c_use_10GbE;
  CONSTANT c_use_10GbE_ring             : BOOLEAN := c_use_10GbE;
  CONSTANT c_use_10GbE_back0            : BOOLEAN := FALSE;
  CONSTANT c_use_10GbE_back1            : BOOLEAN := FALSE;
  CONSTANT c_use_ddr                    : BOOLEAN := g_design_name="unb2_test_ddr"   OR g_design_name="unb2_test_all";
  CONSTANT c_use_MB_I                   : BOOLEAN := c_use_ddr;  -- FIXME: g_design_name="unb2_test_MB_I"   OR g_design_name="unb2_test_all";
  CONSTANT c_use_MB_II                  : BOOLEAN := FALSE;      -- FIXME: g_design_name="unb2_test_MB_II"  OR g_design_name="unb2_test_all";
  CONSTANT c_use_MB_both                : BOOLEAN := FALSE;      -- FIXME: c_use_MB_I OR c_use_MB_II;

  -- transceivers
  CONSTANT c_nof_qsfp                   : NATURAL := c_unb2_board_tr_qsfp.nof_bus * c_unb2_board_tr_qsfp.bus_w;--8
  CONSTANT c_nof_ring                   : NATURAL := c_unb2_board_tr_ring.nof_bus * c_unb2_board_tr_ring.bus_w;
  CONSTANT c_nof_back0                  : NATURAL := c_unb2_board_tr_back.bus_w;
  CONSTANT c_nof_back1                  : NATURAL := c_unb2_board_tr_back.bus_w;

  -- 1GbE
  CONSTANT c_nof_streams_1GbE           : NATURAL := sel_a_b(c_use_1GbE,1,0); --sel_a_b(c_use_1GbE,c_unb2_board_nof_eth,0);

  -- 10GbE
  CONSTANT c_nof_streams_qsfp           : NATURAL := sel_a_b(c_use_10GbE_qsfp,c_nof_qsfp,0);
  CONSTANT c_nof_streams_ring           : NATURAL := sel_a_b(c_use_10GbE_ring,c_nof_ring,0);
  CONSTANT c_nof_streams_back0          : NATURAL := sel_a_b(c_use_10GbE_back0,c_nof_back0,0);
  CONSTANT c_nof_streams_back1          : NATURAL := sel_a_b(c_use_10GbE_back1,c_nof_back1,0);

  CONSTANT c_nof_streams_10GbE          : NATURAL := c_nof_streams_qsfp+c_nof_streams_ring+c_nof_streams_back0+c_nof_streams_back1;

  CONSTANT c_nof_qsfp_bus               : NATURAL := ceil_div(c_nof_streams_qsfp,c_unb2_board_tr_qsfp.bus_w);
  CONSTANT c_nof_ring_bus               : NATURAL := ceil_div(c_nof_streams_ring,c_unb2_board_tr_ring.bus_w);
  CONSTANT c_nof_back_bus               : NATURAL := ceil_div(c_nof_streams_back0,c_unb2_board_tr_back.bus_w) + ceil_div(c_nof_streams_back1,c_unb2_board_tr_back.bus_w);

  CONSTANT c_data_w_32                  : NATURAL := c_eth_data_w;   --  1GbE
  CONSTANT c_data_w_64                  : NATURAL := c_xgmii_data_w; -- 10GbE

  -- ddr
  CONSTANT c_ddr_dp_data_w              : NATURAL := 144;   -- DDR4 with dq_w = 72, rsl = 8 so ctrl data width = 576 and therefore the mixed width FIFO ratio is 576 /144 = 4
  CONSTANT c_ddr_dp_seq_dat_w           : NATURAL := 17;    -- >= 1, test sequence data width. Choose g_dp_seq_dat_w <= g_dp_data_w. The seq data gets replicated to fill g_dp_data_w.
  CONSTANT c_ddr_dp_fifo_depth          : NATURAL := 2048;  -- >= 2048, write FIFO depth and read FIFO depth at DP side of the FIFOs
  CONSTANT c_ddr_db_buf_nof_data        : NATURAL := 1024;

  -- Block generator constants
  CONSTANT c_bg_block_size              : NATURAL := 900;
  CONSTANT c_bg_gapsize                 : NATURAL := 100;
  CONSTANT c_bg_blocks_per_sync         : NATURAL := sel_a_b(g_sim, 10, 200000); -- 200000*(900+100) = 200000000 cycles = 1 second

  CONSTANT c_use_jumbo_frames           : BOOLEAN := FALSE;
  CONSTANT c_def_1GbE_block_size        : NATURAL := 0;   -- 0 first so we have time to set RX demux reg in dest. node
  CONSTANT c_def_10GbE_block_size       : NATURAL := 700; -- (700/1000) * 200MHz * 64b = 8.96Gbps user rate (excl. header overhead (16 words/packet) )

  CONSTANT c_max_frame_len              : NATURAL := sel_a_b(c_use_jumbo_frames, 9018, 1518);
  CONSTANT c_nof_header_bytes           : NATURAL := field_slv_len(c_hdr_field_arr) / c_byte_w;
  CONSTANT c_max_udp_payload_len        : NATURAL := c_max_frame_len-c_nof_header_bytes-c_network_eth_crc_len;

  CONSTANT c_max_udp_payload_nof_words_1GbE  : NATURAL := (c_max_udp_payload_len * c_byte_w) / c_data_w_32;
  CONSTANT c_max_udp_payload_nof_words_10GbE : NATURAL := (c_max_udp_payload_len * c_byte_w) / c_data_w_64;
  CONSTANT c_min_nof_words_per_block         : NATURAL := 1;
  CONSTANT c_max_nof_blocks_per_packet_1GbE  : NATURAL := c_max_udp_payload_nof_words_1GbE/c_min_nof_words_per_block;
  CONSTANT c_max_nof_blocks_per_packet_10GbE : NATURAL := c_max_udp_payload_nof_words_10GbE/c_min_nof_words_per_block;


  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  
  SIGNAL ext_clk200                 : STD_LOGIC;
  SIGNAL ext_rst200                 : STD_LOGIC;
  
  SIGNAL xo_ethclk                  : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_rst                     : STD_LOGIC;

  SIGNAL SA_CLK_buf                 : STD_LOGIC;

  SIGNAL mb_I_ref_rst               : STD_LOGIC;
  SIGNAL mb_II_ref_rst              : STD_LOGIC;
  
  SIGNAL ddr_I_clk200               : STD_LOGIC;
  SIGNAL ddr_I_rst200               : STD_LOGIC;
  SIGNAL ddr_II_clk200              : STD_LOGIC;
  SIGNAL ddr_II_rst200              : STD_LOGIC;
  
  -- PIOs
  SIGNAL pout_wdi                   : STD_LOGIC;

  -- WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;

  -- PPSH
  SIGNAL reg_ppsh_mosi              : t_mem_mosi;
  SIGNAL reg_ppsh_miso              : t_mem_miso;
  
  -- UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;

  -- UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;
  SIGNAL reg_unb_sens_miso          : t_mem_miso;

  -- eth1g ch0
  SIGNAL eth1g_eth0_mm_rst          : STD_LOGIC;
  SIGNAL eth1g_eth0_tse_mosi        : t_mem_mosi;  -- ETH TSE MAC registers
  SIGNAL eth1g_eth0_tse_miso        : t_mem_miso;
  SIGNAL eth1g_eth0_reg_mosi        : t_mem_mosi;  -- ETH control and status registers
  SIGNAL eth1g_eth0_reg_miso        : t_mem_miso;
  SIGNAL eth1g_eth0_reg_interrupt   : STD_LOGIC;   -- Interrupt
  SIGNAL eth1g_eth0_ram_mosi        : t_mem_mosi;  -- ETH rx frame and tx frame memory
  SIGNAL eth1g_eth0_ram_miso        : t_mem_miso;

  -- eth1g ch1
  SIGNAL eth1g_eth1_mm_rst          : STD_LOGIC;
  SIGNAL eth1g_eth1_tse_mosi        : t_mem_mosi;  -- ETH TSE MAC registers
  SIGNAL eth1g_eth1_tse_miso        : t_mem_miso;
  SIGNAL eth1g_eth1_reg_mosi        : t_mem_mosi;  -- ETH control and status registers
  SIGNAL eth1g_eth1_reg_miso        : t_mem_miso;
  SIGNAL eth1g_eth1_reg_interrupt   : STD_LOGIC;   -- Interrupt
  SIGNAL eth1g_eth1_ram_mosi        : t_mem_mosi;  -- ETH rx frame and tx frame memory
  SIGNAL eth1g_eth1_ram_miso        : t_mem_miso;

  -- EPCS read
  SIGNAL reg_dpmm_data_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_data_miso         : t_mem_miso;
  SIGNAL reg_dpmm_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_dpmm_ctrl_miso         : t_mem_miso;

  -- EPCS write
  SIGNAL reg_mmdp_data_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_data_miso         : t_mem_miso;
  SIGNAL reg_mmdp_ctrl_mosi         : t_mem_mosi;
  SIGNAL reg_mmdp_ctrl_miso         : t_mem_miso;

  -- EPCS status/control
  SIGNAL reg_epcs_mosi              : t_mem_mosi;
  SIGNAL reg_epcs_miso              : t_mem_miso;

  -- Remote Update
  SIGNAL reg_remu_mosi              : t_mem_mosi;
  SIGNAL reg_remu_miso              : t_mem_miso;

  -- 10GbE
  SIGNAL i_serial_10G_tx_qsfp_ring_arr   : STD_LOGIC_VECTOR(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0);
  SIGNAL i_serial_10G_rx_qsfp_ring_arr   : STD_LOGIC_VECTOR(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0);
  SIGNAL i_serial_10G_tx_back0_arr       : STD_LOGIC_VECTOR(c_nof_streams_back0-1 DOWNTO 0);
  SIGNAL i_serial_10G_rx_back0_arr       : STD_LOGIC_VECTOR(c_nof_streams_back0-1 DOWNTO 0);
  SIGNAL i_serial_10G_tx_back1_arr       : STD_LOGIC_VECTOR(c_nof_streams_back1-1 DOWNTO 0);
  SIGNAL i_serial_10G_rx_back1_arr       : STD_LOGIC_VECTOR(c_nof_streams_back1-1 DOWNTO 0);

  SIGNAL serial_10G_tx_qsfp_arr          : STD_LOGIC_VECTOR(c_nof_streams_qsfp-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_qsfp_arr          : STD_LOGIC_VECTOR(c_nof_streams_qsfp-1 DOWNTO 0);
  SIGNAL serial_10G_tx_ring_arr          : STD_LOGIC_VECTOR(c_nof_streams_ring-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_ring_arr          : STD_LOGIC_VECTOR(c_nof_streams_ring-1 DOWNTO 0);

  SIGNAL i_QSFP_TX                       : t_unb2_board_qsfp_bus_2arr(c_nof_qsfp_bus-1 DOWNTO 0); 
  SIGNAL i_QSFP_RX                       : t_unb2_board_qsfp_bus_2arr(c_nof_qsfp_bus-1 DOWNTO 0); 
  SIGNAL i_RING_TX                       : t_unb2_board_ring_bus_2arr(c_nof_ring_bus-1 DOWNTO 0); 
  SIGNAL i_RING_RX                       : t_unb2_board_ring_bus_2arr(c_nof_ring_bus-1 DOWNTO 0); 
  SIGNAL i_BCK_TX                        : t_unb2_board_back_bus_2arr(c_nof_back_bus-1 DOWNTO 0); 
  SIGNAL i_BCK_RX                        : t_unb2_board_back_bus_2arr(c_nof_back_bus-1 DOWNTO 0); 

  SIGNAL serial_10G_tx_back_arr          : STD_LOGIC_VECTOR(c_nof_streams_back0+c_nof_streams_back1-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL serial_10G_rx_back_arr          : STD_LOGIC_VECTOR(c_nof_streams_back0+c_nof_streams_back1-1 DOWNTO 0);


  SIGNAL reg_tr_10GbE_qsfp_ring_mosi     : t_mem_mosi;
  SIGNAL reg_tr_10GbE_qsfp_ring_miso     : t_mem_miso;
  SIGNAL reg_tr_10GbE_back0_mosi         : t_mem_mosi;
  SIGNAL reg_tr_10GbE_back0_miso         : t_mem_miso;
  SIGNAL reg_tr_10GbE_back1_mosi         : t_mem_mosi;
  SIGNAL reg_tr_10GbE_back1_miso         : t_mem_miso;

  SIGNAL reg_diag_bg_1GbE_mosi           : t_mem_mosi;
  SIGNAL reg_diag_bg_1GbE_miso           : t_mem_miso;
  SIGNAL ram_diag_bg_1GbE_mosi           : t_mem_mosi;
  SIGNAL ram_diag_bg_1GbE_miso           : t_mem_miso;
  SIGNAL reg_diag_tx_seq_1GbE_mosi       : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_1GbE_miso       : t_mem_miso;

  SIGNAL reg_diag_bg_10GbE_mosi          : t_mem_mosi;
  SIGNAL reg_diag_bg_10GbE_miso          : t_mem_miso;
  SIGNAL ram_diag_bg_10GbE_mosi          : t_mem_mosi;
  SIGNAL ram_diag_bg_10GbE_miso          : t_mem_miso;
  SIGNAL reg_diag_tx_seq_10GbE_mosi      : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_10GbE_miso      : t_mem_miso;

--  SIGNAL reg_dp_offload_tx_1GbE_mosi         : t_mem_mosi;
--  SIGNAL reg_dp_offload_tx_1GbE_miso         : t_mem_miso;
--  SIGNAL reg_dp_offload_tx_1GbE_hdr_dat_mosi : t_mem_mosi;
--  SIGNAL reg_dp_offload_tx_1GbE_hdr_dat_miso : t_mem_miso;
--
--  SIGNAL reg_dp_offload_rx_1GbE_hdr_dat_mosi : t_mem_mosi;
--  SIGNAL reg_dp_offload_rx_1GbE_hdr_dat_miso : t_mem_miso;

  SIGNAL reg_bsn_monitor_1GbE_mosi       : t_mem_mosi;
  SIGNAL reg_bsn_monitor_1GbE_miso       : t_mem_miso;
  SIGNAL reg_bsn_monitor_10GbE_mosi      : t_mem_mosi;
  SIGNAL reg_bsn_monitor_10GbE_miso      : t_mem_miso;

  SIGNAL ram_diag_data_buf_1GbE_mosi     : t_mem_mosi;
  SIGNAL ram_diag_data_buf_1GbE_miso     : t_mem_miso;
  SIGNAL reg_diag_data_buf_1GbE_mosi     : t_mem_mosi;
  SIGNAL reg_diag_data_buf_1GbE_miso     : t_mem_miso;
  SIGNAL reg_diag_rx_seq_1GbE_mosi       : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_1GbE_miso       : t_mem_miso;

  SIGNAL ram_diag_data_buf_10GbE_mosi    : t_mem_mosi;
  SIGNAL ram_diag_data_buf_10GbE_miso    : t_mem_miso;
  SIGNAL reg_diag_data_buf_10GbE_mosi    : t_mem_mosi;
  SIGNAL reg_diag_data_buf_10GbE_miso    : t_mem_miso;
  SIGNAL reg_diag_rx_seq_10GbE_mosi      : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_10GbE_miso      : t_mem_miso;

  SIGNAL dp_offload_tx_1GbE_src_out_arr  : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_tx_1GbE_src_in_arr   : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_1GbE_snk_in_arr   : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_1GbE_snk_out_arr  : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);

  SIGNAL dp_offload_tx_10GbE_src_out_arr : t_dp_sosi_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL dp_offload_tx_10GbE_src_in_arr  : t_dp_siso_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_10GbE_snk_in_arr  : t_dp_sosi_arr(c_nof_streams_10GbE-1 DOWNTO 0);
  SIGNAL dp_offload_rx_10GbE_snk_out_arr : t_dp_siso_arr(c_nof_streams_10GbE-1 DOWNTO 0);

  SIGNAL reg_diag_tx_seq_ddr_mosi        : t_mem_mosi;
  SIGNAL reg_diag_tx_seq_ddr_miso        : t_mem_miso;
  SIGNAL reg_diag_rx_seq_ddr_mosi        : t_mem_mosi;
  SIGNAL reg_diag_rx_seq_ddr_miso        : t_mem_miso;
  SIGNAL reg_diag_data_buf_ddr_mosi      : t_mem_mosi;
  SIGNAL reg_diag_data_buf_ddr_miso      : t_mem_miso;
  SIGNAL ram_diag_data_buf_ddr_mosi      : t_mem_mosi;
  SIGNAL ram_diag_data_buf_ddr_miso      : t_mem_miso;
  SIGNAL reg_io_ddr_mosi                 : t_mem_mosi;
  SIGNAL reg_io_ddr_miso                 : t_mem_miso;

   -- Interface: 1GbE UDP streaming ports
  SIGNAL eth1g_udp_tx_sosi_arr           : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_tx_siso_arr           : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_sosi_arr           : t_dp_sosi_arr(c_nof_streams_1GbE-1 DOWNTO 0);
  SIGNAL eth1g_udp_rx_siso_arr           : t_dp_siso_arr(c_nof_streams_1GbE-1 DOWNTO 0);

  -- QSFP leds
  SIGNAL qsfp_green_led_arr              : STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.nof_bus-1 DOWNTO 0);
  SIGNAL qsfp_red_led_arr                : STD_LOGIC_VECTOR(c_unb2_board_tr_qsfp.nof_bus-1 DOWNTO 0);

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb2_board_lib.ctrl_unb2_board
  GENERIC MAP (
    g_sim                     => g_sim,
    g_design_name             => g_design_name,
    g_design_note             => g_design_note,
    g_stamp_date              => g_stamp_date,
    g_stamp_time              => g_stamp_time, 
    g_stamp_svn               => g_stamp_svn, 
    g_fw_version              => c_fw_version,
    g_mm_clk_freq             => sel_a_b(g_sim,c_unb2_board_mm_clk_freq_25M,c_unb2_board_mm_clk_freq_125M),
    g_eth_clk_freq            => c_unb2_board_eth_clk_freq_125M,
    g_aux                     => c_unb2_board_aux,
    g_udp_offload             => c_use_1GbE,
    g_udp_offload_nof_streams => c_nof_streams_1GbE,
    g_dp_clk_use_pll          => TRUE,
    g_factory_image           => g_factory_image
  )
  PORT MAP (
    -- Clock an reset signals
    cs_sim                   => cs_sim,
    
    ext_clk200               => ext_clk200,
    ext_rst200               => ext_rst200,
    
    xo_ethclk                => xo_ethclk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_rst                   => mm_rst,

    dp_rst                   => dp_rst,
    dp_clk                   => dp_clk,
    dp_pps                   => OPEN,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,

    mb_I_ref_rst             => mb_I_ref_rst,
    mb_II_ref_rst            => mb_II_ref_rst,
    
    -- Toggle WDI
    pout_wdi                 => pout_wdi,

    -- MM buses
    -- REMU
    reg_remu_mosi            => reg_remu_mosi,
    reg_remu_miso            => reg_remu_miso,

    -- EPCS read
    reg_dpmm_data_mosi       => reg_dpmm_data_mosi,
    reg_dpmm_data_miso       => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi       => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso       => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi       => reg_mmdp_data_mosi,
    reg_mmdp_data_miso       => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi       => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso       => reg_mmdp_ctrl_miso,

    -- EPCS status/control
    reg_epcs_mosi            => reg_epcs_mosi,
    reg_epcs_miso            => reg_epcs_miso,

    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- . PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso,
    
    -- eth1g ch0
    eth1g_mm_rst             => eth1g_eth0_mm_rst,
    eth1g_tse_mosi           => eth1g_eth0_tse_mosi,
    eth1g_tse_miso           => eth1g_eth0_tse_miso,
    eth1g_reg_mosi           => eth1g_eth0_reg_mosi,
    eth1g_reg_miso           => eth1g_eth0_reg_miso,
    eth1g_reg_interrupt      => eth1g_eth0_reg_interrupt,
    eth1g_ram_mosi           => eth1g_eth0_ram_mosi,
    eth1g_ram_miso           => eth1g_eth0_ram_miso,
        
    -- eth1g UDP streaming ports
    udp_tx_sosi_arr          =>  eth1g_udp_tx_sosi_arr,
    udp_tx_siso_arr          =>  eth1g_udp_tx_siso_arr,
    udp_rx_sosi_arr          =>  eth1g_udp_rx_sosi_arr,
    udp_rx_siso_arr          =>  eth1g_udp_rx_siso_arr,

    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    SENS_SC                  => SENS_SC,
    SENS_SD                  => SENS_SD,        
    -- . DDR reference clock domains reset creation
    MB_I_REF_CLK             => MB_I_REF_CLK,
    MB_II_REF_CLK            => MB_II_REF_CLK,
    -- . 1GbE Control Interface
    ETH_CLK                  => ETH_CLK,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- MM master
  -----------------------------------------------------------------------------
  u_mmm : ENTITY work.mmm_unb2_test
  GENERIC MAP (
    g_sim               => g_sim,
    g_sim_unb_nr        => g_sim_unb_nr,
    g_sim_node_nr       => g_sim_node_nr,
    g_technology        => g_technology,
    g_bg_block_size     => c_bg_block_size,
    g_hdr_field_arr     => c_hdr_field_arr,
    g_nof_streams_1GbE  => c_unb2_board_nof_eth,
    g_nof_streams_qsfp  => c_unb2_board_tr_qsfp.nof_bus * c_unb2_board_tr_qsfp.bus_w,
    g_nof_streams_ring  => c_unb2_board_tr_ring.nof_bus * c_unb2_board_tr_ring.bus_w,
    g_nof_streams_back0 => c_unb2_board_tr_back.bus_w,
    g_nof_streams_back1 => c_unb2_board_tr_back.bus_w
   )
  PORT MAP(  
    mm_rst                   => mm_rst,
    mm_clk                   => mm_clk,       

    -- PIOs
    pout_wdi                 => pout_wdi,

    -- Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,

    -- system_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso,
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 

    -- UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso, 
 
    -- PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso, 
  
    -- eth1g ch0
    eth1g_eth0_mm_rst        => eth1g_eth0_mm_rst,
    eth1g_eth0_tse_mosi      => eth1g_eth0_tse_mosi,
    eth1g_eth0_tse_miso      => eth1g_eth0_tse_miso,
    eth1g_eth0_reg_mosi      => eth1g_eth0_reg_mosi,
    eth1g_eth0_reg_miso      => eth1g_eth0_reg_miso,
    eth1g_eth0_reg_interrupt => eth1g_eth0_reg_interrupt,
    eth1g_eth0_ram_mosi      => eth1g_eth0_ram_mosi,
    eth1g_eth0_ram_miso      => eth1g_eth0_ram_miso,

    -- eth1g ch1
    eth1g_eth1_mm_rst        => eth1g_eth1_mm_rst,
    eth1g_eth1_tse_mosi      => eth1g_eth1_tse_mosi,
    eth1g_eth1_tse_miso      => eth1g_eth1_tse_miso,
    eth1g_eth1_reg_mosi      => eth1g_eth1_reg_mosi,
    eth1g_eth1_reg_miso      => eth1g_eth1_reg_miso,
    eth1g_eth1_reg_interrupt => eth1g_eth1_reg_interrupt,
    eth1g_eth1_ram_mosi      => eth1g_eth1_ram_mosi,
    eth1g_eth1_ram_miso      => eth1g_eth1_ram_miso,

    -- EPCS read
    reg_dpmm_data_mosi       => reg_dpmm_data_mosi,
    reg_dpmm_data_miso       => reg_dpmm_data_miso,
    reg_dpmm_ctrl_mosi       => reg_dpmm_ctrl_mosi,
    reg_dpmm_ctrl_miso       => reg_dpmm_ctrl_miso,

    -- EPCS write
    reg_mmdp_data_mosi       => reg_mmdp_data_mosi,
    reg_mmdp_data_miso       => reg_mmdp_data_miso,
    reg_mmdp_ctrl_mosi       => reg_mmdp_ctrl_mosi,
    reg_mmdp_ctrl_miso       => reg_mmdp_ctrl_miso,

    -- EPCS status/control
    reg_epcs_mosi            => reg_epcs_mosi,
    reg_epcs_miso            => reg_epcs_miso,

    -- Remote Update
    reg_remu_mosi            => reg_remu_mosi,
    reg_remu_miso            => reg_remu_miso,

    -- block gen
    ram_diag_bg_1GbE_mosi          => ram_diag_bg_1GbE_mosi,
    ram_diag_bg_1GbE_miso          => ram_diag_bg_1GbE_miso,
    reg_diag_bg_1GbE_mosi          => reg_diag_bg_1GbE_mosi,
    reg_diag_bg_1GbE_miso          => reg_diag_bg_1GbE_miso,
    reg_diag_tx_seq_1GbE_mosi      => reg_diag_tx_seq_1GbE_mosi,
    reg_diag_tx_seq_1GbE_miso      => reg_diag_tx_seq_1GbE_miso,

    ram_diag_bg_10GbE_mosi         => ram_diag_bg_10GbE_mosi,
    ram_diag_bg_10GbE_miso         => ram_diag_bg_10GbE_miso,
    reg_diag_bg_10GbE_mosi         => reg_diag_bg_10GbE_mosi,
    reg_diag_bg_10GbE_miso         => reg_diag_bg_10GbE_miso,
    reg_diag_tx_seq_10GbE_mosi     => reg_diag_tx_seq_10GbE_mosi,
    reg_diag_tx_seq_10GbE_miso     => reg_diag_tx_seq_10GbE_miso,

    -- dp_offload_tx
--    reg_dp_offload_tx_1GbE_mosi          => reg_dp_offload_tx_1GbE_mosi,
--    reg_dp_offload_tx_1GbE_miso          => reg_dp_offload_tx_1GbE_miso,
--    reg_dp_offload_tx_1GbE_hdr_dat_mosi  => reg_dp_offload_tx_1GbE_hdr_dat_mosi,
--    reg_dp_offload_tx_1GbE_hdr_dat_miso  => reg_dp_offload_tx_1GbE_hdr_dat_miso,
--
--    -- dp_offload_rx
--    reg_dp_offload_rx_1GbE_hdr_dat_mosi  => reg_dp_offload_rx_1GbE_hdr_dat_mosi,
--    reg_dp_offload_rx_1GbE_hdr_dat_miso  => reg_dp_offload_rx_1GbE_hdr_dat_miso,

    -- bsn
    reg_bsn_monitor_1GbE_mosi      => reg_bsn_monitor_1GbE_mosi,
    reg_bsn_monitor_1GbE_miso      => reg_bsn_monitor_1GbE_miso,
    reg_bsn_monitor_10GbE_mosi     => reg_bsn_monitor_10GbE_mosi,
    reg_bsn_monitor_10GbE_miso     => reg_bsn_monitor_10GbE_miso,

    -- databuffer
    ram_diag_data_buf_1GbE_mosi    => ram_diag_data_buf_1GbE_mosi,
    ram_diag_data_buf_1GbE_miso    => ram_diag_data_buf_1GbE_miso,
    reg_diag_data_buf_1GbE_mosi    => reg_diag_data_buf_1GbE_mosi,
    reg_diag_data_buf_1GbE_miso    => reg_diag_data_buf_1GbE_miso,
    reg_diag_rx_seq_1GbE_mosi      => reg_diag_rx_seq_1GbE_mosi,
    reg_diag_rx_seq_1GbE_miso      => reg_diag_rx_seq_1GbE_miso,

    ram_diag_data_buf_10GbE_mosi   => ram_diag_data_buf_10GbE_mosi,
    ram_diag_data_buf_10GbE_miso   => ram_diag_data_buf_10GbE_miso,
    reg_diag_data_buf_10GbE_mosi   => reg_diag_data_buf_10GbE_mosi,
    reg_diag_data_buf_10GbE_miso   => reg_diag_data_buf_10GbE_miso,
    reg_diag_rx_seq_10GbE_mosi     => reg_diag_rx_seq_10GbE_mosi,
    reg_diag_rx_seq_10GbE_miso     => reg_diag_rx_seq_10GbE_miso,

    -- 10GbE
    reg_tr_10GbE_qsfp_ring_mosi    => reg_tr_10GbE_qsfp_ring_mosi,
    reg_tr_10GbE_qsfp_ring_miso    => reg_tr_10GbE_qsfp_ring_miso,

    reg_tr_10GbE_back0_mosi        => reg_tr_10GbE_back0_mosi,
    reg_tr_10GbE_back0_miso        => reg_tr_10GbE_back0_miso,

    reg_tr_10GbE_back1_mosi        => reg_tr_10GbE_back1_mosi,
    reg_tr_10GbE_back1_miso        => reg_tr_10GbE_back1_miso,

    reg_diag_tx_seq_ddr_mosi       => reg_diag_tx_seq_ddr_mosi,
    reg_diag_tx_seq_ddr_miso       => reg_diag_tx_seq_ddr_miso,
    reg_diag_rx_seq_ddr_mosi       => reg_diag_rx_seq_ddr_mosi,
    reg_diag_rx_seq_ddr_miso       => reg_diag_rx_seq_ddr_miso,
    reg_diag_data_buf_ddr_mosi     => reg_diag_data_buf_ddr_mosi,
    reg_diag_data_buf_ddr_miso     => reg_diag_data_buf_ddr_miso,
    ram_diag_data_buf_ddr_mosi     => ram_diag_data_buf_ddr_mosi,
    ram_diag_data_buf_ddr_miso     => ram_diag_data_buf_ddr_miso,
    reg_io_ddr_mosi                => reg_io_ddr_mosi,
    reg_io_ddr_miso                => reg_io_ddr_miso
  );


  gen_udp_stream_1GbE : IF c_use_1GbE = TRUE GENERATE
    u_udp_stream_1GbE : ENTITY work.udp_stream
    GENERIC MAP (
      g_sim                       => g_sim,
      g_nof_streams               => c_nof_streams_1GbE,
      g_data_w                    => c_data_w_32,
      g_bg_block_size             => c_bg_block_size,
      g_bg_gapsize                => c_bg_gapsize,
      g_bg_blocks_per_sync        => c_bg_blocks_per_sync,
      g_def_block_size            => c_def_1GbE_block_size,
      g_max_nof_blocks_per_packet => c_max_nof_blocks_per_packet_1GbE,
      g_remove_crc                => TRUE
    )
    PORT MAP (
      mm_rst                         => mm_rst,
      mm_clk                         => mm_clk,

      dp_rst                         => dp_rst,
      dp_clk                         => dp_clk,

      ID                             => ID,

      -- blockgen MM
      reg_diag_bg_mosi               => reg_diag_bg_1GbE_mosi,
      reg_diag_bg_miso               => reg_diag_bg_1GbE_miso,
      ram_diag_bg_mosi               => ram_diag_bg_1GbE_mosi,
      ram_diag_bg_miso               => ram_diag_bg_1GbE_miso,
      reg_diag_tx_seq_mosi           => reg_diag_tx_seq_1GbE_mosi,
      reg_diag_tx_seq_miso           => reg_diag_tx_seq_1GbE_miso,

      -- dp_offload_tx
--      reg_dp_offload_tx_mosi         => reg_dp_offload_tx_1GbE_mosi,
--      reg_dp_offload_tx_miso         => reg_dp_offload_tx_1GbE_miso,
--      reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_1GbE_hdr_dat_mosi,
--      reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_1GbE_hdr_dat_miso,
      dp_offload_tx_src_out_arr      => dp_offload_tx_1GbE_src_out_arr,
      dp_offload_tx_src_in_arr       => dp_offload_tx_1GbE_src_in_arr,

      -- dp_offload_rx
--      reg_dp_offload_rx_hdr_dat_mosi => reg_dp_offload_rx_1GbE_hdr_dat_mosi,
--      reg_dp_offload_rx_hdr_dat_miso => reg_dp_offload_rx_1GbE_hdr_dat_miso,
      dp_offload_rx_snk_in_arr       => dp_offload_rx_1GbE_snk_in_arr,
      dp_offload_rx_snk_out_arr      => dp_offload_rx_1GbE_snk_out_arr,

      -- bsn
      reg_bsn_monitor_mosi           => reg_bsn_monitor_1GbE_mosi,
      reg_bsn_monitor_miso           => reg_bsn_monitor_1GbE_miso,

      -- databuffer
      reg_diag_data_buf_mosi         => reg_diag_data_buf_1GbE_mosi,
      reg_diag_data_buf_miso         => reg_diag_data_buf_1GbE_miso,
      ram_diag_data_buf_mosi         => ram_diag_data_buf_1GbE_mosi,
      ram_diag_data_buf_miso         => ram_diag_data_buf_1GbE_miso,
      reg_diag_rx_seq_mosi           => reg_diag_rx_seq_1GbE_mosi,
      reg_diag_rx_seq_miso           => reg_diag_rx_seq_1GbE_miso
    );
  END GENERATE;

  -----------------------------------------------------------------------------
  -- Interface : 1GbE
  -----------------------------------------------------------------------------
  gen_wires_1GbE : IF c_use_1GbE=TRUE GENERATE
    gen_1GbE_wires : FOR i IN 0 TO c_nof_streams_1GbE-1 GENERATE

      eth1g_udp_tx_sosi_arr(i)         <= dp_offload_tx_1GbE_src_out_arr(i);
      dp_offload_tx_1GbE_src_in_arr(i) <= eth1g_udp_tx_siso_arr(i);
      dp_offload_rx_1GbE_snk_in_arr(i) <= eth1g_udp_rx_sosi_arr(i);
      eth1g_udp_rx_siso_arr(i)         <= dp_offload_rx_1GbE_snk_out_arr(i);

    END GENERATE;
  END GENERATE;


  gen_udp_stream_10GbE : IF c_use_10GbE = TRUE GENERATE
    u_udp_stream_10GbE : ENTITY work.udp_stream
    GENERIC MAP (
      g_sim                       => g_sim,
      g_nof_streams               => c_nof_streams_qsfp + c_nof_streams_ring + c_nof_streams_back0 + c_nof_streams_back1,
      g_data_w                    => c_data_w_64,
      g_bg_block_size             => c_bg_block_size,
      g_bg_gapsize                => c_bg_gapsize,
      g_bg_blocks_per_sync        => c_bg_blocks_per_sync,
      g_def_block_size            => c_def_10GbE_block_size,
      g_max_nof_blocks_per_packet => c_max_nof_blocks_per_packet_10GbE,
      g_remove_crc                => FALSE
    )
    PORT MAP (
      mm_rst                         => mm_rst,
      mm_clk                         => mm_clk,
      dp_rst                         => dp_rst,
      dp_clk                         => dp_clk,
      ID                             => ID,
      -- blockgen MM
      reg_diag_bg_mosi               => reg_diag_bg_10GbE_mosi,
      reg_diag_bg_miso               => reg_diag_bg_10GbE_miso,
      ram_diag_bg_mosi               => ram_diag_bg_10GbE_mosi,
      ram_diag_bg_miso               => ram_diag_bg_10GbE_miso,
      reg_diag_tx_seq_mosi           => reg_diag_tx_seq_10GbE_mosi,
      reg_diag_tx_seq_miso           => reg_diag_tx_seq_10GbE_miso,

      dp_offload_tx_src_out_arr      => dp_offload_tx_10GbE_src_out_arr,
      dp_offload_tx_src_in_arr       => dp_offload_tx_10GbE_src_in_arr,

      dp_offload_rx_snk_in_arr       => dp_offload_rx_10GbE_snk_in_arr,
      dp_offload_rx_snk_out_arr      => dp_offload_rx_10GbE_snk_out_arr,

      reg_bsn_monitor_mosi           => reg_bsn_monitor_10GbE_mosi,
      reg_bsn_monitor_miso           => reg_bsn_monitor_10GbE_miso,

      reg_diag_data_buf_mosi         => reg_diag_data_buf_10GbE_mosi,
      reg_diag_data_buf_miso         => reg_diag_data_buf_10GbE_miso,
      ram_diag_data_buf_mosi         => ram_diag_data_buf_10GbE_mosi,
      ram_diag_data_buf_miso         => ram_diag_data_buf_10GbE_miso,
      reg_diag_rx_seq_mosi           => reg_diag_rx_seq_10GbE_mosi,
      reg_diag_rx_seq_miso           => reg_diag_rx_seq_10GbE_miso
    );


--  u_sa_clk_buf : ENTITY tech_clkbuf_lib.tech_clkbuf
--  GENERIC MAP (
--    g_technology   => g_technology,
--    g_clock_net    => "GLOBAL"
--  )
--  PORT MAP (
--    inclk  => SA_CLK,
--    outclk => SA_CLK_buf
--  );

    u_tr_10GbE_qsfp_and_ring: ENTITY unb2_board_lib.unb2_board_10gbe -- QSFP and Ring lines
    GENERIC MAP (
      g_technology    => g_technology,
      g_sim           => g_sim,
      g_sim_level     => 1,
      g_nof_macs      => c_nof_streams_qsfp + c_nof_streams_ring,
      g_tx_fifo_fill  => c_def_10GbE_block_size,
      g_tx_fifo_size  => c_def_10GbE_block_size*2
    )
    PORT MAP (
      tr_ref_clk          => SA_CLK,
      mm_rst              => mm_rst,
      mm_clk              => mm_clk,
      reg_mac_mosi        => reg_tr_10GbE_qsfp_ring_mosi,
      reg_mac_miso        => reg_tr_10GbE_qsfp_ring_miso,
      dp_rst              => dp_rst,
      dp_clk              => dp_clk,

      src_out_arr         => dp_offload_rx_10GbE_snk_in_arr(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0),
      src_in_arr          => dp_offload_rx_10GbE_snk_out_arr(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0),
      snk_out_arr         => dp_offload_tx_10GbE_src_in_arr(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0),
      snk_in_arr          => dp_offload_tx_10GbE_src_out_arr(c_nof_streams_qsfp+c_nof_streams_ring-1 DOWNTO 0),

      serial_tx_arr       => i_serial_10G_tx_qsfp_ring_arr,
      serial_rx_arr       => i_serial_10G_rx_qsfp_ring_arr
    );

    gen_qsfp_wires: FOR i IN 0 TO c_nof_streams_qsfp-1 GENERATE
        serial_10G_tx_qsfp_arr(i)      <= i_serial_10G_tx_qsfp_ring_arr(i);
      i_serial_10G_rx_qsfp_ring_arr(i) <=   serial_10G_rx_qsfp_arr(i);
    END GENERATE;


    i_QSFP_RX(0) <= QSFP_0_RX;
    i_QSFP_RX(1) <= QSFP_1_RX;
    i_QSFP_RX(2) <= QSFP_2_RX;
    i_QSFP_RX(3) <= QSFP_3_RX;
    i_QSFP_RX(4) <= QSFP_4_RX;
    i_QSFP_RX(5) <= QSFP_5_RX;

    QSFP_0_TX <= i_QSFP_TX(0);
    QSFP_1_TX <= i_QSFP_TX(1);
    QSFP_2_TX <= i_QSFP_TX(2);
    QSFP_3_TX <= i_QSFP_TX(3);
    QSFP_4_TX <= i_QSFP_TX(4);
    QSFP_5_TX <= i_QSFP_TX(5);



    u_front_io : ENTITY unb2_board_lib.unb2_board_front_io
    GENERIC MAP (
      g_nof_qsfp_bus => c_nof_qsfp_bus
    )
    PORT MAP (
      serial_tx_arr => serial_10G_tx_qsfp_arr,
      serial_rx_arr => serial_10G_rx_qsfp_arr,

      green_led_arr => qsfp_green_led_arr(c_nof_qsfp_bus-1 DOWNTO 0),
      red_led_arr   => qsfp_red_led_arr(c_nof_qsfp_bus-1 DOWNTO 0),

      QSFP_RX    => i_QSFP_RX,
      QSFP_TX    => i_QSFP_TX,

      --QSFP_SDA   => QSFP_SDA,
      --QSFP_SCL   => QSFP_SCL,
      --QSFP_RST   => QSFP_RST,

      QSFP_LED   => QSFP_LED
    );


    gen_ring_wires: FOR i IN 0 TO c_nof_streams_ring-1 GENERATE
        serial_10G_tx_ring_arr(i) <= i_serial_10G_tx_qsfp_ring_arr(i+c_nof_streams_qsfp);
      i_serial_10G_rx_qsfp_ring_arr(i+c_nof_streams_qsfp) <= serial_10G_rx_ring_arr(i);
    END GENERATE;

    i_RING_RX(0) <= RING_0_RX;
    i_RING_RX(1) <= RING_1_RX;
    RING_0_TX <= i_RING_TX(0);
    RING_1_TX <= i_RING_TX(1);

    u_ring_io : ENTITY unb2_board_lib.unb2_board_ring_io
    GENERIC MAP (
      g_nof_ring_bus => c_nof_ring_bus
    )
    PORT MAP (
      serial_tx_arr => serial_10G_tx_ring_arr,
      serial_rx_arr => serial_10G_rx_ring_arr,
      RING_RX => i_RING_RX,
      RING_TX => i_RING_TX
    );




    u_front_led : ENTITY unb2_board_lib.unb2_board_qsfp_leds
    GENERIC MAP (
      g_sim             => g_sim,
      g_factory_image   => g_factory_image,
      g_nof_qsfp        => c_nof_qsfp_bus,
      g_pulse_us        => 1000 / (10**9 / c_unb2_board_ext_clk_freq_200M) -- nof clk cycles to get us period
    )
    PORT MAP (
      rst               => dp_rst,
      clk               => dp_clk,
      tx_siso_arr       => dp_offload_tx_10GbE_src_in_arr(c_nof_streams_qsfp-1 DOWNTO 0),
      tx_sosi_arr       => dp_offload_tx_10GbE_src_out_arr(c_nof_streams_qsfp-1 DOWNTO 0),
      rx_sosi_arr       => dp_offload_rx_10GbE_snk_in_arr(c_nof_streams_qsfp-1 DOWNTO 0),
      green_led_arr     => qsfp_green_led_arr(c_nof_qsfp_bus-1 DOWNTO 0),
      red_led_arr       => qsfp_red_led_arr(c_nof_qsfp_bus-1 DOWNTO 0)
    );
  END GENERATE;

  gen_no_udp_stream_10GbE : IF c_use_10GbE = FALSE GENERATE
    u_front_io : ENTITY unb2_board_lib.unb2_board_front_io
    GENERIC MAP (
      g_nof_qsfp_bus => c_unb2_board_tr_qsfp.nof_bus
    )
    PORT MAP (
      green_led_arr => qsfp_green_led_arr,
      red_led_arr   => qsfp_red_led_arr,
      QSFP_LED      => QSFP_LED
    );
    u_front_led : ENTITY unb2_board_lib.unb2_board_qsfp_leds
    GENERIC MAP (
      g_sim           => g_sim,
      g_factory_image => g_factory_image,
      g_nof_qsfp      => c_unb2_board_tr_qsfp.nof_bus,
      g_pulse_us      => 1000 / (10**9 / c_mm_clk_freq) -- nof clk cycles to get us period
    )
    PORT MAP (
      rst             => mm_rst,
      clk             => mm_clk,
      green_led_arr   => qsfp_green_led_arr,
      red_led_arr     => qsfp_red_led_arr
    );
  END GENERATE;


  gen_stream_MB_I : IF c_use_MB_I = TRUE OR c_use_MB_both=TRUE GENERATE
    u_mms_io_ddr_diag : ENTITY io_ddr_lib.mms_io_ddr_diag
    GENERIC MAP (
      -- System
      g_sim_model_ddr   => g_sim_model_ddr,
      g_technology      => g_technology,
      
      g_dp_data_w       => c_ddr_dp_data_w,      -- DP data width, func_tech_ddr_ctlr_data_w(g_io_tech_ddr)/g_dp_data_w must be a power of 2 due to the mixed width FIFO
      g_dp_seq_dat_w    => c_ddr_dp_seq_dat_w,   -- >= 1, test sequence data width. Choose g_dp_seq_dat_w <= g_dp_data_w. The seq data gets replicated to fill g_dp_data_w.
      g_dp_fifo_depth   => c_ddr_dp_fifo_depth,  -- >= 2048, write FIFO depth and read FIFO depth at DP side of the FIFOs
      
      -- IO_DDR
      g_io_tech_ddr     => g_ddr_MB_I,
      
      -- DIAG data buffer
      g_db_use_db       => FALSE,
      g_db_buf_nof_data => c_ddr_db_buf_nof_data    -- nof words per data buffer
    )
    PORT MAP (
      ---------------------------------------------------------------------------
      -- System
      ---------------------------------------------------------------------------
      mm_rst              => mm_rst,
      mm_clk              => mm_clk,
      dp_rst              => dp_rst,
      dp_clk              => dp_clk,  -- use alternative external clock or externally connect to ctlr_clk_out
      
      ---------------------------------------------------------------------------
      -- IO_DDR
      ---------------------------------------------------------------------------
      -- DDR reference clock
      ctlr_ref_clk        => MB_I_REF_CLK,
      ctlr_ref_rst        => mb_I_ref_rst,

      -- DDR controller clock domain
      ctlr_clk_out        => ddr_I_clk200,
      ctlr_rst_out        => ddr_I_rst200,
  
      ctlr_clk_in         => ddr_I_clk200,  -- connect ctlr_clk_out to ctlr_clk_in at top level to avoid potential delta-cycle differences between the same clock
      ctlr_rst_in         => ddr_I_rst200,  -- connect ctlr_rst_out to ctlr_rst_in at top level
  
      -- MM interface
      reg_io_ddr_mosi     => reg_io_ddr_mosi,  -- register for DDR controller status info
      reg_io_ddr_miso     => reg_io_ddr_miso,
  
      -- Write / read FIFO status for monitoring purposes (in dp_clk domain)
      wr_fifo_usedw       => OPEN,
      rd_fifo_usedw       => OPEN,
  
      -- DDR4 PHY external interface
      phy4_in             => MB_I_IN,
      phy4_io             => MB_I_IO,
      phy4_ou             => MB_I_OU, 
  
      ---------------------------------------------------------------------------
      -- DIAG Tx seq
      ---------------------------------------------------------------------------
      -- MM interface
      reg_tx_seq_mosi   => reg_diag_tx_seq_ddr_mosi,
      reg_tx_seq_miso   => reg_diag_tx_seq_ddr_miso,
      
      ---------------------------------------------------------------------------
      -- DIAG rx seq with optional data buffer
      ---------------------------------------------------------------------------
      -- MM interface
      reg_data_buf_mosi => reg_diag_data_buf_ddr_mosi,
      reg_data_buf_miso => reg_diag_data_buf_ddr_miso,
  
      ram_data_buf_mosi => ram_diag_data_buf_ddr_mosi,
      ram_data_buf_miso => ram_diag_data_buf_ddr_miso,
  
      reg_rx_seq_mosi   => reg_diag_rx_seq_ddr_mosi,
      reg_rx_seq_miso   => reg_diag_rx_seq_ddr_miso
    );  
  END GENERATE;

  gen_stream_MB_II : IF c_use_MB_II = TRUE OR c_use_MB_both=TRUE GENERATE
  END GENERATE;

END str;

