-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Output frame busy control signal for array of streams
-- Description:
--   See dp_frame_busy.

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_frame_busy_arr IS
  GENERIC (
    g_nof_inputs : NATURAL := 1;
    g_pipeline   : NATURAL := 0
  );
  PORT (
    rst             : IN  STD_LOGIC;
    clk             : IN  STD_LOGIC;
    snk_in_arr      : IN  t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0);
    snk_in_busy_arr : OUT STD_LOGIC_VECTOR(g_nof_inputs-1 DOWNTO 0)
  );
END dp_frame_busy_arr;


ARCHITECTURE str OF dp_frame_busy_arr IS  
BEGIN

  gen_nof_inputs : FOR I IN 0 TO g_nof_inputs-1 GENERATE
    u_dp_frame_busy : ENTITY work.dp_frame_busy
    GENERIC MAP (
      g_pipeline => g_pipeline
    )
    PORT MAP (
      rst         => rst,
      clk         => clk,
      snk_in      => snk_in_arr(I),
      snk_in_busy => snk_in_busy_arr(I)
    );
  END GENERATE;

END str;
