--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

-- Purpose:
-- . Provide an array of multipliers with streaming I/O
-- Description:
-- . Multiplies snk_in_2arr_2[i][0] with snk_in_2arr_2[i][1] yielding 
--   src_out_2arr[i] for i in 0..g_nof_inputs-1.

ENTITY corr_multiplier IS
  GENERIC (
    g_nof_inputs  : NATURAL;
    g_data_w      : NATURAL;
    g_conjugate   : BOOLEAN -- Take conjugate of snk_in_2arr_2[i][1]
   ); 
  PORT (
    rst            : IN  STD_LOGIC;
    clk            : IN  STD_LOGIC;

    snk_in_2arr_2  : IN  t_dp_sosi_2arr_2(g_nof_inputs-1 DOWNTO 0);

    src_out_arr   : OUT t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0)
  );
END corr_multiplier;

ARCHITECTURE str OF corr_multiplier IS

  SIGNAL common_complex_mult_src_out_arr : t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0);

BEGIN

  -----------------------------------------------------------------------------
  -- Complex Multipliers
  -----------------------------------------------------------------------------
  gen_common_complex_mult : FOR i IN 0 TO g_nof_inputs-1 GENERATE
    --u_common_complex_mult : ENTITY common_lib.common_complex_mult(rtl)
    --u_common_complex_mult : ENTITY common_lib.common_complex_mult(rtl_dsp)
    --u_common_complex_mult : ENTITY common_lib.common_complex_mult(altera_rtl)
    --u_common_complex_mult : ENTITY common_lib.common_complex_mult(str)
    u_common_complex_mult : ENTITY common_lib.common_complex_mult(stratix4)
    --u_dut : ENTITY common_lib.common_complex_mult(str_stratix4)
    GENERIC MAP (
      g_in_a_w           => g_data_w,
      g_in_b_w           => g_data_w,
      g_out_p_w          => 2*g_data_w,     -- default use g_out_p_w = g_in_a_w+g_in_b_w
      g_conjugate_b      => g_conjugate,
      g_pipeline_input   => 1,
      g_pipeline_product => 0,
      g_pipeline_adder   => 1,
      g_pipeline_output  => 1
    )
    PORT MAP (
      clk        => clk,
      clken      => '1',
      rst        => '0',
      in_ar      => snk_in_2arr_2(i)(0).re(g_data_w-1 DOWNTO 0),
      in_ai      => snk_in_2arr_2(i)(0).im(g_data_w-1 DOWNTO 0),
      in_br      => snk_in_2arr_2(i)(1).re(g_data_w-1 DOWNTO 0),
      in_bi      => snk_in_2arr_2(i)(1).im(g_data_w-1 DOWNTO 0),
      in_val     => snk_in_2arr_2(i)(0).valid,
      out_pr     => common_complex_mult_src_out_arr(i).re(2*g_data_w-1 DOWNTO 0),
      out_pi     => common_complex_mult_src_out_arr(i).im(2*g_data_w-1 DOWNTO 0),
      out_val    => common_complex_mult_src_out_arr(i).valid
    );     

    src_out_arr(i).re    <= RESIZE_DP_DSP_DATA(common_complex_mult_src_out_arr(i).re(2*g_data_w-1 DOWNTO 0));
    src_out_arr(i).im    <= RESIZE_DP_DSP_DATA(common_complex_mult_src_out_arr(i).im(2*g_data_w-1 DOWNTO 0));
    src_out_arr(i).valid <= common_complex_mult_src_out_arr(i).valid;  
  END GENERATE;
  
END str;
