-------------------------------------------------------------------------------
-- TOP LEVEL 
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity pipeline is
  port (

    data_in_Data : in std_logic_vector(31 downto 0 );
    data_in_Control : in std_logic;
    data_in_Read : out std_logic;
    data_in_Exists : in std_logic;

    data_out_Data : out std_logic_vector(31 downto 0 );
    data_out_Control : out std_logic;
    data_out_Write : out std_logic;
    data_out_Full : in std_logic;

    TEST_STOP : out std_logic_vector(9 downto 0 );
    TEST_ERROR : out std_logic_vector(9 downto 0 );
    TEST_FIFO_FULL : out std_logic_vector(8 downto 0 );
    TEST_BLOCK_RD : out std_logic_vector(9 downto 0 );
    address : in std_logic_vector(18 downto 0 );
    read_data : out std_logic_vector(31 downto 0 );
    read_en : in std_logic;
    write_en : in std_logic;
    write_data : in std_logic_vector(31 downto 0 );
    pci_clk : in std_logic;

    KPN_CLK : in std_logic;
    KPN_RST : in std_logic
  );
end pipeline;

architecture STRUCTURE of pipeline is

  component pipeline2rtl_hwn_nd_2_ip_wrapper is
    port (
		data_in_Rd : out std_logic;
		data_in_Din : in std_logic_vector(31 downto 0);
		data_in_Exist : in std_logic;
		data_in_CLK : out std_logic;
		data_in_CTRL : in std_logic;

		ND_2OP_1_Wr : out std_logic;
		ND_2OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_2OP_1_Full : in std_logic;
		ND_2OP_1_CLK : out std_logic;
		ND_2OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_3_ip_wrapper is
    port (
		ND_3IP_1_Rd : out std_logic;
		ND_3IP_1_Din : in std_logic_vector(31 downto 0);
		ND_3IP_1_Exist : in std_logic;
		ND_3IP_1_CLK : out std_logic;
		ND_3IP_1_CTRL : in std_logic;

		ND_3OP_1_Wr : out std_logic;
		ND_3OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_3OP_1_Full : in std_logic;
		ND_3OP_1_CLK : out std_logic;
		ND_3OP_1_CTRL : out std_logic;

		ND_3OP_1_d1_Wr : out std_logic;
		ND_3OP_1_d1_Dout : out std_logic_vector(31 downto 0);
		ND_3OP_1_d1_Full : in std_logic;
		ND_3OP_1_d1_CLK : out std_logic;
		ND_3OP_1_d1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_3_PARAM_FIFO_FULL : out std_logic;
		ND_3_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_4_ip_wrapper is
    port (
		ND_4IP_2_Rd : out std_logic;
		ND_4IP_2_Din : in std_logic_vector(31 downto 0);
		ND_4IP_2_Exist : in std_logic;
		ND_4IP_2_CLK : out std_logic;
		ND_4IP_2_CTRL : in std_logic;

		ND_4OP_1_Wr : out std_logic;
		ND_4OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_4OP_1_Full : in std_logic;
		ND_4OP_1_CLK : out std_logic;
		ND_4OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_5_ip_wrapper is
    port (
		ND_5IP_3_Rd : out std_logic;
		ND_5IP_3_Din : in std_logic_vector(31 downto 0);
		ND_5IP_3_Exist : in std_logic;
		ND_5IP_3_CLK : out std_logic;
		ND_5IP_3_CTRL : in std_logic;

		ND_5OP_1_Wr : out std_logic;
		ND_5OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_5OP_1_Full : in std_logic;
		ND_5OP_1_CLK : out std_logic;
		ND_5OP_1_CTRL : out std_logic;

		ND_5OP_1_d1_Wr : out std_logic;
		ND_5OP_1_d1_Dout : out std_logic_vector(31 downto 0);
		ND_5OP_1_d1_Full : in std_logic;
		ND_5OP_1_d1_CLK : out std_logic;
		ND_5OP_1_d1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_5_PARAM_FIFO_FULL : out std_logic;
		ND_5_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_6_ip_wrapper is
    port (
		ND_6IP_4_Rd : out std_logic;
		ND_6IP_4_Din : in std_logic_vector(31 downto 0);
		ND_6IP_4_Exist : in std_logic;
		ND_6IP_4_CLK : out std_logic;
		ND_6IP_4_CTRL : in std_logic;

		ND_6OP_1_Wr : out std_logic;
		ND_6OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_6OP_1_Full : in std_logic;
		ND_6OP_1_CLK : out std_logic;
		ND_6OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_6_PARAM_FIFO_FULL : out std_logic;
		ND_6_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_7_ip_wrapper is
    port (
		ND_7IP_5_Rd : out std_logic;
		ND_7IP_5_Din : in std_logic_vector(31 downto 0);
		ND_7IP_5_Exist : in std_logic;
		ND_7IP_5_CLK : out std_logic;
		ND_7IP_5_CTRL : in std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_7_PARAM_FIFO_FULL : out std_logic;
		ND_7_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_8_ip_wrapper is
    port (
		ND_8IP_6_Rd : out std_logic;
		ND_8IP_6_Din : in std_logic_vector(31 downto 0);
		ND_8IP_6_Exist : in std_logic;
		ND_8IP_6_CLK : out std_logic;
		ND_8IP_6_CTRL : in std_logic;

		ND_8OP_1_Wr : out std_logic;
		ND_8OP_1_Dout : out std_logic_vector(31 downto 0);
		ND_8OP_1_Full : in std_logic;
		ND_8OP_1_CLK : out std_logic;
		ND_8OP_1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_8_PARAM_FIFO_FULL : out std_logic;
		ND_8_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_9_ip_wrapper is
    port (
		ND_9IP_7_Rd : out std_logic;
		ND_9IP_7_Din : in std_logic_vector(31 downto 0);
		ND_9IP_7_Exist : in std_logic;
		ND_9IP_7_CLK : out std_logic;
		ND_9IP_7_CTRL : in std_logic;

		ND_9IP_8_Rd : out std_logic;
		ND_9IP_8_Din : in std_logic_vector(31 downto 0);
		ND_9IP_8_Exist : in std_logic;
		ND_9IP_8_CLK : out std_logic;
		ND_9IP_8_CTRL : in std_logic;

		ND_9OP_1_ND_9OP_1_d1_Wr : out std_logic;
		ND_9OP_1_ND_9OP_1_d1_Dout : out std_logic_vector(31 downto 0);
		ND_9OP_1_ND_9OP_1_d1_Full : in std_logic;
		ND_9OP_1_ND_9OP_1_d1_CLK : out std_logic;
		ND_9OP_1_ND_9OP_1_d1_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		ND_9_PARAM_FIFO_FULL : out std_logic;
		ND_9_SYNC_NUM : out std_logic_vector(9 downto 0);
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

  component pipeline2rtl_hwn_nd_10_ip_wrapper is
    port (
		ND_10IP_9_ND_10IP_10_Rd : out std_logic;
		ND_10IP_9_ND_10IP_10_Din : in std_logic_vector(31 downto 0);
		ND_10IP_9_ND_10IP_10_Exist : in std_logic;
		ND_10IP_9_ND_10IP_10_CLK : out std_logic;
		ND_10IP_9_ND_10IP_10_CTRL : in std_logic;

		data_out_Wr : out std_logic;
		data_out_Dout : out std_logic_vector(31 downto 0);
		data_out_Full : in std_logic;
		data_out_CLK : out std_logic;
		data_out_CTRL : out std_logic;

		PARAM_DT : in std_logic_vector(20 downto 0);
		PARAM_LD : in std_logic;
		STOP : out std_logic;
		ERROR : out std_logic;
		BLOCK_RD : out std_logic;

		RST : in std_logic;
		CLK : in std_logic
	);
	end component;

component pipeline2rtl_control_if_ip_wrapper is
	port (
		PARAM_DT : out std_logic_vector(20 downto 0);
		PARAM_LD : out std_logic;
		SYNC_NUM : in std_logic_vector(9 downto 0);
		PARAM_FIFO_FULL : in std_logic;
		PARAMETERS_IN : in std_logic_vector(10 downto 0);
		PARAMETERS_IN_LD : in std_logic;
		RST : in std_logic;
		CLK : in std_logic

	);
end component;

component pipeline2rtl_param_ext_ip_wrapper is
	port (
		PARAM_SPLIT_X : in std_logic_vector(31 downto 0);
		COMMIT : in std_logic;
		PARAMETERS : out std_logic_vector(10 downto 0);
		PARAMETERS_LD : out std_logic;
		PCI_CLK : in std_logic;
		RST : in std_logic;
		CLK : in std_logic

	);
end component;

component pipeline2rtl_register_rf_ip_wrapper is
	port (
		address : in std_logic_vector(18 downto 0);
		read_data : out std_logic_vector(31 downto 0);
		read_en : in std_logic;
		write_en : in std_logic;
		write_data : in std_logic_vector(31 downto 0);
		pci_clk : in std_logic;
		split_x_rf_read_data : in std_logic_vector(31 downto 0);
		split_x_rf_read_en : out std_logic;
		split_x_rf_write_en : out std_logic;
		split_x_rf_write_data : out std_logic_vector(31 downto 0);
		commit_rf_read_data : in std_logic_vector(31 downto 0);
		commit_rf_read_en : out std_logic;
		commit_rf_write_en : out std_logic;
		commit_rf_write_data : out std_logic_vector(31 downto 0);
		RST : in std_logic;
		CLK : in std_logic

	);
end component;

  component pipeline2rtl_ed_1_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_2_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_3_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_4_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_5_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_6_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_7_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_8_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  component pipeline2rtl_ed_9_10_ip_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic;
      FSL_Full : out std_logic;
      FSL_Has_Data : out std_logic;
      FSL_Control_IRQ : out std_logic
    );
  end component;

  -- Internal signals

	signal signal_ed_1_out_FSL_M_Control : std_logic;
	signal signal_ed_1_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_1_out_FSL_M_Full : std_logic;
	signal signal_ed_1_out_FSL_M_Write : std_logic;
	signal signal_ed_1_in_FSL_S_Control : std_logic;
	signal signal_ed_1_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_1_in_FSL_S_Exists : std_logic;
	signal signal_ed_1_in_FSL_S_Read : std_logic;
	signal signal_ed_2_out_FSL_M_Control : std_logic;
	signal signal_ed_2_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_2_out_FSL_M_Full : std_logic;
	signal signal_ed_2_out_FSL_M_Write : std_logic;
	signal signal_ed_4_out_FSL_M_Control : std_logic;
	signal signal_ed_4_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_4_out_FSL_M_Full : std_logic;
	signal signal_ed_4_out_FSL_M_Write : std_logic;
	signal signal_ed_2_in_FSL_S_Control : std_logic;
	signal signal_ed_2_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_2_in_FSL_S_Exists : std_logic;
	signal signal_ed_2_in_FSL_S_Read : std_logic;
	signal signal_ed_3_out_FSL_M_Control : std_logic;
	signal signal_ed_3_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_3_out_FSL_M_Full : std_logic;
	signal signal_ed_3_out_FSL_M_Write : std_logic;
	signal signal_ed_3_in_FSL_S_Control : std_logic;
	signal signal_ed_3_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_3_in_FSL_S_Exists : std_logic;
	signal signal_ed_3_in_FSL_S_Read : std_logic;
	signal signal_ed_5_out_FSL_M_Control : std_logic;
	signal signal_ed_5_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_5_out_FSL_M_Full : std_logic;
	signal signal_ed_5_out_FSL_M_Write : std_logic;
	signal signal_ed_6_out_FSL_M_Control : std_logic;
	signal signal_ed_6_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_6_out_FSL_M_Full : std_logic;
	signal signal_ed_6_out_FSL_M_Write : std_logic;
	signal signal_ed_4_in_FSL_S_Control : std_logic;
	signal signal_ed_4_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_4_in_FSL_S_Exists : std_logic;
	signal signal_ed_4_in_FSL_S_Read : std_logic;
	signal signal_ed_7_out_FSL_M_Control : std_logic;
	signal signal_ed_7_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_7_out_FSL_M_Full : std_logic;
	signal signal_ed_7_out_FSL_M_Write : std_logic;
	signal signal_ed_5_in_FSL_S_Control : std_logic;
	signal signal_ed_5_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_5_in_FSL_S_Exists : std_logic;
	signal signal_ed_5_in_FSL_S_Read : std_logic;
	signal signal_ed_6_in_FSL_S_Control : std_logic;
	signal signal_ed_6_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_6_in_FSL_S_Exists : std_logic;
	signal signal_ed_6_in_FSL_S_Read : std_logic;
	signal signal_ed_8_out_FSL_M_Control : std_logic;
	signal signal_ed_8_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_8_out_FSL_M_Full : std_logic;
	signal signal_ed_8_out_FSL_M_Write : std_logic;
	signal signal_ed_7_in_FSL_S_Control : std_logic;
	signal signal_ed_7_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_7_in_FSL_S_Exists : std_logic;
	signal signal_ed_7_in_FSL_S_Read : std_logic;
	signal signal_ed_8_in_FSL_S_Control : std_logic;
	signal signal_ed_8_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_8_in_FSL_S_Exists : std_logic;
	signal signal_ed_8_in_FSL_S_Read : std_logic;
	signal signal_ed_9_10_out_FSL_M_Control : std_logic;
	signal signal_ed_9_10_out_FSL_M_Data : std_logic_vector(0 to 31);
	signal signal_ed_9_10_out_FSL_M_Full : std_logic;
	signal signal_ed_9_10_out_FSL_M_Write : std_logic;
	signal signal_ed_9_10_in_FSL_S_Control : std_logic;
	signal signal_ed_9_10_in_FSL_S_Data : std_logic_vector(0 to 31);
	signal signal_ed_9_10_in_FSL_S_Exists : std_logic;
	signal signal_ed_9_10_in_FSL_S_Read : std_logic;

  -- AD HOC Internal signals

	signal signal_hwn_nd_1_STOP : std_logic;
	signal signal_hwn_nd_1_ERROR : std_logic;
	signal signal_hwn_nd_1_BLOCK_RD : std_logic;
	signal signal_hwn_nd_2_STOP : std_logic;
	signal signal_hwn_nd_2_ERROR : std_logic;
	signal signal_hwn_nd_2_BLOCK_RD : std_logic;
	signal signal_hwn_nd_3_STOP : std_logic;
	signal signal_hwn_nd_3_ERROR : std_logic;
	signal signal_hwn_nd_3_BLOCK_RD : std_logic;
	signal signal_hwn_nd_4_STOP : std_logic;
	signal signal_hwn_nd_4_ERROR : std_logic;
	signal signal_hwn_nd_4_BLOCK_RD : std_logic;
	signal signal_hwn_nd_5_STOP : std_logic;
	signal signal_hwn_nd_5_ERROR : std_logic;
	signal signal_hwn_nd_5_BLOCK_RD : std_logic;
	signal signal_hwn_nd_6_STOP : std_logic;
	signal signal_hwn_nd_6_ERROR : std_logic;
	signal signal_hwn_nd_6_BLOCK_RD : std_logic;
	signal signal_hwn_nd_7_STOP : std_logic;
	signal signal_hwn_nd_7_ERROR : std_logic;
	signal signal_hwn_nd_7_BLOCK_RD : std_logic;
	signal signal_hwn_nd_8_STOP : std_logic;
	signal signal_hwn_nd_8_ERROR : std_logic;
	signal signal_hwn_nd_8_BLOCK_RD : std_logic;
	signal signal_hwn_nd_9_STOP : std_logic;
	signal signal_hwn_nd_9_ERROR : std_logic;
	signal signal_hwn_nd_9_BLOCK_RD : std_logic;
	signal signal_hwn_nd_10_STOP : std_logic;
	signal signal_hwn_nd_10_ERROR : std_logic;
	signal signal_hwn_nd_10_BLOCK_RD : std_logic;
	signal signal_ed_1_FIFO_FULL : std_logic;
	signal signal_ed_2_FIFO_FULL : std_logic;
	signal signal_ed_3_FIFO_FULL : std_logic;
	signal signal_ed_4_FIFO_FULL : std_logic;
	signal signal_ed_5_FIFO_FULL : std_logic;
	signal signal_ed_6_FIFO_FULL : std_logic;
	signal signal_ed_7_FIFO_FULL : std_logic;
	signal signal_ed_8_FIFO_FULL : std_logic;
	signal signal_ed_9_10_FIFO_FULL : std_logic;
	signal signal_PARAM_DT : std_logic_vector(20 downto 0);
	signal signal_PARAM_LD : std_logic;
	signal signal_ND_3_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_3_PARAM_FIFO_FULL : std_logic;
	signal signal_ND_5_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_5_PARAM_FIFO_FULL : std_logic;
	signal signal_ND_6_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_6_PARAM_FIFO_FULL : std_logic;
	signal signal_ND_7_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_7_PARAM_FIFO_FULL : std_logic;
	signal signal_ND_8_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_8_PARAM_FIFO_FULL : std_logic;
	signal signal_ND_9_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_ND_9_PARAM_FIFO_FULL : std_logic;
	signal signal_PARAMETERS : std_logic_vector(10 downto 0);
	signal signal_PARAMETERS_LD : std_logic;
	signal signal_COMMIT : std_logic;
	signal signal_PARAMETER_COMMIT_READ : std_logic_vector(31 downto 0);
	signal signal_PARAM_FIFO_FULL : std_logic;
	signal signal_SYNC_NUM : std_logic_vector(9 downto 0);
	signal signal_address : std_logic_vector(18 downto 0);
	signal signal_read_data : std_logic_vector(31 downto 0);
	signal signal_write_data : std_logic_vector(31 downto 0);
	signal signal_read_en : std_logic;
	signal signal_write_en : std_logic;
	signal signal_PARAMETER_SPLIT_X_WRITE : std_logic_vector(31 downto 0);
	signal signal_PARAMETER_SPLIT_X_READ : std_logic_vector(31 downto 0);
	signal signal_PARAMETER_SPLIT_X_WRITE_ENABLE : std_logic;
	signal signal_param_ext_pci_clk : std_logic;
	signal signal_register_rf_pci_clk : std_logic;

  -- Hierarchical signals 

	signal I_data_in_Control : std_logic;
	signal I_data_in_Data : std_logic_vector(31 downto 0);
	signal I_data_in_Exists : std_logic;
	signal I_data_in_Read : std_logic;
	signal I_data_out_Control : std_logic;
	signal I_data_out_Data : std_logic_vector(31 downto 0);
	signal I_data_out_Full : std_logic;
	signal I_data_out_Write : std_logic;

	-- Default signals

	signal net_gnd0 : std_logic;
	signal net_gnd16 : std_logic_vector(15 downto 0);

	signal sys_clk_s : std_logic;
	signal sys_rst_s : std_logic;


	-- START the actual definition of a Design

begin

	-- Connect Clock

	sys_clk_s <= KPN_CLK;
	sys_rst_s <= KPN_RST;
	
	signal_PARAM_FIFO_FULL <= signal_ND_3_PARAM_FIFO_FULL or signal_ND_6_PARAM_FIFO_FULL or signal_ND_7_PARAM_FIFO_FULL or signal_ND_8_PARAM_FIFO_FULL or signal_ND_9_PARAM_FIFO_FULL;
	signal_SYNC_NUM <= signal_nd_3_SYNC_NUM;

	-- Connect Hiercical Interconnections

  -- Hierarchical signals -3- compaandesign.com:pipeline2rtl:design_pipeline:1:data_in
	I_data_in_Data <= data_in_Data;
	I_data_in_Control <= data_in_Control;
	data_in_Read <= I_data_in_Read;
	I_data_in_Exists <= data_in_Exists;
  -- Hierarchical signals -3- compaandesign.com:pipeline2rtl:design_pipeline:1:data_out
	data_out_Data <= I_data_out_Data;
	data_out_Control <= I_data_out_Control;
	data_out_Write <= I_data_out_Write;
	I_data_out_Full <= data_out_Full;

	-- AD HOC EXTERNAL CONNECTIONS 
 -- DESIGN ID : compaandesign.com:pipeline2rtl:design_pipeline:1
	TEST_STOP(0) <= signal_hwn_nd_1_STOP;
	TEST_ERROR(0) <= signal_hwn_nd_1_ERROR;
	TEST_BLOCK_RD(0) <= signal_hwn_nd_1_BLOCK_RD;
	TEST_STOP(1) <= signal_hwn_nd_2_STOP;
	TEST_ERROR(1) <= signal_hwn_nd_2_ERROR;
	TEST_BLOCK_RD(1) <= signal_hwn_nd_2_BLOCK_RD;
	TEST_STOP(2) <= signal_hwn_nd_3_STOP;
	TEST_ERROR(2) <= signal_hwn_nd_3_ERROR;
	TEST_BLOCK_RD(2) <= signal_hwn_nd_3_BLOCK_RD;
	TEST_STOP(3) <= signal_hwn_nd_4_STOP;
	TEST_ERROR(3) <= signal_hwn_nd_4_ERROR;
	TEST_BLOCK_RD(3) <= signal_hwn_nd_4_BLOCK_RD;
	TEST_STOP(4) <= signal_hwn_nd_5_STOP;
	TEST_ERROR(4) <= signal_hwn_nd_5_ERROR;
	TEST_BLOCK_RD(4) <= signal_hwn_nd_5_BLOCK_RD;
	TEST_STOP(5) <= signal_hwn_nd_6_STOP;
	TEST_ERROR(5) <= signal_hwn_nd_6_ERROR;
	TEST_BLOCK_RD(5) <= signal_hwn_nd_6_BLOCK_RD;
	TEST_STOP(6) <= signal_hwn_nd_7_STOP;
	TEST_ERROR(6) <= signal_hwn_nd_7_ERROR;
	TEST_BLOCK_RD(6) <= signal_hwn_nd_7_BLOCK_RD;
	TEST_STOP(7) <= signal_hwn_nd_8_STOP;
	TEST_ERROR(7) <= signal_hwn_nd_8_ERROR;
	TEST_BLOCK_RD(7) <= signal_hwn_nd_8_BLOCK_RD;
	TEST_STOP(8) <= signal_hwn_nd_9_STOP;
	TEST_ERROR(8) <= signal_hwn_nd_9_ERROR;
	TEST_BLOCK_RD(8) <= signal_hwn_nd_9_BLOCK_RD;
	TEST_STOP(9) <= signal_hwn_nd_10_STOP;
	TEST_ERROR(9) <= signal_hwn_nd_10_ERROR;
	TEST_BLOCK_RD(9) <= signal_hwn_nd_10_BLOCK_RD;
	TEST_FIFO_FULL(0) <= signal_ed_1_FIFO_FULL;
	TEST_FIFO_FULL(1) <= signal_ed_2_FIFO_FULL;
	TEST_FIFO_FULL(2) <= signal_ed_3_FIFO_FULL;
	TEST_FIFO_FULL(3) <= signal_ed_4_FIFO_FULL;
	TEST_FIFO_FULL(4) <= signal_ed_5_FIFO_FULL;
	TEST_FIFO_FULL(5) <= signal_ed_6_FIFO_FULL;
	TEST_FIFO_FULL(6) <= signal_ed_7_FIFO_FULL;
	TEST_FIFO_FULL(7) <= signal_ed_8_FIFO_FULL;
	TEST_FIFO_FULL(8) <= signal_ed_9_10_FIFO_FULL;
	signal_address <= address( 18 downto 0);
	read_data( 31 downto 0) <= signal_read_data;
	signal_write_data <= write_data( 31 downto 0);
	signal_read_en <= read_en;
	signal_write_en <= write_en;
	signal_param_ext_pci_clk <= pci_clk;
	signal_register_rf_pci_clk <= pci_clk;

-- Give default signals, default values

	net_gnd0 <= '0';
	net_gnd16(15 downto 0) <= B"0000000000000000";

-- Instanciate the wrappers (HWN and Edges)


  pipeline2rtl_hwn_nd_2_ip : pipeline2rtl_hwn_nd_2_ip_wrapper
	port map (
		data_in_Rd => I_data_in_Read,
		data_in_Din => I_data_in_Data(31 downto 0),
		data_in_Exist => I_data_in_Exists,
		data_in_CLK => open,
		data_in_CTRL => I_data_in_Control,
		ND_2OP_1_Wr => signal_ed_1_out_FSL_M_Write,
		ND_2OP_1_Dout(31 downto 0) => signal_ed_1_out_FSL_M_Data(0 to 31),
		ND_2OP_1_Full => signal_ed_1_out_FSL_M_Full,
		ND_2OP_1_CLK => open,
		ND_2OP_1_CTRL => signal_ed_1_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_2_STOP,
		ERROR => signal_hwn_nd_2_ERROR,
		BLOCK_RD => signal_hwn_nd_2_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_3_ip : pipeline2rtl_hwn_nd_3_ip_wrapper
	port map (
		ND_3IP_1_Rd => signal_ed_1_in_FSL_S_Read,
		ND_3IP_1_Din(31 downto 0) => signal_ed_1_in_FSL_S_Data(0 to 31),
		ND_3IP_1_Exist => signal_ed_1_in_FSL_S_Exists,
		ND_3IP_1_CLK => open,
		ND_3IP_1_CTRL => signal_ed_1_in_FSL_S_Control,
		ND_3OP_1_Wr => signal_ed_2_out_FSL_M_Write,
		ND_3OP_1_Dout(31 downto 0) => signal_ed_2_out_FSL_M_Data(0 to 31),
		ND_3OP_1_Full => signal_ed_2_out_FSL_M_Full,
		ND_3OP_1_CLK => open,
		ND_3OP_1_CTRL => signal_ed_2_out_FSL_M_Control,
		ND_3OP_1_d1_Wr => signal_ed_4_out_FSL_M_Write,
		ND_3OP_1_d1_Dout(31 downto 0) => signal_ed_4_out_FSL_M_Data(0 to 31),
		ND_3OP_1_d1_Full => signal_ed_4_out_FSL_M_Full,
		ND_3OP_1_d1_CLK => open,
		ND_3OP_1_d1_CTRL => signal_ed_4_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_3_PARAM_FIFO_FULL => signal_ND_3_PARAM_FIFO_FULL,
		ND_3_SYNC_NUM => signal_ND_3_SYNC_NUM,
		STOP => signal_hwn_nd_3_STOP,
		ERROR => signal_hwn_nd_3_ERROR,
		BLOCK_RD => signal_hwn_nd_3_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_4_ip : pipeline2rtl_hwn_nd_4_ip_wrapper
	port map (
		ND_4IP_2_Rd => signal_ed_2_in_FSL_S_Read,
		ND_4IP_2_Din(31 downto 0) => signal_ed_2_in_FSL_S_Data(0 to 31),
		ND_4IP_2_Exist => signal_ed_2_in_FSL_S_Exists,
		ND_4IP_2_CLK => open,
		ND_4IP_2_CTRL => signal_ed_2_in_FSL_S_Control,
		ND_4OP_1_Wr => signal_ed_3_out_FSL_M_Write,
		ND_4OP_1_Dout(31 downto 0) => signal_ed_3_out_FSL_M_Data(0 to 31),
		ND_4OP_1_Full => signal_ed_3_out_FSL_M_Full,
		ND_4OP_1_CLK => open,
		ND_4OP_1_CTRL => signal_ed_3_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_4_STOP,
		ERROR => signal_hwn_nd_4_ERROR,
		BLOCK_RD => signal_hwn_nd_4_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_5_ip : pipeline2rtl_hwn_nd_5_ip_wrapper
	port map (
		ND_5IP_3_Rd => signal_ed_3_in_FSL_S_Read,
		ND_5IP_3_Din(31 downto 0) => signal_ed_3_in_FSL_S_Data(0 to 31),
		ND_5IP_3_Exist => signal_ed_3_in_FSL_S_Exists,
		ND_5IP_3_CLK => open,
		ND_5IP_3_CTRL => signal_ed_3_in_FSL_S_Control,
		ND_5OP_1_Wr => signal_ed_5_out_FSL_M_Write,
		ND_5OP_1_Dout(31 downto 0) => signal_ed_5_out_FSL_M_Data(0 to 31),
		ND_5OP_1_Full => signal_ed_5_out_FSL_M_Full,
		ND_5OP_1_CLK => open,
		ND_5OP_1_CTRL => signal_ed_5_out_FSL_M_Control,
		ND_5OP_1_d1_Wr => signal_ed_6_out_FSL_M_Write,
		ND_5OP_1_d1_Dout(31 downto 0) => signal_ed_6_out_FSL_M_Data(0 to 31),
		ND_5OP_1_d1_Full => signal_ed_6_out_FSL_M_Full,
		ND_5OP_1_d1_CLK => open,
		ND_5OP_1_d1_CTRL => signal_ed_6_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_5_PARAM_FIFO_FULL => signal_ND_5_PARAM_FIFO_FULL,
		ND_5_SYNC_NUM => signal_ND_5_SYNC_NUM,
		STOP => signal_hwn_nd_5_STOP,
		ERROR => signal_hwn_nd_5_ERROR,
		BLOCK_RD => signal_hwn_nd_5_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_6_ip : pipeline2rtl_hwn_nd_6_ip_wrapper
	port map (
		ND_6IP_4_Rd => signal_ed_4_in_FSL_S_Read,
		ND_6IP_4_Din(31 downto 0) => signal_ed_4_in_FSL_S_Data(0 to 31),
		ND_6IP_4_Exist => signal_ed_4_in_FSL_S_Exists,
		ND_6IP_4_CLK => open,
		ND_6IP_4_CTRL => signal_ed_4_in_FSL_S_Control,
		ND_6OP_1_Wr => signal_ed_7_out_FSL_M_Write,
		ND_6OP_1_Dout(31 downto 0) => signal_ed_7_out_FSL_M_Data(0 to 31),
		ND_6OP_1_Full => signal_ed_7_out_FSL_M_Full,
		ND_6OP_1_CLK => open,
		ND_6OP_1_CTRL => signal_ed_7_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_6_PARAM_FIFO_FULL => signal_ND_6_PARAM_FIFO_FULL,
		ND_6_SYNC_NUM => signal_ND_6_SYNC_NUM,
		STOP => signal_hwn_nd_6_STOP,
		ERROR => signal_hwn_nd_6_ERROR,
		BLOCK_RD => signal_hwn_nd_6_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_7_ip : pipeline2rtl_hwn_nd_7_ip_wrapper
	port map (
		ND_7IP_5_Rd => signal_ed_5_in_FSL_S_Read,
		ND_7IP_5_Din(31 downto 0) => signal_ed_5_in_FSL_S_Data(0 to 31),
		ND_7IP_5_Exist => signal_ed_5_in_FSL_S_Exists,
		ND_7IP_5_CLK => open,
		ND_7IP_5_CTRL => signal_ed_5_in_FSL_S_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_7_PARAM_FIFO_FULL => signal_ND_7_PARAM_FIFO_FULL,
		ND_7_SYNC_NUM => signal_ND_7_SYNC_NUM,
		STOP => signal_hwn_nd_7_STOP,
		ERROR => signal_hwn_nd_7_ERROR,
		BLOCK_RD => signal_hwn_nd_7_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_8_ip : pipeline2rtl_hwn_nd_8_ip_wrapper
	port map (
		ND_8IP_6_Rd => signal_ed_6_in_FSL_S_Read,
		ND_8IP_6_Din(31 downto 0) => signal_ed_6_in_FSL_S_Data(0 to 31),
		ND_8IP_6_Exist => signal_ed_6_in_FSL_S_Exists,
		ND_8IP_6_CLK => open,
		ND_8IP_6_CTRL => signal_ed_6_in_FSL_S_Control,
		ND_8OP_1_Wr => signal_ed_8_out_FSL_M_Write,
		ND_8OP_1_Dout(31 downto 0) => signal_ed_8_out_FSL_M_Data(0 to 31),
		ND_8OP_1_Full => signal_ed_8_out_FSL_M_Full,
		ND_8OP_1_CLK => open,
		ND_8OP_1_CTRL => signal_ed_8_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_8_PARAM_FIFO_FULL => signal_ND_8_PARAM_FIFO_FULL,
		ND_8_SYNC_NUM => signal_ND_8_SYNC_NUM,
		STOP => signal_hwn_nd_8_STOP,
		ERROR => signal_hwn_nd_8_ERROR,
		BLOCK_RD => signal_hwn_nd_8_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_9_ip : pipeline2rtl_hwn_nd_9_ip_wrapper
	port map (
		ND_9IP_7_Rd => signal_ed_7_in_FSL_S_Read,
		ND_9IP_7_Din(31 downto 0) => signal_ed_7_in_FSL_S_Data(0 to 31),
		ND_9IP_7_Exist => signal_ed_7_in_FSL_S_Exists,
		ND_9IP_7_CLK => open,
		ND_9IP_7_CTRL => signal_ed_7_in_FSL_S_Control,
		ND_9IP_8_Rd => signal_ed_8_in_FSL_S_Read,
		ND_9IP_8_Din(31 downto 0) => signal_ed_8_in_FSL_S_Data(0 to 31),
		ND_9IP_8_Exist => signal_ed_8_in_FSL_S_Exists,
		ND_9IP_8_CLK => open,
		ND_9IP_8_CTRL => signal_ed_8_in_FSL_S_Control,
		ND_9OP_1_ND_9OP_1_d1_Wr => signal_ed_9_10_out_FSL_M_Write,
		ND_9OP_1_ND_9OP_1_d1_Dout(31 downto 0) => signal_ed_9_10_out_FSL_M_Data(0 to 31),
		ND_9OP_1_ND_9OP_1_d1_Full => signal_ed_9_10_out_FSL_M_Full,
		ND_9OP_1_ND_9OP_1_d1_CLK => open,
		ND_9OP_1_ND_9OP_1_d1_CTRL => signal_ed_9_10_out_FSL_M_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		ND_9_PARAM_FIFO_FULL => signal_ND_9_PARAM_FIFO_FULL,
		ND_9_SYNC_NUM => signal_ND_9_SYNC_NUM,
		STOP => signal_hwn_nd_9_STOP,
		ERROR => signal_hwn_nd_9_ERROR,
		BLOCK_RD => signal_hwn_nd_9_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_hwn_nd_10_ip : pipeline2rtl_hwn_nd_10_ip_wrapper
	port map (
		ND_10IP_9_ND_10IP_10_Rd => signal_ed_9_10_in_FSL_S_Read,
		ND_10IP_9_ND_10IP_10_Din(31 downto 0) => signal_ed_9_10_in_FSL_S_Data(0 to 31),
		ND_10IP_9_ND_10IP_10_Exist => signal_ed_9_10_in_FSL_S_Exists,
		ND_10IP_9_ND_10IP_10_CLK => open,
		ND_10IP_9_ND_10IP_10_CTRL => signal_ed_9_10_in_FSL_S_Control,
		data_out_Wr => I_data_out_Write,
		data_out_Dout => I_data_out_Data(31 downto 0),
		data_out_Full => I_data_out_Full,
		data_out_CLK => open,
		data_out_CTRL => I_data_out_Control,
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		STOP => signal_hwn_nd_10_STOP,
		ERROR => signal_hwn_nd_10_ERROR,
		BLOCK_RD => signal_hwn_nd_10_BLOCK_RD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_control_if_ip : pipeline2rtl_control_if_ip_wrapper
	port map (
		PARAM_DT => signal_PARAM_DT,
		PARAM_LD => signal_PARAM_LD,
		SYNC_NUM => signal_SYNC_NUM,
		PARAM_FIFO_FULL => signal_PARAM_FIFO_FULL,
		PARAMETERS_IN => signal_PARAMETERS,
		PARAMETERS_IN_LD => signal_PARAMETERS_LD,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_param_ext_ip : pipeline2rtl_param_ext_ip_wrapper
	port map (
		PARAM_SPLIT_X => signal_PARAMETER_SPLIT_X_WRITE,
		COMMIT => signal_COMMIT,
		PARAMETERS => signal_PARAMETERS,
		PARAMETERS_LD => signal_PARAMETERS_LD,
		pci_clk => signal_param_ext_pci_clk,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_register_rf_ip : pipeline2rtl_register_rf_ip_wrapper
	port map (
		address => signal_address,
		read_data => signal_read_data,
		read_en => signal_read_en,
		write_en => signal_write_en,
		write_data => signal_write_data,
		pci_clk => signal_register_rf_pci_clk,
		split_x_rf_read_data => signal_PARAMETER_SPLIT_X_READ,
		split_x_rf_write_en => signal_PARAMETER_SPLIT_X_WRITE_ENABLE,
		split_x_rf_write_data => signal_PARAMETER_SPLIT_X_WRITE,
		commit_rf_read_data => signal_PARAMETER_COMMIT_READ,
		commit_rf_write_en => signal_COMMIT,
		RST => sys_rst_s,
		CLK => sys_clk_s
	);

  pipeline2rtl_ed_1 : pipeline2rtl_ed_1_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_1_out_FSL_M_Data,
		FSL_M_Control => signal_ed_1_out_FSL_M_Control,
		FSL_M_Write => signal_ed_1_out_FSL_M_Write,
		FSL_M_Full => signal_ed_1_out_FSL_M_Full,
		FSL_S_Data => signal_ed_1_in_FSL_S_Data,
		FSL_S_Control => signal_ed_1_in_FSL_S_Control,
		FSL_S_Read => signal_ed_1_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_1_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_1_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_2 : pipeline2rtl_ed_2_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_2_out_FSL_M_Data,
		FSL_M_Control => signal_ed_2_out_FSL_M_Control,
		FSL_M_Write => signal_ed_2_out_FSL_M_Write,
		FSL_M_Full => signal_ed_2_out_FSL_M_Full,
		FSL_S_Data => signal_ed_2_in_FSL_S_Data,
		FSL_S_Control => signal_ed_2_in_FSL_S_Control,
		FSL_S_Read => signal_ed_2_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_2_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_2_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_3 : pipeline2rtl_ed_3_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_3_out_FSL_M_Data,
		FSL_M_Control => signal_ed_3_out_FSL_M_Control,
		FSL_M_Write => signal_ed_3_out_FSL_M_Write,
		FSL_M_Full => signal_ed_3_out_FSL_M_Full,
		FSL_S_Data => signal_ed_3_in_FSL_S_Data,
		FSL_S_Control => signal_ed_3_in_FSL_S_Control,
		FSL_S_Read => signal_ed_3_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_3_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_3_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_4 : pipeline2rtl_ed_4_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_4_out_FSL_M_Data,
		FSL_M_Control => signal_ed_4_out_FSL_M_Control,
		FSL_M_Write => signal_ed_4_out_FSL_M_Write,
		FSL_M_Full => signal_ed_4_out_FSL_M_Full,
		FSL_S_Data => signal_ed_4_in_FSL_S_Data,
		FSL_S_Control => signal_ed_4_in_FSL_S_Control,
		FSL_S_Read => signal_ed_4_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_4_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_4_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_5 : pipeline2rtl_ed_5_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_5_out_FSL_M_Data,
		FSL_M_Control => signal_ed_5_out_FSL_M_Control,
		FSL_M_Write => signal_ed_5_out_FSL_M_Write,
		FSL_M_Full => signal_ed_5_out_FSL_M_Full,
		FSL_S_Data => signal_ed_5_in_FSL_S_Data,
		FSL_S_Control => signal_ed_5_in_FSL_S_Control,
		FSL_S_Read => signal_ed_5_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_5_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_5_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_6 : pipeline2rtl_ed_6_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_6_out_FSL_M_Data,
		FSL_M_Control => signal_ed_6_out_FSL_M_Control,
		FSL_M_Write => signal_ed_6_out_FSL_M_Write,
		FSL_M_Full => signal_ed_6_out_FSL_M_Full,
		FSL_S_Data => signal_ed_6_in_FSL_S_Data,
		FSL_S_Control => signal_ed_6_in_FSL_S_Control,
		FSL_S_Read => signal_ed_6_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_6_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_6_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_7 : pipeline2rtl_ed_7_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_7_out_FSL_M_Data,
		FSL_M_Control => signal_ed_7_out_FSL_M_Control,
		FSL_M_Write => signal_ed_7_out_FSL_M_Write,
		FSL_M_Full => signal_ed_7_out_FSL_M_Full,
		FSL_S_Data => signal_ed_7_in_FSL_S_Data,
		FSL_S_Control => signal_ed_7_in_FSL_S_Control,
		FSL_S_Read => signal_ed_7_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_7_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_7_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_8 : pipeline2rtl_ed_8_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_8_out_FSL_M_Data,
		FSL_M_Control => signal_ed_8_out_FSL_M_Control,
		FSL_M_Write => signal_ed_8_out_FSL_M_Write,
		FSL_M_Full => signal_ed_8_out_FSL_M_Full,
		FSL_S_Data => signal_ed_8_in_FSL_S_Data,
		FSL_S_Control => signal_ed_8_in_FSL_S_Control,
		FSL_S_Read => signal_ed_8_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_8_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_8_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

  pipeline2rtl_ed_9_10 : pipeline2rtl_ed_9_10_ip_wrapper
	port map (
		FSL_Clk => sys_clk_s,
		SYS_Rst => sys_rst_s,
		FSL_Rst => open,
		FSL_M_Clk => net_gnd0,
		FSL_M_Data => signal_ed_9_10_out_FSL_M_Data,
		FSL_M_Control => signal_ed_9_10_out_FSL_M_Control,
		FSL_M_Write => signal_ed_9_10_out_FSL_M_Write,
		FSL_M_Full => signal_ed_9_10_out_FSL_M_Full,
		FSL_S_Data => signal_ed_9_10_in_FSL_S_Data,
		FSL_S_Control => signal_ed_9_10_in_FSL_S_Control,
		FSL_S_Read => signal_ed_9_10_in_FSL_S_Read,
		FSL_S_Exists => signal_ed_9_10_in_FSL_S_Exists,
		FSL_S_Clk => net_gnd0,
		FSL_FULL => signal_ed_9_10_FIFO_FULL,
		FSL_Has_Data => open,
		FSL_Control_IRQ => open
    );

end architecture STRUCTURE;
