-------------------------------------------------------------------------------
-- pipeline2rtl_hwn_nd_4.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity pipeline2rtl_hwn_nd_4 is
   generic (
      STIM_DIR   : string  := "bla";
      RESET_HIGH : NATURAL := 1;
      PAR_WIDTH  : NATURAL := 16;
      QUANT      : NATURAL := 32;
      WRAP       : BOOLEAN := true
    );
  port (

    ND_4IP_2_Rd     : out std_logic;
    ND_4IP_2_Din    : in  std_logic_vector(31 downto 0);
    ND_4IP_2_Exist  : in  std_logic;
    ND_4IP_2_CLK    : out std_logic;
    ND_4IP_2_CTRL   : in  std_logic;

    ND_4OP_1_Wr     : out std_logic;
    ND_4OP_1_Dout   : out std_logic_vector(31 downto 0);
    ND_4OP_1_Full   : in  std_logic;
    ND_4OP_1_CLK    : out std_logic;
    ND_4OP_1_CTRL   : out std_logic;

    PARAM_DT    : in  std_logic_vector(20 downto 0);
    PARAM_LD    : in  std_logic;
    STOP   : out std_logic;
    ERROR   : out std_logic;
    BLOCK_RD   : out std_logic;
    CLK : in std_logic;
    RST : in std_logic

  );
end pipeline2rtl_hwn_nd_4;

architecture RTL of pipeline2rtl_hwn_nd_4 is

  component invert2rtl_invert is
    port (

      data_in_Data    : in  std_logic_vector(31 downto 0);
      data_in_Control : in  std_logic;
      data_in_Read    : out std_logic;
      data_in_Exists  : in  std_logic;

      data_out_Data    : out std_logic_vector(31 downto 0);
      data_out_Control : out std_logic;
      data_out_Write   : out std_logic;
      data_out_Full    : in  std_logic;

      TEST_STOP    : out std_logic_vector(2 downto 0);
      TEST_ERROR    : out std_logic_vector(2 downto 0);
      TEST_FIFO_FULL    : out std_logic_vector(1 downto 0);
      TEST_BLOCK_RD    : out std_logic_vector(2 downto 0);

      KPN_RST : in std_logic;
      KPN_CLK : in std_logic
    );
  end component;

begin


  pipeline2rtl_design_hwn_nd_4_ip : invert2rtl_invert
	port map (
		data_in_Read    => ND_4IP_2_Rd,
		data_in_Data    => ND_4IP_2_Din,
		data_in_Exists  => ND_4IP_2_Exist,
		data_in_Control => ND_4IP_2_CTRL,

		data_out_Write   => ND_4OP_1_Wr,
		data_out_Data    => ND_4OP_1_Dout,
		data_out_Full    => ND_4OP_1_Full,
		data_out_Control => ND_4OP_1_CTRL,

		KPN_RST => RST,
		KPN_CLK => CLK
	);

end RTL;
