-- System TestBench; automatically generated by KpnMapper
-- Use this file to test the system generated by XPS
-- The interface of the tested System includes only the FIFO interfaces
-- declared as external interfaces and not the FIFO interfaces connected
-- to platform FIFOs
-- To generate a System with complete interface select the *noboard* platform option
-- 
-- =====================================================================================
-- To use this testbench file you have to:
--   1. Set propper Time-Out interval (constant TIMEOUT)
--   2. If you read stimuli from files, provide a path to the directory that contains the stimuli files (constant STIM_DIR)
--   3. For each input select whether stimuli is read from a file (default) or from a table (see processes *_STIM_DRV)
--   4. For each output select whether stimuli is read from a file (default) or from a table (see processes *_STIM_CMP)
--   5. For each stimuli that is read from a table fill the stimuli data in the table (constant *_STIM)
-- =====================================================================================
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library std;
use std.textio.all;

entity system_ext_TB is
end system_ext_TB;

architecture RTL of system_ext_TB is

    constant CLK_PERIOD   : TIME    := 10 ns;            -- Period of the system clock
    constant RESET_LENGTH : natural := 5;                -- Reset duration [clock cycles]
    constant STIM_DELAY   : natural := RESET_LENGTH + 5; -- When stimuli supply starts [clock cycles]
    --
    -- Set Time-0ut interval sufficienly long for your application to complete
    constant TIMEOUT : natural := 600000;    -- Time-Out [clock cycles]
    --
    constant STIM_DIR        : string := "";   -- Provide here the path to your stimuli files directory
    -- Input stimuli files: to provide data streams to input FIFOs
    constant STIM_FILE_data_in    : string := "STIM_FILE_data_in.txt";
    constant STIM_FILE_data_out    : string := "STIM_FILE_data_out.txt";

    signal   ENDSIM       : boolean := false;    -- Simulation has finished
    signal   ENDSTIM_IN   : boolean := false;    -- All input stimuli has been sent
    signal   ENDSTIM_OUT  : boolean := false;    -- All expected data has been received
    signal   ENDTIMEOUT   : boolean := false;    -- Simulation Time-Out has occured
    signal   timeout_cntr : natural;
    signal   ERROR_SYS    : boolean := false;    -- Error: Some of te system nodes indicated error
    signal   ERROR_OUT    : boolean := false;    -- Error: Detected output data differs from the expected output data
    signal   FIRST_ERROR  : time;                -- The time when the first error occured
    signal   ENDSTOP      : boolean := true;    -- All system nodes have flagged 'Stop'

    -- 
    -- Component Under Test
    component pipeline is
    port (
        -- FIFO_In Interface: data_in
        data_in_Data    : in  std_logic_vector(31 downto 0);  
        data_in_Control : in  std_logic;  
        data_in_Read    : out std_logic;  
        data_in_Exists  : in  std_logic;  

        -- FIFO_Out Interface: data_out
        data_out_Data    : out std_logic_vector(31 downto 0);  
        data_out_Control : out std_logic;  
        data_out_Write   : out std_logic;  
        data_out_Full    : in  std_logic;  

        TEST_STOP : out std_logic_vector(9 downto 0);  
        TEST_ERROR : out std_logic_vector(9 downto 0);  
        TEST_FIFO_FULL : out std_logic_vector(8 downto 0);  
        TEST_BLOCK_RD : out std_logic_vector(9 downto 0);  
        address : in  std_logic_vector(18 downto 0);  
        read_data : out std_logic_vector(31 downto 0);  
        read_en : in  std_logic;  
        write_en : in  std_logic;  
        write_data : in  std_logic_vector(31 downto 0);  
        pci_clk : in  std_logic;  
        --
        KPN_CLK              : in  std_logic;
        KPN_RST              : in  std_logic
    );
    end component;
    -- 
    signal RST      : STD_LOGIC := '0';
    signal CLK      : STD_LOGIC := '0';
    -- 
    type FIFO_SRC_REC is record
        Data        : integer;
        Control     : std_logic;
        Read        : std_logic;
        Exists      : std_logic;
        -- 
        Count       : natural;
        Done        : boolean;
    end record;
    -- 
    type FIFO_SNK_REC is record
        Data        : integer;
        Control     : std_logic;
        Write       : std_logic;
        Full        : std_logic;
        -- 
        Count       : natural;
        Done        : boolean;
        Error       : boolean;
        First_error : time;
    end record;
    -- 
    signal data_in    : FIFO_SRC_REC;
    signal data_out    : FIFO_SNK_REC;
    -- 
    signal data_in_Data           : std_logic_vector(31 downto 0);
    signal data_out_Data           : std_logic_vector(31 downto 0);
    signal TEST_STOP           : std_logic_vector(9 downto 0);
    signal TEST_ERROR           : std_logic_vector(9 downto 0);
    signal TEST_FIFO_FULL           : std_logic_vector(8 downto 0);
    signal TEST_BLOCK_RD           : std_logic_vector(9 downto 0);
    signal address           : std_logic_vector(18 downto 0);
    signal read_data           : std_logic_vector(31 downto 0);
    signal read_en           : std_logic;
    signal write_en           : std_logic;
    signal write_data           : std_logic_vector(31 downto 0);
    signal pci_clk           : std_logic;
    --
    -- record keeping values of input and output stimuli
    type STIM_REC is record
        Data    : integer;
        Control : std_logic;
    end record;
    --
    -- Function that reads a STIM_REC from a (stimuli) file
    impure function FREAD_STIM(file F : TEXT) return STIM_REC is
        variable VECTOR    : STIM_REC;
        variable IN_LINE   : LINE;
    begin
        readline(F ,IN_LINE);
        read(IN_LINE, VECTOR.Data);
        deallocate(IN_LINE);
        VECTOR.Control := '0';    -- Control bit is not used at the moment
        return VECTOR;
    end;
    --
    -- table of records
    type STIM_ARRAY is array(positive range <>) of STIM_REC;
    --
    -- Stimuli can be read either from a file or from the constant tables below
    -- If you will use constant tables, uncomment below those you need
--    constant data_in_STIM : STIM_ARRAY := (
--    -- Provide your stimuli here
--    -- ( Data, Control),
--    -- e.g. (  0, '0'),
--    -- e.g. (  0, '0') 
--    );
    --
--    constant data_out_STIM : STIM_ARRAY := (
--    -- Provide your stimuli here
--    -- ( Data, Control),
--    -- e.g. (  0, '0'),
--    -- e.g. (  0, '0') 
--    );
    --
begin
    --
    -- =============================================
    -- = System Under Test 
    -- =============================================
    SUT : pipeline port map(
        --
        data_in_Data    => data_in_Data    ,
        data_in_Control => data_in.Control ,
        data_in_Read    => data_in.Read    ,
        data_in_Exists  => data_in.Exists  ,
        --
        data_out_Data    => data_out_Data    ,
        data_out_Control => data_out.Control ,
        data_out_Write   => data_out.Write   ,
        data_out_Full    => data_out.Full    ,
        TEST_STOP    => TEST_STOP    ,
        TEST_ERROR    => TEST_ERROR    ,
        TEST_FIFO_FULL    => TEST_FIFO_FULL    ,
        TEST_BLOCK_RD    => TEST_BLOCK_RD    ,
        address      => address    ,
        read_data    => read_data    ,
        read_en      => read_en    ,
        write_en      => write_en    ,
        write_data      => write_data    ,
        pci_clk      => pci_clk    ,
        --
        KPN_CLK              => CLK,
        KPN_RST              => RST   
    );
    --
    pci_clk <= CLK;
    --
    data_in_Data      <= STD_LOGIC_VECTOR(TO_SIGNED(data_in.Data,   data_in_Data'Length));
    data_out.Data      <= TO_INTEGER(SIGNED(data_out_Data));

    -- Adjust these values to changes values in the Register file to change parameters and shmem
reg_file : process    
    variable read_in : std_logic_vector(31 downto 0);
begin	    	
    address <= (others => '0');
    write_en <= '0';
    read_en <= '0';
    address <= STD_LOGIC_VECTOR(TO_UNSIGNED(0,19));
    write_data <= STD_LOGIC_VECTOR(TO_UNSIGNED(0,32));

 wait for 100ns;	
     address <= STD_LOGIC_VECTOR(TO_UNSIGNED(0,19));
     write_data <= STD_LOGIC_VECTOR(TO_UNSIGNED(0,32));
     wait for 1*CLK_PERIOD;
     write_en <= '1';
     wait for 2*CLK_PERIOD;
     write_en <= '0';


--     wait for 1*CLK_PERIOD;
--     --read_en <= '1';
--     wait for 2*CLK_PERIOD;
--     --read_en <= '0';
--     wait for 1*CLK_PERIOD;
--     wait for 300ns;	
    wait;	
end process;

    --
    -- Stimuli Driver for input stream : data_in
    data_in_STIM_DRV : process
        variable VECTOR    : STIM_REC;
        file     STIM_FILE : TEXT open READ_MODE is STIM_DIR&STIM_FILE_data_in;
    begin
        data_in.Exists <= '0';
        data_in.Count  <=  0;
        data_in.Done   <=  false;
        wait for STIM_DELAY*CLK_PERIOD;
        wait until rising_edge(CLK);
----------------------------------------------
--        -- Uncomment if stimuli for data_in is read from a constant tables
--        for i in data_in_STIM'range loop
--            VECTOR:= data_in_STIM(i);
----------------------------------------------
        -- Uncomment if stimuli for data_in is read from a file
        while not( endfile(STIM_FILE)) loop
            VECTOR := FREAD_STIM(STIM_FILE);
--------------------------------------------
            data_in.Data    <= VECTOR.Data;
            data_in.Control <= VECTOR.Control;
            data_in.Exists  <= '1';
            L1: loop
                wait until rising_edge(CLK);
                exit L1 when (data_in.Read = '1');
            end loop L1;
            data_in.Count <= data_in.Count + 1;
        end loop;
        data_in.Exists <= '0';
        data_in.Done <= true;
        wait for 10*CLK_PERIOD;
        wait;
    end process;
    --
    ENDSTIM_IN <= data_in.Done;
    --
    -- Stimuli Comparator for output stream data_out
    data_out_STIM_CMP : process
        variable VECTOR     : STIM_REC;
        file     STIM_FILE  : TEXT open READ_MODE is STIM_DIR&STIM_FILE_data_out;
    begin
        data_out.Full  <= '1';
        data_out.Count <= 0;
        data_out.Done  <= false;
        data_out.Error <= false;
        wait for STIM_DELAY*CLK_PERIOD;
        wait until rising_edge(CLK);
----------------------------------------------
--        -- Uncomment if stimuli for data_out is read from a constant tables
--        for i in data_out_STIM'range loop
--            VECTOR := data_out_STIM(i);
----------------------------------------------
        -- Uncomment if stimuli for data_out is read from a file
        while not( endfile(STIM_FILE)) loop
            VECTOR := FREAD_STIM(STIM_FILE);
----------------------------------------------
            --
            data_out.Full  <= '0';
            L1: loop
                wait until rising_edge(CLK);
                exit L1 when (data_out.Write = '1');
            end loop L1;
            data_out.Count <= data_out.Count + 1;
            if (data_out.Data /= VECTOR.Data) then
                report "TB_ERROR: Output 'data_out': the detected value " & integer'image(data_out.Data) & " differs from the expected value " & integer'image(VECTOR.Data) & "!!! (@time " & time'image(now) & ")."
                severity WARNING;
                if (not data_out.Error) then
                    data_out.First_error <= now;
                end if;
                data_out.Error <= true;
            end if;
        end loop;
        data_out.Full  <= '1';
        data_out.Done <= true;
        wait for 10*CLK_PERIOD;
        wait;
    end process;
    --
    ENDSTIM_OUT <= data_out.Done;
    ERROR_OUT   <= data_out.Error;
    --
    -- Record the time when the first error occures
    FIRST_ERROR_TIME : process
    begin
        wait until (ERROR_OUT'event and ERROR_OUT=true) or (ERROR_SYS'event and ERROR_SYS=true);
        FIRST_ERROR <= now;
        wait;
    end process;
    --
--=============================================
--= All Nodes stopped ?
--=============================================
	process(test_stop)
		variable s : std_logic;
		variable e : std_logic;
	begin
	s := '1';
	e := '0';
	--
	for i in 0 to TEST_ERROR'Length-1 loop
		s := s and test_stop(i);
		e := e or  test_error(i);
	end loop;
	--
		ENDSTOP   <= (s = '1');
		ERROR_SYS <= (e = '1');
	--
	end process;
    -- Timeout counter
    TO_CTRL : process(CLK)
    begin
        if (rising_edge(CLK)) then
            if (RST = '1') then
                timeout_cntr <= 0;
            else
                if (timeout_cntr = TIMEOUT) then
                    ENDTIMEOUT <= true;
                else
                    timeout_cntr <= timeout_cntr + 1;
                end if;
            end if;
        end if;
    end process;
    --
    -- Simulation control
    ENDSIM <= (ENDSTIM_IN and ENDSTOP and ENDSTIM_OUT) or ENDTIMEOUT;
    --
    PRINT_REPORT : process
        variable ERROR_CODE : natural;
        variable l          : line;
    begin
       -- write(l, "***TB_REPOT: Simulation in progress...");
        writeline(output,l);
        wait until ENDSIM=true;
       -- write(l, "***TB_REPOT: Simulation END.");
        writeline(output,l);
        ERROR_CODE := 0;
        -- 
        if (ENDTIMEOUT) then
            ERROR_CODE := ERROR_CODE + 1;
            write(l, "***TB_REPOT: [TIMEOUT] Simulation terminated by a TIMEOUT after " & integer'image(timeout_cntr) & " clock cycles.");
            writeline(output,l);
        end if;
        -- 
        if (ERROR_OUT or ERROR_SYS) then
            ERROR_CODE := ERROR_CODE + 2;
            write(l, "***TB_REPOT: [ERROR]   Simulation terminated with ERRORS!!! First error occured at time " & time'image(FIRST_ERROR) & ".");
            writeline(output,l);
         --   write(l, "***TB_REPOT: [ERROR]   Flags ERROR = b");
         --   write(l, test_error, RIGHT, test_error'Length);
         --   writeline(output,l);
        end if;
        -- 
        write(l, "***TB_REPOT: [ERROR_CODE=" & integer'image(ERROR_CODE) & "]");
        writeline(output,l);
        -- 
        if (ERROR_CODE = 0) then
            write(l, "***TB_REPOT: [OK] Simulation completed successfully in " & integer'image(timeout_cntr) & " cycles !!!");
            writeline(output,l);
         --   write(l, "None of the processors flagged ERROR. ");
            writeline(output,l);
        end if;
        wait;
    end process;
    --
    -- CLK generator
    CLK_GEN: process
    begin
        if (ENDSIM=false) then
            CLK <= '0';
            wait for CLK_PERIOD/2;
            CLK <= '1';
            wait for CLK_PERIOD/2;
        else
            wait;
        end if;
    end process;
    --
    -- RESET generator
    RST_GEN: process
    begin
        RST <='1';
        wait for RESET_LENGTH*CLK_PERIOD;
        RST <='0';
        wait;
    end process;
    --
end RTL;
